
//**********************************************************************
//
//  MPLCLIENT.H
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  This is the main public include file for MPL client modules (Eg. NSM)
//  It is NOT meant to be used internally by MPL.
//
//  Note: This file includes all relavent MPL include files.
//  Therefore clients need only include this one file to obtain
//  all needed definitions to use the MPL interfaces.
// 
//**********************************************************************

#ifndef _MPLCLIENT_H_
#define _MPLCLIENT_H_

// Include platform customized data type definitions.
#include <oai.h>

// Include common data structures between MPL and clients
#include <mplcommon.h>

// Include Debug
#include <mpldebug.h>

// Include MPL diagnostics APIs
#ifdef MPL_DIAG_MODE
#endif //MPL_DIAG_MODE

// Include all task offload related definitions
#ifdef MPL_TASK_STAT
#include <mpltaskstat.h>
#endif //MPL_TASK_STAT

#ifdef MPL_TASK_VLAN
#include <mpltaskvlan.h>
#endif //MPL_TASK_VLAN

#ifdef MPL_TASK_CHKSUM
#include <mpltaskchksum.h>
#endif //MPL_TASK_CHKSUM

#ifdef MPL_TASK_RECEIVE_FILTER
#include <mpltaskrxfilter.h>
#endif // MPL_TASK_RECEIVE_FILTER

//************************* MPL CORE FUNCTION PROTOTYPES **********************
// MPL basic device operation (initialization, tx/rx, configuration) APIs 
//*****************************************************************************


// MPL Operational Mode
typedef enum {
      MPL_MODE_NORMAL, 
      MPL_MODE_MONITOR,
      MPL_MODE_LOAD_ONLY
} MPL_MODE;

// Transmit Capabilities
typedef struct _MPL_CAPS_TRANSMIT {
      NS_UINT8   priorityQCnt;
      NS_UINT16  fifoSize;
      NS_UINT16  maxDma;
      NS_UINT16  minDrain;
      NS_UINT16  minFill;
      NS_UINT16  maxPacketSize;
      NS_UINT    txFlags; // None Defined Yet
} MPL_CAPS_TRANSMIT;
 
// Receive Capabilities
typedef struct _MPL_CAPS_RECEIVE {
      NS_UINT8   priorityQCnt;
      NS_UINT16  fifoSize;
      NS_UINT16  maxDma;
      NS_UINT16  minDrain;
      NS_UINT16  maxPacketSize;
      NS_UINT    rxFlags; // None Defined Yet     
} MPL_CAPS_RECEIVE;

// Interrupt Capabilities
typedef struct _MPL_CAPS_INTERRUPT {
      NS_UINT16  maxTimerHold;
      NS_UINT16  maxTransmitHold;
      NS_UINT16  maxReceiveHold;
      NS_UINT    intrFlags; // None Defined Yet 
} MPL_CAPS_INTERRUPT;

// Link Capabilities
typedef struct _MPL_CAPS_LINK {
      NS_UINT           linkFlags;
#define  MPL_LINK_PAUSE_RECEIVE      0x00000001 // Can rx pause reqs, but not tx
#define  MPL_LINK_PAUSE_TRANSMIT     0x00000002 // Can tx pause reqs, but not rx

} MPL_CAPS_LINK;

// Wol Capabilities
typedef struct _MPL_CAPS_WOL {
      NS_UINT8   maxPatterns;
      NS_UINT16  maxPatternSize;
      NS_UINT    wolFlags; // None Defined Yet
} MPL_CAPS_WOL;

// Offload Task Capabilities
typedef struct _MPL_CAPS_TASK {

      // Statistics Task
      struct {
         NS_BOOLEAN  supported;
         NS_UINT     statFlags;      
#define MPL_TASK_STAT_ADV  0x00000001

      } stat;

      // Checksum Task
      struct {
         NS_BOOLEAN  supported;
         NS_UINT     chkSumFlags;      
#define MPL_TASK_CHKSUM_GLOBAL     0x00000001
#define MPL_TASK_CHKSUM_PERPACKET  0x00000002
#define MPL_TASK_CHKSUM_REJECT_IP  0x00000004
#define MPL_TASK_CHKSUM_REJECT_UDP 0x00000008
#define MPL_TASK_CHKSUM_REJECT_TCP 0x00000010

      } chkSum;

      // VLAN Task
      struct {
         NS_BOOLEAN  supported;
         NS_UINT16   maxFilterEntries;
         NS_UINT     vlanFlags;      
#define MPL_TASK_VLAN_DETECT_TAGS       0x00000001 //Rx Only - Reports Tag data
#define MPL_TASK_VLAN_REMOVE_TAGS       0x00000002
#define MPL_TASK_VLAN_DISCARD_TAGGED    0x00000004
#define MPL_TASK_VLAN_DISCARD_UNTAGGED  0x00000008
#define MPL_TASK_VLAN_PERPACKET         0x00000010 //Tx Only
#define MPL_TASK_VLAN_FILTER     0x00000020 // Only when maxFilterEntries > 0x0

      } vlan;

      // Receive Filter Task
      struct {
         NS_BOOLEAN  supported;
         NS_UINT16   maxFilterEntries;
         NS_UINT     rxFilterFlags;
#define MPL_TASK_FILTER_MCAST    0x00000001 // Support Multicast Listen List
#define MPL_TASK_FILTER_UCAST    0x00000002 // Support Unicast Listen List
#define MPL_TASK_FILTER_PERFECT  0x00000004 // Support Perfect Filtering

      } rxFilter;

} MPL_CAPS_TASK;

// Device Capabilities Structure
typedef struct _MPL_CAPS {
      MPL_CAPS_TRANSMIT   txCaps;
      MPL_CAPS_RECEIVE    rxCaps;
      MPL_CAPS_INTERRUPT  intrCaps;
      MPL_CAPS_LINK       linkCaps;
      MPL_CAPS_WOL        wolCaps;
      MPL_CAPS_TASK       taskCaps;
} MPL_CAPS;

// Transmit Engine Configuration Structure
typedef struct MPL_TRANSMIT_CFG {
      NS_UINT8  priorityQueueCnt;
#define MPL_TRANSMIT_MAX_PRIORITYQ_CNT 4
      NS_UINT  descCnt[MPL_TRANSMIT_MAX_PRIORITYQ_CNT];
      NS_UINT  fillThreshold;
      NS_UINT  drainThreshold;
      NS_UINT  maxDmaBurst;
      NS_UINT  maxIndications;

      NS_UINT  cfgFlags;
#define MPL_TRANSMIT_CFG_FIFO     0x00000001
#define MPL_TRANSMIT_CFG_QUEUES   0x00000002
#define MPL_TRANSMIT_CFG_AUTO_PAD 0x00000004
#define MPL_TRANSMIT_CFG_QUEUE_RR 0x00000008 // En round-robin algo btn queues

} MPL_TRANSMIT_CFG;

// Receive Engine Configuration Structure
typedef struct MPL_RECEIVE_CFG {
      NS_UINT8  priorityQueueCnt;
#define MPL_RECEIVE_MAX_PRIORITYQ_CNT 1
      NS_UINT  descCnt[MPL_RECEIVE_MAX_PRIORITYQ_CNT];
      NS_UINT  drainThreshold;
      NS_UINT  maxDmaBurst;
      NS_UINT  maxIndications;

      NS_UINT  cfgFlags;
#define MPL_RECEIVE_CFG_FIFO    0x00000001
#define MPL_RECEIVE_CFG_QUEUES  0x00000002
} MPL_RECEIVE_CFG;

// Link Negotiation Mode
typedef enum {
      MPL_LINK_MODE_AUTO,
      MPL_LINK_MODE_FORCED 
} MPL_LINK_NEGOTIATION;

// Link Speed
typedef enum {
      MPL_LINK_SPEED_HUNDREDMBPS,
      MPL_LINK_SPEED_TENMBPS 
} MPL_LINK_SPEED;

// Link Duplex Type
typedef enum {
      MPL_LINK_DUPLEX_FULL,
      MPL_LINK_DUPLEX_HALF
} MPL_LINK_DUPLEX;

// Link Configuration Structure
typedef struct _MPL_LINK_CFG {
      MPL_LINK_NEGOTIATION mode;
      MPL_LINK_SPEED       speed;
      MPL_LINK_DUPLEX      duplex;

#define  MPL_LINK_PAUSE_SYMMETRICAL  \
              (MPL_LINK_PAUSE_TRANSMIT | MPL_LINK_PAUSE_RECEIVE)
#define  MPL_LINK_PAUSE_NONE 0x0

      NS_UINT              pauseType;
      NS_UINT              pauseCtr;
      NS_UINT              pauseRxDALO;
      NS_UINT              pauseRxDAHI;
      NS_UINT              pauseRxSTLO;
      NS_UINT              pauseRxSTHI;
} MPL_LINK_CFG;

// Link Status Structure
typedef enum _MPL_LINK_STATUS {
      MPL_LINK_STATUS_NONE,
      MPL_LINK_STATUS_DOWN,
      MPL_LINK_STATUS_ACTIVE,
      MPL_LINK_STATUS_UP
} MPL_LINK_STATUS;

// Device Ids
typedef enum {
      MPL_DEVICE_ID_UNKNOWN,
      MPL_DEVICE_ID_DP83815,
      MPL_DEVICE_ID_DP83816,
      MPL_DEVICE_ID_DP83818
} MPL_DEVICE_ID;

// MPL Packet Fragment Descriptor
typedef struct _MPL_PKT_FRAG {
      struct _MPL_PKT_FRAG  *pNextFrag;
      NS_UINT                fragSize;
      NS_ADDR                physAddr;
      NS_UINT8              *pAddr;
      NS_VOID               *pPacketHndl;
      NS_VOID               *pMplPrivate;
      NS_VOID               *pNsmPrivate;
} MPL_PKT_FRAG;

// MPL Packet Tx Out-Of-Band data - MPL client should not modify this directly,
//  instead Call MplSetPacketXXX functions
typedef struct _MPL_PKT_TRANSMIT_OOB {
      NS_UINT32   cmdSts;
      NS_UINT32   extCtl; // Extended Control
      NS_UINT8    pQueue;
} MPL_PKT_TRANSMIT_OOB;


// MPL Packet Rx Out-Of-Band data - MPL client should not modify this directly,
//  instead Call MplQueryPacketXXX functions
typedef struct _MPL_PKT_RECEIVE_OOB {
      NS_UINT32   cmdSts;
      NS_UINT32   extSts; // Extended Status
} MPL_PKT_RECEIVE_OOB;


// MPL Packet Descriptor
typedef struct _MPL_PKT {
      struct _MPL_PKT *pNextPacket;
      NS_UINT          fragCount;
      MPL_PKT_FRAG    *pFragHead;

      union {
          MPL_PKT_TRANSMIT_OOB tx;
          MPL_PKT_RECEIVE_OOB  rx;
      } OOB;
#define txOOB OOB.tx
#define rxOOB OOB.rx

      MPL_STATUS       packetStatus;
      NS_UINT          packetSize;
      NS_VOID         *pMplPrivate;
      NS_VOID         *pNsmPrivate;
} MPL_PKT;

// Transmit Done Indication (MPL to NSM)
typedef struct _MPL_TRANSMIT_DONE {
      NS_VOID         *pNsmPrivate;
      MPL_STATUS       packetStatus;
      NS_UINT32        cmdSts;
} MPL_TRANSMIT_DONE;

// Power State
typedef enum _MPL_POWER_STATE {
      MPL_POWER_STATE_LOW,
      MPL_POWER_STATE_HIGH
} MPL_POWER_STATE;

// WoL Pattern Container
typedef struct _MPL_WOL_PATTERN {
      NS_UINT   patternSize;
      NS_UINT8 *pPattern;
      NS_UINT   maskSize;
      NS_UINT8 *pMask;
} MPL_WOL_PATTERN;


//+++++ Device Initialization and Shutdown APIs
//############################################

#if defined(__cplusplus)
extern "C"
{
#endif

//*****************************************************************************
//   MplInitialize 
//      Allocate and initialize a MPL context fot the device.
//
//   Parameters
//      pNsmHandle
//         NSM device handle for the device hardware.  This handle is retained 
//          by MPL and passed in to subsequent calls to the NSM.
//      pBaseAddr
//         The logical base address for the device.
//      diagEnable
//         A NS_TRUE notifies MPL to go into a diagnostic mode where MPL lets 
//          the diag application to control the adapter, MPL now handles only
//          Txd and RXD ring operation. For most production drivers this should
//          be passed as NS_FALSE.
//      pMplHandle
//         MPL handle for the device’s context.  The NSM must retain this handle
//          and pass it for all subsequent MPL calls.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The device was successfully initialized and the MPL context is 
//          returned
//      NS_STATUS_INVALID_PARAM
//         An invalid parameter value was detected
//      NS_STATUS_RESOURCES
//         Unable to initialize a context (lack of system resources)
//      NS_STATUS_HARDWARE_FAILURE
//         Unable to initialize a context (unexpected h/w error)
//         or failed to detect MacPhyter device
//
//*****************************************************************************
MPL_STATUS
   MplInitialize (
      IN NS_VOID       *pNsmHandle,
      IN NS_VOID       *pBaseAddr,
      IN NS_BOOLEAN     diagEnable,
      OUT NS_VOID     **pMplHandle
      );

//*****************************************************************************
//   MplOpen
//      Opens the device for network operation. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize. 
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The device was successfully opened and is ready for network operation
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected or this function is called 
//          prior to MplInitialize
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error
//
//*****************************************************************************
MPL_STATUS
   MplOpen (
      IN NS_VOID       *pMplHandle
      );


//*****************************************************************************
//   MplClose
//      Closes the network device, disabling network transmission/reception and
//       interrupts. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The device was successfully closed
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected or this function is called 
//          prior to MplInitialize or MplOpen
//
//*****************************************************************************
MPL_STATUS
   MplClose (
      IN NS_VOID       *pMplHandle
      );

//*****************************************************************************
//   MplUnload
//      Shuts down the device hardware and frees all allocated resources claimed
//       by MplInitialize. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         MPL was successfully unloaded and all resources released.
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected or this function is called 
//          prior to MplInitialize
//
//*****************************************************************************
MPL_STATUS
   MplUnload (
      IN NS_VOID       *pMplHandle
      );

//*****************************************************************************
//   MplMacReset
//      Resets the network device. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The network device was successfully reset.
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error
//
//*****************************************************************************
MPL_STATUS
   MplMacReset (
      IN NS_VOID       *pMplHandle
      );


//*****************************************************************************
//   MplGetCaps
//      Returns the capabilities of the device and the MPL implementation. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pMplCaps
//         Pointer to a caller allocated MPL_CAPS structure in which the 
//          capabilities of the device and MPL is returned
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The capabilities were successfully returned
//
//*****************************************************************************
MPL_STATUS
   MplGetCaps (
      IN NS_VOID      *pMplHandle,
      OUT MPL_CAPS    *pMplCaps
      );

//*****************************************************************************
//   MplCfgMTU
//      Configures the maximum packet size supported by MPL
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      mtu
//         The maximum packet size queued for transmission or reception
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The requested MTU was successfully configured on the device.
//      NS_STATUS_INVALID_PARM
//         The size is greater than the maximum supported on the device.
//
//*****************************************************************************
MPL_STATUS
   MplCfgMTU (
      IN NS_VOID  *pMplHandle,
      IN NS_UINT   mtu
      );

//+++++ Transmit Engine APIs
//##########################

//*****************************************************************************
//   MplTransmitCfg
//      Configures the transmit engine on the device. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pCfgTransmit
//         Pointer to a caller allocated MPL_TRANSMIT_CFG structure with 
//          configuration values for the transmit engine
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The configurations were successfully applied
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected
//      NS_STATUS_RESOURCES
//         Failed to allocate required resources
//
//*****************************************************************************
MPL_STATUS
   MplTransmitCfg (
      IN NS_VOID            *pMplHandle,
      IN MPL_TRANSMIT_CFG   *pCfgTransmit
      );


//*****************************************************************************
//   MplTransmitGetFreeDesc
//      Returns the total free transmit descriptors currently available. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pQueue
//         Queue (1,2...max tx queues) from which the free count is desired
//      pTxdCnt
//         Pointer to a caller allocated fields in which the count of free 
//          transmit descriptors is returned
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The count was successfully returned.
//      NS_STATUS_INVALID_PARM
//         The priority queue count is invalid
//
//*****************************************************************************
MPL_STATUS
   MplTransmitGetFreeDesc (
      IN NS_VOID    *pMplHandle,
      IN NS_UINT8    pQueue,
      OUT NS_UINT   *pTxdCnt
      );

//*****************************************************************************
//   MplTransmit
//      Queues up one or more packets for transmission by the device.
//      NOTE: This function is non-reentrant with respect to itself, 
//        MplTransmitReset and MplTransmitDone
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pktCount
//         The count of packets being queued for transmission
//      pPackets
//         Pointer to a list of MPL_PKT structures due for transmission
//
//   Return Value
//      NS_STATUS_SUCCESS
//         All packets were successfully queued up for transmission
//      NS_STATUS_FAILURE
//         One or more of the packets where unacceptable for transmission
//           Per packet status (pPacket->packetStatus) is one of
//             NS_STATUS_INVALID_PARM
//               The priority queue count is invalid
//             NS_STATUS_RESOURCES
//               Failed to allocate the required resources (Txds)
//
//*****************************************************************************
MPL_STATUS
   MplTransmit (
      IN NS_VOID      *pMplHandle,
      IN NS_UINT       pktCount,
      IN MPL_PKT      *pPackets
      );


//*****************************************************************************
//   MplTrasmitDone
//      Processes transmit engine related events.
//      NOTE: This function is non-reentrant with respect to itself, 
//        MplTransmitReset and MplTransmit
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      maxEvents
//         The maximum transmit engine related events (e.g. transmit complete) 
//          that MPL should process before returning back to the caller.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The transmit event was successfully handled.
//      NS_STATUS_ABORTED
//         The processing was aborted since the maximum event count specified
//          was met.
//
//*****************************************************************************
MPL_STATUS
   MplTransmitDone (
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    maxEvents
      );

//*****************************************************************************
//   MplTransmitReset
//      Resets the transmit engine on the device and associated resources. 
//      NOTE: This function is non-reentrant with respect to itself, 
//        MplTransmit and MplTransmitDone
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The transmit engine was successfully reset.
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error - No reset done notification from Hw
//
//*****************************************************************************
MPL_STATUS
   MplTransmitReset (
      IN NS_VOID       *pMplHandle
      );

//+++++ Receive Engine APIs
//##########################

//*****************************************************************************
//   MplReceiveCfg
//      Configures the receive engine on the device. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pCfgReceive
//         Pointer to a caller allocated MPL_RECEIVE_CFG structure with 
//          configuration values for the receive engine
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The configurations were successfully applied
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected
//
//*****************************************************************************
MPL_STATUS
   MplReceiveCfg (
      IN NS_VOID            *pMplHandle,
      IN MPL_RECEIVE_CFG    *pCfgReceive
      );

//*****************************************************************************
//   MplReceiveSetFilter
//       MplReceiveSetFilter enables receive filters on the device. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      filterFlag
//         Bit-map of receiver filter types that need to be enabled on the 
//          device
//       Note: This function overwrites all previous filter settings and 
//       sets the device's behavior to that of the filterFlag passed to it.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The filter configurations were successfully applied
//      NS_STATUS_NOT_SUPPORTED
//         The device did not support one or more filters
//
//*****************************************************************************
MPL_STATUS
   MplReceiveSetFilter (
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    filterFlag
      );

// MPL Receive Filter Bits
#define MPL_RECEIVE_FILTER_ACCEPTALL_BROADCAST 0x00000001
#define MPL_RECEIVE_FILTER_ACCEPTALL_MCAST     0x00000002
#define MPL_RECEIVE_FILTER_ACCEPTALL_UNICAST   0x00000004
#define MPL_RECEIVE_FILTER_ACCEPTALL_ARP       0x00000008
#define MPL_RECEIVE_FILTER_PROMISCUOUS_MODE    0x00000010
#define MPL_RECEIVE_FILTER_DIRECTED_UNICAST    0x00000020
#define MPL_RECEIVE_FILTER_DIRECTED_MCAST      0x00000040
#define MPL_RECEIVE_FILTER_ERROREDPKTS         0x00000080


//*****************************************************************************
//   MplReceive
//      Handles the packet reception event and moves the received packet from
//       the device to the host.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      maxEvents
//         The maximum receive engine related events (e.g. receive complete)
//          that MPL should process before returning back to the caller.
//      pBufFragConsumed
//         A caller supplied pointer variable in which this function returns the
//          number of buffer fragments that were consumed while processing this
//          receive pass.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The receive events were successfully processed.
//      NS_STATUS_ABORTED
//         The processing was aborted since the maximum event count specified 
//          was met.
//
//*****************************************************************************
MPL_STATUS
   MplReceive (
      IN NS_VOID     *pMplHandle,
      IN NS_UINT      maxEvents,
      OUT NS_UINT    *pBufFragConsumed
      );

//*****************************************************************************
//   MplReceiveReplenish
//      Restock the MPL receive engine with fresh buffers for packet reception.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//      bufFragCount
//         Number of buffer fragments being replenished.
//      pFragList
//         Pointer to a caller supplied variable of type MPL_PKT_FRAG pointing 
//          to the head node of the buffer fragment list being replenished.
//          If all the buffer fragments were not consumed by MPL then the 
//          pointer to the first unused fragment is returned back to NSM.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The buffers were successfully replenished.
//
//*****************************************************************************
MPL_STATUS
   MplReceiveReplenish (
      IN NS_VOID           *pMplHandle,
      IN NS_UINT            bufFragCount,
      IN OUT MPL_PKT_FRAG **pFragList
      );


//*****************************************************************************
//   MplQueryPacketLength
//      Returns the size (in bytes) of the data frame within a MPL_PKT structure
//
//   Parameters
//      pPacket
//         Pointer to the MPL_PKT structure who data frame length is being 
//          queried.
//
//   Return Value
//      The total size of the data frame in bytes (could be 0x0 also).
//
//*****************************************************************************
NS_UINT
   MplQueryPacketLength (
      IN MPL_PKT   *pPacket
      );


//*****************************************************************************
//   MplReceiveReset
//      Resets the receive engine on the device and associated software 
//       resources. 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The receive engine was successfully reset.
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error - No reset done notification from Hw
//
//*****************************************************************************
MPL_STATUS
   MplReceiveReset (
      IN NS_VOID       *pMplHandle
      );

//*****************************************************************************
//   MplReceiveGetEmptyDesc
//      Returns the count of the Rxds that are empty i.e. no attached buffers
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pQueue
//         Queue (1,2...max tx queues) from which the empty count is desired
//      pRxdCnt
//         Pointer to a caller allocated fields in which the count of empty
//          receive descriptors is returned
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The count was successfully returned.
//      NS_STATUS_INVALID_PARM
//         The priority queue count is invalid
//
//*****************************************************************************
MPL_STATUS
   MplReceiveGetEmptyDesc (
      IN NS_VOID    *pMplHandle,
      IN NS_UINT8    pQueue,
      OUT NS_UINT   *pRxdCnt
      );


//+++++ Interrupt Module APIs
//###########################

//*****************************************************************************
//   MplInterruptCfg
//      Configure the interrupt hold-off delay (for interrupt coalescing).
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      delayMSec
//         Interrupt hold off delay in microseconds
//      transmitHold
//         Interrupts hold off based on number of Tx pkts sent
//      receiveHold
//         Interrupts hold off based on number of Rx pkts sent
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Interrupts were successfully configured
//
//*****************************************************************************
MPL_STATUS
   MplInterruptCfg (
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    delayMSec,
      IN NS_UINT    transmitHold,
      IN NS_UINT    receiveHold
      );

//*****************************************************************************
//   MplInterruptEnable
//      Enable the interrupt line on the network device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Interrupts were successfully enabled
//
//*****************************************************************************
MPL_STATUS
   MplInterruptEnable (
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplInterruptDisable
//      Disable the interrupt line on the network device.
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//   Return Value
//      NS_STATUS_SUCCESS
//         Interrupts were successfully disabled
//
//*****************************************************************************
MPL_STATUS
   MplInterruptDisable (
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplInterruptCheck
//      Checks for and clears pending interrupt conditions on the device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//      pIrqsPresent
//         A caller supplied variable in which the current interrupting 
//          conditions are noted.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The current interrupting conditions were successfully read from the 
//          device and pending events do exist.
//      NS_STATUS_FAILURE
//         Interrupts are currently disabled, so no interrupt conditions exist.
//          or the mask bits and status bits don't match
//
//*****************************************************************************
MPL_STATUS
   MplInterruptCheck (
      IN NS_VOID    *pMplHandle
      );

//*****************************************************************************
//   MplInterruptCheckTransmit
//      Checks for the presence of transmit engine related interrupt events.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Pending transmit related events exist on the device
//      NS_STATUS_FAILURE
//         No transmit related events detected.
//
//*****************************************************************************
MPL_STATUS
   MplInterruptCheckTransmit (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplInterruptCheckReceive
//      Checks for the presence of receive engine related interrupt events.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Pending receive related events exist on the device
//      NS_STATUS_FAILURE
//         No receive related events detected.
//
//*****************************************************************************
MPL_STATUS
   MplInterruptCheckReceive (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplInterruptCheckInternal
//      Checks for the presence of MAC/PHY internal (non data transfer related)
//       interrupt events.
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//   Return Value
//      NS_STATUS_SUCCESS
//         Pending internal events exist on the device
//      NS_STATUS_FAILURE
//         No internal events detected.
//
//*****************************************************************************
MPL_STATUS
   MplInterruptCheckInternal (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplInterruptDoneInternal
//      Processes device’s internal (non data-type) interrupts.
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      maxEvents
//         The maximum internal events (e.g. link change) that MPL should 
//          process before returning back to the caller.
//   Return Value
//      NS_STATUS_SUCCESS
//         The internal event was successfully handled.
//      NS_STATUS_ABORTED
//         The processing was aborted since the maximum event count specified 
//          was met.
//
//*****************************************************************************
MPL_STATUS
   MplInterruptDoneInternal (
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    maxEvents
      );


//+++++ Link Management APIs
//###########################


//*****************************************************************************
//   MplLinkCfg
//      Configures the network media speed (10/100), duplex (half/full), 
//       negotiation (Auto/Forced) and network PAUSE parameters.
//      When configuring auto mode, the speed and duplex setting passed should
//       reflect the best case scenario i.e most desired speed and duplex
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pLinkCfg
//         Points to a MPL_LINK_CFG structure with the desired link 
//          configuration.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Link configured and the link is UP.
//      NS_STATUS_ASYNCH_COMPLETION
//         Link is currently being configured or being negotiated.
//      NS_STATUS_INVALID_PARM
//         An invalid parameter was detected in the configuration structure.
//      NS_STATUS_HARDWARE_FAILURE
//         An expected failure occured while setting the link config
//
//*****************************************************************************
MPL_STATUS
   MplLinkCfg (
      IN NS_VOID        *pMplHandle,
      IN MPL_LINK_CFG   *pLinkCfg
      );


//*****************************************************************************
//   MplLinkGetCfg
//      Returns the current link configuration on the device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pLinkCfg
//         Pointer to a caller allocated MPL_LINK_CFG structure in which the 
//          current link configuration is returned.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link configuration was successfully returned.
//      NS_STATUS_FAILURE
//         The link is not configured (yet).
//
//*****************************************************************************
MPL_STATUS
   MplLinkGetCfg (
      IN NS_VOID        *pMplHandle,
      OUT MPL_LINK_CFG  *pLinkCfg
      );

//*****************************************************************************
//   MplLinkGetStatus
//      Returns the link status reported on the device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      MPL_LINK_STATUS_NONE
//         An unexpected hardware error occurred while processing this request
//          and the link state was not retrieved.
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up
//
//*****************************************************************************
MPL_LINK_STATUS
   MplLinkGetStatus (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplLinkProcessChange
//      Handles the link change event
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      MPL_LINK_STATUS_NONE
//         An unexpected hardware error occurred while processing this request
//          and the link state was not retrieved.
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up.
//
//*****************************************************************************
MPL_LINK_STATUS
   MplLinkProcessChange (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplLinkUpComplete
//      Completes the configuration of the device following the 
//       auto-negotiation process.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link configuration is complete.
//      NS_STATUS_FAILURE
//         The link state is not UP.
//
//*****************************************************************************
MPL_STATUS
   MplLinkUpComplete (
      IN NS_VOID      *pMplHandle
      );

//*****************************************************************************
//   MplLinkInterrupt
//      Processes an incoming link related interrupt
//      At this time this is an MPL internal function (called from
//       MplIntteruptDoneInternal) 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The interrupt was successfully fielded.
//
//*****************************************************************************
MPL_STATUS
   MplLinkInterrupt(
      IN NS_VOID   *pMplHandle
      );

//+++++ PHY Management APIs
//###################################

//*****************************************************************************
//   MplPhyDetect
//      Detect the presence of a MacPhyter device on the system.
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         A MacPhyter device was successfully probed
//      NS_STATUS_HARDWARE_FAILURE
//         Could not find a MacPhyter device
//
//*****************************************************************************
MPL_STATUS
   MplPhyDetect(
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplPhyReset
//      Reset the PHY device
//      
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         A MacPhyter device was successfully probed
//      NS_STATUS_HARDWARE_FAILURE
//         Could not find a MacPhyter device
//
//*****************************************************************************
MPL_STATUS
   MplPhyReset(
      IN NS_VOID       *pMplHandle
      );


//*****************************************************************************
//   MplPhyLinkSetup
//      Setup the link on the PHY with the requested settings
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link was set on the PHY device
//      NS_STATUS_INVALID_PARM
//         An invalid link configuration was detected
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error
//
//*****************************************************************************
MPL_STATUS
   MplPhyLinkSetup (
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplPhyGetLinkStatus
//      Returns the status of the link
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_STATUS_NONE
//         An unexpected hardware error occurred while processing this request
//          and the link state was not retrieved.
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up
//
//*****************************************************************************
MPL_LINK_STATUS
   MplPhyGetLinkStatus (
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplPhyGetLinkSpeed
//      Returns current link speed. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_SPEED_TENMBPS 
//         The current link speed is 10Mbps
//      MPL_LINK_SPEED_HUNDREDMBPS
//         The current link speed is 100Mbps
//
//*****************************************************************************
MPL_LINK_SPEED 
   MplPhyGetLinkSpeed(
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplPhyGetLinkDuplex
//      Returns current link duplex mode. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_DUPLEX_HALF
//         The current duplex mode is half
//      MPL_LINK_DUPLEX_FULL
//         The current duplex mode is full
//
//*****************************************************************************
MPL_LINK_DUPLEX
   MplPhyGetLinkDuplex(
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplPhyAutoNegDone
//      Returns if Auto Neg process is completed. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The link was setup following a Auto-Neg process
//      NS_FALSE
//         The link was setup in forced mode
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyAutoNegDone(
      IN NS_VOID   *pMplHandle
      );


//*****************************************************************************
//   MplPhyLinkSetupComplete
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The link was sucessfully setup
//      NS_FALSE
//         The link is not up (Assert - should not happen)
//      
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyLinkSetupComplete(
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplPhyRequiresPatch
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The PHY requires patches to be applied
//      NS_FALSE
//         No Patch work required
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyRequiresPatch(
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplPhyGetDeviceAddr
//      Return the Phy device address
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//     Phy device addr
//
//*****************************************************************************
NS_UINT
   MplPhyGetDeviceAddr (
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplPhyMdioRead
//      Read Phy register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to read (in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//
//   Return Value
//     Reg data
//
//*****************************************************************************
NS_UINT32
   MplPhyMdioRead(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex
      );

//*****************************************************************************
//   MplPhyMdioWrite
//      Write Phy register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to write(in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//      regData
//         Data to write
//
//   Return Value
//     None
//
//*****************************************************************************
NS_VOID
   MplPhyMdioWrite(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex,
      IN NS_UINT32  regData
      );

//+++++ Power Management and WoL APIs
//###################################

//*****************************************************************************
//   MplPowerSetState
//      Set a new power state on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      newState
//         New power state that needs to be set on the device
//      pmeEnable
//         When set to NS_TRUE, MPL enables PME on the device
//         Some OSs do this automatically, some do not
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The new state was successfully set
//      NS_STATUS_FAILURE
//         The device could not be set to the new state
//
//*****************************************************************************
MPL_STATUS
   MplPowerSetState(
      IN NS_VOID         *pMplHandle,
      IN MPL_POWER_STATE  newState,
      IN NS_BOOLEAN       pmeEnable
      );

//*****************************************************************************
//   MplWolCfg
//     Configure the WOL operation on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      wakeEnable
//         Flag to enable (set to NS_TRUE) or disable (set to NS_FALSE) Wol
//      wakeType
//         When WOL is enabled, represents a bit-map of wake-up events that
//         needs to be enabled. It is a logical OR of,
//          MPL_WOL_MAGIC - Enable wake on Magic Pattern.
//          MPL_WOL_PATTERN - Enable wake on pattern match (Patterns needs
//                            to be separately defined).
//          MPL_WOL_BROADCAST - Enable wake on any broadcast packet.
//          MPL_WOL_MULTICAST - Enable wake on multicast packet.
//          MPL_WOL_DIRECTED - Enable wake on directed (unicast) packet to 
//                             the device's address.
//          MPL_WOL_LINK - Enable wake on link status change.
//          MPL_WOL_ARP - Enable wake on any ARP packet.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The WOL events were successfully enabled or disabled (depending
//         on wakeEnable).
//      NS_STATUS_INVALID_PARM
//         The wake event type is not supported on the device.
//
//*****************************************************************************
MPL_STATUS
   MplWolCfg(
      IN NS_VOID     *pMplHandle,
      IN NS_BOOLEAN   wakeEnable,
      IN NS_UINT      wakeType
      );

// Wol Type Flag bits
#define MPL_WOL_MAGIC      0x00000001
#define MPL_WOL_PATTERNS   0x00000002
#define MPL_WOL_BROADCAST  0x00000004
#define MPL_WOL_MULTICAST  0x00000008
#define MPL_WOL_DIRECTED   0x00000010
#define MPL_WOL_LINK       0x00000020
#define MPL_WOL_ARP        0x00000040

//*****************************************************************************
//   MplWolGetCfg
//     Retrieve the current WOL configuration on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWakeType
//         Pointer to caller provider variable in which the bit-map of wake-up
//         events currently enabled is returned. It is a logical OR of,
//          MPL_WOL_MAGIC - Enable wake on Magic Pattern.
//          MPL_WOL_PATTERN - Enable wake on pattern match (Patterns needs to
//                            be separately defined).
//          MPL_WOL_BROADCAST - Enable wake on any broadcast packet.
//          MPL_WOL_MULTICAST - Enable wake on multicast packet.
//          MPL_WOL_DIRECTED - Enable wake on directed (unicast) packet to
//                             the device's address.
//          MPL_WOL_LINK - Enable wake on link status change.
//          MPL_WOL_ARP - Enable wake on any ARP packet.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The WOL configuration was returned successfully
//
//*****************************************************************************
MPL_STATUS
   MplWolGetCfg(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT   *pWakeType
      );

//*****************************************************************************
//   MplWolClearPattern
//     Clear all WOL wake patterns on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake patterns were successfully cleared
//
//*****************************************************************************
MPL_STATUS
   MplWolClearPattern(
      IN NS_VOID   *pMplHandle
      );

//*****************************************************************************
//   MplWolAddPattern
//     Add a new WOL wake pattern to the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWolPattern
//         Pointer to a MPL_WOL_PATTERN structure in which the new pattern 
//         and its associated mask is noted
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake pattern was successfully added.
//      NS_STATUS_RESOURCES
//         The wake pattern not added since it exceeds the maximum supported
//         on this device.
//      NS_STATUS_INVALID_PARM
//         The wake pattern buffer is larger than the maximum supported by
//         the device.
//
//*****************************************************************************
MPL_STATUS
   MplWolAddPattern(
      IN NS_VOID         *pMplHandle,
      IN MPL_WOL_PATTERN *pWolPattern
      );

//*****************************************************************************
//   MplWolRemovePattern
//     Remove a WOL wake pattern from the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWolPattern
//         Pointer to a MPL_WOL_PATTERN structure in which the pattern 
//         and its associated mask is noted
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake pattern was deleted successfully.
//      NS_STATUS_INVALID_PARM
//         The wake pattern was not found in the list
//
//*****************************************************************************
MPL_STATUS
   MplWolRemovePattern(
      IN NS_VOID         *pMplHandle,
      IN MPL_WOL_PATTERN *pWolPattern
      );

//+++++ Miscellaneous APIs
//###################################

//*****************************************************************************
//   MplSetMacAddress
//      Set the MAC's address on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pMacAddr
//         Pointer to caller supplied variable with the new MAC address
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The MAC address was sucessfully programmed on the device
//
//*****************************************************************************
MPL_STATUS
   MplSetMacAddress (
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR    pMacAddr
      );

//*****************************************************************************
//   MplGetMacAddress
//      Get the current MAC's address on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//      eeProm
//         If set to NS_TRUE, the permanent (EEPROM programmed) mac address 
//          for the device is returned back to the caller, else the current
//          active address is returned.
//      pMacAddr
//         Pointer to a caller supplied MPL_MAC_ADDR variable where the current
//          or default network address is returned.
//
//   Return Value
//      NS_STATUS_SUCCESS
//       The mac address was successfully read and is returned.
//      NS_STATUS_FAILURE
//       Failed to get the MAC address
//*****************************************************************************
MPL_STATUS
   MplGetMacAddress (
      IN NS_VOID        *pMplHandle,
      IN NS_BOOLEAN      eeProm,
      OUT MPL_MAC_ADDR   pMacAddr
      );

//*****************************************************************************
//   MplGetDeviceId
//      Get the device Id to identify the MacPhyter version
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//
//   Return Value
//      MPL_DEVICE_ID_DP83815 = Macphyter-I found
//      MPL_DEVICE_ID_DP83816 = Macphyter-II found
//      MPL_DEVICE_ID_DP83818 = Macphyter-III found
//*****************************************************************************
MPL_DEVICE_ID
   MplGetDeviceId(
      IN NS_VOID        *pMplHandle
      );

//*****************************************************************************
//   MplGetRegs
//      Get the current operational registers values
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//      pRegs
//         A caller supplied buffer (atleast 255*4 bytes)
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   MplGetRegs(
      IN NS_VOID        *pMplHandle,
      IN NS_UINT32      *pRegs
      );

//*****************************************************************************
//   MplRegRead
//      Read a 32-bit register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      regIndex
//         Register to read (in terms of absolute offset e.g. CR = 0x0, 
//                             TXCFG = 0x24)
//
//   Return Value
//     Reg data
//
//*****************************************************************************
NS_UINT32
   MplRegRead(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    regIndex
      );

//*****************************************************************************
//   MplRegWrite
//      Write a 32-bit register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      regIndex
//         Register to read (in terms of absolute offset e.g. CR = 0x0, 
//                             TXCFG = 0x24)
//      regData
//         Data to write
//
//   Return Value
//     None
//
//*****************************************************************************
NS_VOID
   MplRegWrite(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    regIndex,
      IN NS_UINT32  regData
      );

//*****************************************************************************
//   MplDumpTransmitRing
//      Dump the Txd ring
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   MplDumpTransmitRing(
      IN NS_VOID        *pMplHandle
      );

//*****************************************************************************
//   MplDumpReceiveRing
//      Dump the Rxd ring
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   MplDumpReceiveRing(
      IN NS_VOID        *pMplHandle
      );

#ifndef MPL_NO_EEPROM
//*****************************************************************************
//   MplEERead
//     Read 16-bit word from a given EEPROM offset 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize.
//      index
//         Offset from where to read EEPROM contents
//     *pData
//         Caller provided variable where the 16-bit contents is returned
//
//   Return Value
//      NS_STATUS_SUCCESS
//       The EEPROM was successfully read
//      NS_STATUS_INVALID_PARM
//       The EEPROM offset is invalid
//*****************************************************************************
MPL_STATUS
   MplEERead(
      IN NS_VOID    *pMplHandle,
      IN NS_UINT     index,
      IN NS_UINT16  *pData   
      );
#endif// MPL_NO_EEPROM

//+++++ MPL Callbacks into NSM

//*****************************************************************************
//   NsmTransmitDone
//     Transmit Done Notification 
//
//   Parameters
//      pClientDevHndl
//         NSM device handle registered with MPL (during MplInitialize)
//      packetCnt
//         Count of packets whose completion is being reported.
//      pTxDone
//         MPL allocated MPL_TRANSMIT_DONE structure array in which the 
//          NSM packet handles and statuses are returned.
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   NsmTransmitDone(
    IN NS_VOID *pClientDevHndl,
    IN NS_UINT  packetCnt,
    IN MPL_TRANSMIT_DONE *pTxDone
    );

//*****************************************************************************
//   NsmReceive
//     Receive Done Notification
//
//   Parameters
//      pClientDevHndl
//         NSM device handle registered with MPL (during MplInitialize)
//      packetCnt
//         Count of packets whose reception is being reported.
//      pPkt
//         MPL allocated MPL_PKT structure array in which 
//         a list of the newly received packets are reported
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   NsmReceive(
    IN NS_VOID *pClientDevHndl,
    IN NS_UINT  packetCnt,
    IN MPL_PKT *pPkt
    );

//*****************************************************************************
//   NsmLinkStatusChange
//     Link status change Notification
//
//   Parameters
//      pClientDevHndl
//         NSM device handle registered with MPL (during MplInitialize)
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   NsmLinkStatusChange(
    IN NS_VOID *pClientDevHndl
    );

#if defined(__cplusplus)
}
#endif 

#endif // _MPLCLIENT_H_
