
//******************************************************************************
//
//  MPLTASKRXFILTER.H
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  MPL receive filtering Offload Task Module.
//
//  This file declares the APIs for
//     o Filtering of Receive packets based on
//       - Dest. Multicast Address
//       - Dest. Unicast   Address
//       - TBD
//
//  MPL clients should not include this file directly, instead include 
//  mplclient.h and defining MPL
//******************************************************************************

#ifndef _MPLTASKRXFILTER_H_
#define _MPLTASKRXFILTER_H_

#if defined(__cplusplus)
extern "C"
{
#endif

//*****************************************************************************
//   MplTaskFilterCfg
//      Configure the receive filtering module 
//      (Currently Multicast and Unicast Destination Address Filtering)
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      enableFlag
//         Set to NS_TRUE to enable the reception of multicast frames and 
//         filtering, NS_FALSE to disable.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The task offload was successfully enabled or disabled
//      NS_STATUS_NOT_SUPPORTED
//         This offload task or selected configuration option is not 
//         supported by MPL.
//      NS_STATUS_INVALID_PARM
//         An invalid parameter was detected.
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterCfg(
      IN NS_VOID         *pMplHandle,
      IN NS_BOOLEAN       enableFlag
      );

//*****************************************************************************
//   MplTaskFilterReload
//      Reload receive hash filters from previous defined listen list
//      Useful for if a Mac Reset is done and the listen lists are to be
//      restored to their previous saved values
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The listen list was successfully reloaded
//      NS_STATUS_FAILURE
//         This offload feature is disabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterReload(
      IN NS_VOID         *pMplHandle
      );

//*****************************************************************************
//   MplTaskFilterMcastAddAddr
//      Add a new multicast address to the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Multicast addr to be added to the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was successfully added
//      NS_STATUS_RESOURCES
//         Unable to allocate required resources
//      NS_STATUS_FAILURE
//         This offload feature is disabled
//      NS_STATUS_INVALID_PARM
//         An invalid parameter was detected.
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterMcastAddAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterMcastDeleteAddr
//      Delete a multicast address from the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Multicast addr to be deleted from the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was successfully deleted
//      NS_STATUS_FAILURE
//         Could not find the address to delete or feature not enabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterMcastDeleteAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterMcastCheckAddr
//      Check if a multicast address is present in the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Multicast addr to be checked in the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was found in the receive accept list
//      NS_STATUS_FAILURE
//         Could not find the address or feature not enabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterMcastCheckAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterMcastClearList
//      Clear the multicast clear list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The list was successfully cleared
//      NS_STATUS_FAILURE
//         This offload feature is disabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterMcastClearList(
      IN NS_VOID         *pMplHandle
      );

//*****************************************************************************
//   MplTaskFilterUcastAddAddr
//      Add a new unicast address to the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Unicast addr to be added to the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was successfully added
//      NS_STATUS_RESOURCES
//         Unable to allocate required resources
//      NS_STATUS_FAILURE
//         This offload feature is disabled
//      NS_STATUS_INVALID_PARM
//         An invalid parameter was detected.
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterUcastAddAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterUcastDeleteAddr
//      Delete a unicast address from the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Unicast addr to be deleted from the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was successfully deleted
//      NS_STATUS_FAILURE
//         Could not find the address to delete or feature not enabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterUcastDeleteAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterUcastCheckAddr
//      Check if a unicast address is present in the receive accept list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pAddr
//         Unicast addr to be checked in the filter list.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The address was found in the receive accept list
//      NS_STATUS_FAILURE
//         Could not find the address or feature not enabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterUcastCheckAddr(
      IN NS_VOID         *pMplHandle,
      IN MPL_MAC_ADDR     pAddr
      );

//*****************************************************************************
//   MplTaskFilterUcastClearList
//      Clear the unicast clear list
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The list was successfully cleared
//      NS_STATUS_FAILURE
//         This offload feature is disabled
//
//*****************************************************************************
MPL_STATUS
   MplTaskFilterUcastClearList(
      IN NS_VOID         *pMplHandle
      );
#if defined(__cplusplus)
}
#endif 

#endif //_MPLTASKRXFILTER_H_
