
//******************************************************************************
//
//  MPLLINK.C
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  MPL Link Processing.
//
//  This file contains the API implementations for
//     o MPL link configuration
//     o Processing link change events
//     o Reporting link status
//
//******************************************************************************

#include <mplinternal.h>

// Local helful debug macros
#undef ENTER
#undef EXIT
#undef ASSERT
#undef PRINT
#undef PRINTI
#undef PRINTS
#undef TBREAK

#define ENTER(fn) MPL_CENTER(DZONE_MPL_LINK, fn)
#define EXIT(fn) MPL_CEXIT(DZONE_MPL_LINK, fn)
#define ASSERT(le) MPL_CASSERT(DZONE_MPL_LINK, le)
#define PRINT(fmt) MPL_CTRACE(DZONE_MPL_LINK, fmt)
#define PRINTI(v) MPL_CTRACE(DZONE_MPL_LINK, (" "#v": %x \n",(NS_UINT)(v)))
#define PRINTS(v) MPL_CTRACE(DZONE_MPL_LINK, (" "#v": %s \n", v))
#define TBREAK(fmt) MPL_CTRACEBREAK(DZONE_MPL_LINK, fmt)

// Locals
static NS_BOOLEAN linkCompare(MPL_CONTEXT *pMplCtx, MPL_LINK_CFG *pLinkCfg);
static NS_VOID setPause(MPL_CONTEXT *pMplCtx);

//*****************************************************************************
//   MplLinkCfg
//      Configures the network media speed (10/100), duplex (half/full), 
//       negotiation (Auto/Forced) and network PAUSE parameters.
//      When configuring auto mode, the speed and duplex setting passed should
//       reflect the best case scenario i.e most desired speed and duplex
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pLinkCfg
//         Points to a MPL_LINK_CFG structure with the desired link 
//          configuration.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         Link configured and the link is UP.
//      NS_STATUS_ASYNCH_COMPLETION
//         Link is currently being configured or being negotiated.
//      NS_STATUS_INVALID_PARM
//         An invalid parameter was detected in the configuration structure.
//      NS_STATUS_HARDWARE_FAILURE
//         An expected failure occured while setting the link config
//
//*****************************************************************************
MPL_STATUS
   MplLinkCfg (
      IN NS_VOID        *pMplHandle,
      IN MPL_LINK_CFG   *pLinkCfg
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   MPL_STATUS status;

   ENTER(MplLinkCfg);

   // FM: Assert for pLinkCfg == NULL

   // We start with no link
   pMplCtx->linkStatus = MPL_LINK_STATUS_NONE;

   // Make a copy of the requested configuration.
   OaiMemCopy(&pMplCtx->linkCfg, pLinkCfg, sizeof(MPL_LINK_CFG));

   // If the current link matches what's requested then don't bother setting it.
   if (linkCompare(pMplCtx, pLinkCfg) == NS_TRUE)
   {
      pMplCtx->linkStatus = MPL_LINK_STATUS_UP;
      EXIT(MplLinkCfg);
      return NS_STATUS_SUCCESS;
   }

   // Make sure we support the requested pause type
   if ((pLinkCfg->pauseType) && 
         (pMplCtx->caps.linkCaps.linkFlags != MPL_LINK_PAUSE_SYMMETRICAL))
   {
      // If the Mac does not support symmetric pause then make sure the
      //  requested direction matches the one supported
      if (pLinkCfg->pauseType != pMplCtx->caps.linkCaps.linkFlags)
      {
         EXIT(MplLinkCfg);
         return NS_STATUS_INVALID_PARM;
      }
   }

   // Link is now down - Start the link setup process
   pMplCtx->linkStatus = MPL_LINK_STATUS_DOWN;

   // Setup the link on the Phy
   status = MplPhyLinkSetup(pMplHandle);

   // Check for Status
   if (status != NS_STATUS_SUCCESS)
   {
      EXIT(MplLinkCfg);
      return NS_STATUS_HARDWARE_FAILURE;
   }
   else
   {
      EXIT(MplLinkCfg);
      return NS_STATUS_ASYNCH_COMPLETION;
   }
}


//*****************************************************************************
//   MplLinkGetCfg
//      Returns the current link configuration on the device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pLinkCfg
//         Pointer to a caller allocated MPL_LINK_CFG structure in which the 
//          current link configuration is returned.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link configuration was successfully returned.
//      NS_STATUS_FAILURE
//         The link is not configured (yet).
//
//*****************************************************************************
MPL_STATUS
   MplLinkGetCfg (
      IN NS_VOID        *pMplHandle,
      OUT MPL_LINK_CFG  *pLinkCfg
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplLinkGetCfg);

   // Make sure the link has been configured
   if (pMplCtx->linkStatus == MPL_LINK_STATUS_NONE)
   {
      EXIT(MplLinkGetCfg);
      return NS_STATUS_FAILURE;
   }

   // Copy our current configuration and return
   OaiMemCopy(pLinkCfg, &pMplCtx->linkCfg, sizeof(MPL_LINK_CFG));

   EXIT(MplLinkGetCfg);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplLinkGetStatus
//      Returns the link status reported on the device.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      MPL_LINK_STATUS_NONE
//         An unexpected hardware error occurred while processing this request
//          and the link state was not retrieved.
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up
//
//*****************************************************************************
MPL_LINK_STATUS
   MplLinkGetStatus (
      IN NS_VOID   *pMplHandle
      )
{
   ENTER(MplLinkGetStatus);

   EXIT(MplLinkGetStatus);
   // Query the hardware and return the status
   return MplPhyGetLinkStatus(pMplHandle);
}

//*****************************************************************************
//   MplLinkProcessChange
//      Handles the link change event
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up.
//
//*****************************************************************************
MPL_LINK_STATUS
   MplLinkProcessChange (
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplLinkProcessChange);

   // Query the hardware on the latest status
   switch (MplPhyGetLinkStatus(pMplHandle))
   {
      case MPL_LINK_STATUS_UP : // Link Up
            // FM : Bounce logic needed?
            pMplCtx->linkStatus = MPL_LINK_STATUS_UP;
            break;

      case MPL_LINK_STATUS_ACTIVE : // Link is Up, but AutoNeg still on 
            pMplCtx->linkStatus = MPL_LINK_STATUS_ACTIVE;
            break;

      case MPL_LINK_STATUS_DOWN : // Link Down 
      default :
            pMplCtx->linkStatus = MPL_LINK_STATUS_DOWN;
            break;
   }

   EXIT(MplLinkProcessChange);
   return pMplCtx->linkStatus;
}


//*****************************************************************************
//   MplLinkUpComplete
//      Completes the configuration of the device following the 
//       auto-negotiation process.
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link configuration is complete.
//      NS_STATUS_FAILURE
//         The link state is not UP.
//
//*****************************************************************************
MPL_STATUS
   MplLinkUpComplete (
      IN NS_VOID      *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 txCfgVal, rxCfgVal;

   ENTER(MplLinkUpComplete);

   // Check if the link is really up
   if (pMplCtx->linkStatus != MPL_LINK_STATUS_UP)
   {
      EXIT(MplLinkUpComplete);
      return NS_STATUS_FAILURE;
   }

   // Finish PHY link setup
   if (MplPhyLinkSetupComplete(pMplHandle) == NS_FALSE)
   {
      EXIT(MplLinkUpComplete);
      return NS_STATUS_FAILURE;
   }

   // Setup MAC operations
   txCfgVal = MPL_READ32(pMplCtx, TXCFG);
   rxCfgVal = MPL_READ32(pMplCtx, RXCFG);
   if (pMplCtx->linkCfg.duplex == MPL_LINK_DUPLEX_FULL)
   {
      // Set MAC to full duplex operation

      // Tx Engine - Ignore : Carrier Sense and Heartbeat
      MPL_WRITE32(pMplCtx, TXCFG, txCfgVal | CARRIER_IGNORE | SQE_IGNORE);

      // Rx Engine - Enable : FD operation
      MPL_WRITE32(pMplCtx, RXCFG, rxCfgVal | FULLDUPLEX_EN);
   }
   else
   {
      // Set MAC to half duplex operation

      // Tx Engine - Enable : Carrier Sense and Heartbeat
      MPL_WRITE32(pMplCtx, TXCFG, (txCfgVal & ~(CARRIER_IGNORE | SQE_IGNORE)));

      // Rx Engine - Disable : FD operation
      MPL_WRITE32(pMplCtx, RXCFG, (rxCfgVal & ~(FULLDUPLEX_EN)));
   }

   // Set Pause configuration on the MAC
   setPause(pMplCtx);

   EXIT(MplLinkUpComplete);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplLinkInterrupt
//      Processes an incoming link related interrupt
//      At this time this is an MPL internal function (called from
//       MplIntteruptDoneInternal) 
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The interrupt was successfully fielded.
//
//*****************************************************************************
MPL_STATUS
   MplLinkInterrupt(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
  
   // Notify NSM, it will ensure further processing after
   //  taking care of synchronization issues
   NsmLinkStatusChange(pMplCtx->pClientHandle);
   
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//        
//   linkCompare
//      Check if the requested link configuration matches the one on the 
//       device
//
//   Parameters
//      pMplCtx
//         Pointer to MPL Context
//
//   Return Value
//      NS_TRUE
//        The requested link configuration matches the one on the device
//      NS_FALSE
//        The requested link configuration does NOT match the one on the device
//
//*****************************************************************************
static
NS_BOOLEAN
   linkCompare(
      IN MPL_CONTEXT    *pMplCtx,
      IN MPL_LINK_CFG   *pLinkCfg
      )
{
   NS_BOOLEAN autoDone;

   ENTER(linkCompare);

   // FM: Check if this version of the PHY would need some patch work
   // If so, then we need to reestablish link
   if (MplPhyRequiresPatch(pMplCtx) == NS_TRUE)
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if link is up, if not no match
   if (MplPhyGetLinkStatus(pMplCtx) != MPL_LINK_STATUS_UP)
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if speed matches
   if (MplPhyGetLinkSpeed(pMplCtx) != pLinkCfg->speed)
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if duplex matches
   if (MplPhyGetLinkDuplex(pMplCtx) != pLinkCfg->duplex)
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if current pause matches
   if (pMplCtx->pauseType != pLinkCfg->pauseType)
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if Auto-Neg is requested, but prev setup was in forced mode
   autoDone = MplPhyAutoNegDone(pMplCtx);
   if ((pLinkCfg->mode == MPL_LINK_MODE_AUTO) && (autoDone == NS_FALSE))
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // Check if forced mode is requested, but prev setup was in Auto-Neg mode
   if ((pLinkCfg->mode == MPL_LINK_MODE_FORCED) && (autoDone == NS_TRUE))
   {
      EXIT(linkCompare);
      return NS_FALSE;
   }

   // All OK, link cfg matches
   return NS_TRUE;
}

//*****************************************************************************
//        
//   setPause
//      Set the pause configurations on the MAC
//
//   Parameters
//      pMplCtx
//         Pointer to MPL Context
//
//   Return Value
//      None
//
//*****************************************************************************
static
NS_VOID
   setPause(
      IN MPL_CONTEXT    *pMplCtx
      )
{
   NS_UINT32 pcrVal = 0x0;
   MPL_LINK_CFG *pLinkCfg = &pMplCtx->linkCfg;

   ENTER(setPause);

   // Check if pause frame reception is enabled
   if (pMplCtx->pauseType & MPL_LINK_PAUSE_RECEIVE)
   {
      pcrVal |= RXPAUSE_EN | PS_MCAST | PS_DA;
   }

   // Check if pause frame transmission is enabled
   if (pMplCtx->pauseType & MPL_LINK_PAUSE_TRANSMIT)
   {
      // Status FIFO - Lower Threshold
      if (pLinkCfg->pauseRxSTLO == 0x0) 
         pcrVal |= PTHRESHLO_SFIFO_DIS; 
      else if (pLinkCfg->pauseRxSTLO < 2) 
              pcrVal |= PTHRESHLO_SFIFO2PKTS; 
           else if (pLinkCfg->pauseRxSTLO < 4) 
                   pcrVal |= PTHRESHLO_SFIFO4PKTS; 
                else 
                   pcrVal |= PTHRESHLO_SFIFO8PKTS; 
       
      
      // Status FIFO - Higher Threshold
      if (pLinkCfg->pauseRxSTHI < pLinkCfg->pauseRxSTLO) 
         pLinkCfg->pauseRxSTHI = pLinkCfg->pauseRxSTLO;

      if (pLinkCfg->pauseRxSTHI == 0x0) 
         pcrVal |= PTHRESHHI_SFIFO_DIS; 
      else if (pLinkCfg->pauseRxSTHI >= 4) 
              pcrVal |= PTHRESHHI_SFIFO8PKTS; 
           else if (pLinkCfg->pauseRxSTHI >= 2) 
                   pcrVal |= PTHRESHHI_SFIFO4PKTS; 
                else 
                   pcrVal |= PTHRESHHI_SFIFO2PKTS; 

      // Data FIFO - Lower Threshold
      if (pLinkCfg->pauseRxDALO == 0x0) 
         pcrVal |= PTHRESHLO_DFIFO_DIS; 
      else if (pLinkCfg->pauseRxDALO < 512) 
              pcrVal |= PTHRESHLO_DFIFO512; 
           else if (pLinkCfg->pauseRxDALO < 1024) 
                   pcrVal |= PTHRESHLO_DFIFO1024; 
                else 
                   pcrVal |= PTHRESHLO_DFIFO1536; 
      
      // Data FIFO - Higher Threshold
      if (pLinkCfg->pauseRxDAHI < pLinkCfg->pauseRxDALO) 
         pLinkCfg->pauseRxDAHI = pLinkCfg->pauseRxDALO;

      if (pLinkCfg->pauseRxDAHI == 0x0) 
         pcrVal |= PTHRESHHI_DFIFO_DIS; 
      else if (pLinkCfg->pauseRxDAHI >= 1024) 
              pcrVal |= PTHRESHHI_DFIFO1536; 
           else if (pLinkCfg->pauseRxDAHI >= 512) 
                   pcrVal |= PTHRESHHI_DFIFO1024; 
                else 
                   pcrVal |= PTHRESHHI_DFIFO512; 

      // Set pause counter
      pcrVal |= pLinkCfg->pauseCtr; 
   }

   MPL_WRITE32(pMplCtx, PCR, pcrVal);
   EXIT(setPause);
   return;
}
