
//******************************************************************************
//
//  MPLPHY.C
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  MPL Phy Interface Module.
//
//  This file contains the API implementations for
//     o Detection of PHY
//     o Configuring the PHY
//     o Reporting PHY status
//
//******************************************************************************

#include <mplinternal.h>

// Local helful debug macros
#undef ENTER
#undef EXIT
#undef ASSERT
#undef PRINT
#undef PRINTI
#undef PRINTS
#undef TBREAK

#define ENTER(fn) MPL_CENTER(DZONE_MPL_LINK, fn)
#define EXIT(fn) MPL_CEXIT(DZONE_MPL_LINK, fn)
#define ASSERT(le) MPL_CASSERT(DZONE_MPL_LINK, le)
#define PRINT(fmt) MPL_CTRACE(DZONE_MPL_LINK, fmt)
#define PRINTI(v) MPL_CTRACE(DZONE_MPL_LINK, (" "#v": %x \n",(NS_UINT)(v)))
#define PRINTS(v) MPL_CTRACE(DZONE_MPL_LINK, (" "#v": %s \n", v))
#define TBREAK(fmt) MPL_CTRACEBREAK(DZONE_MPL_LINK, fmt)

// Local functions
static NS_VOID mdioIdle(MPL_CONTEXT *pMplCtx);
static NS_VOID mdioSync(MPL_CONTEXT *pMplCtx);

#ifndef MPL_EXTERNAL_PHY
static NS_VOID phyAdjust(MPL_CONTEXT *pMplCtx);
static NS_VOID atanPatchUp(MPL_CONTEXT *pMplCtx);
static NS_VOID atanPatchDown(MPL_CONTEXT *pMplCtx);
#else
static NS_VOID mdioWriteReg(MPL_CONTEXT *pMplCtx, NS_UINT phyDevAddr, 
                             NS_UINT regIndex,NS_UINT16  regData);
static NS_UINT16 mdioReadReg(MPL_CONTEXT *pMplCtx, NS_UINT phyDevAddr,
                               NS_UINT regIndex);
#endif // MPL_EXTERNAL_PHY

//*****************************************************************************
//   MplPhyDetect
//      Detect the presence of a MacPhyter device on the system.
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         A MacPhyter device was successfully probed
//      NS_STATUS_HARDWARE_FAILURE
//         Could not find a MacPhyter device
//
//*****************************************************************************
MPL_STATUS
   MplPhyDetect(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 phyIdr1, phyIdr2;

   ENTER(MplPhyDetect);

   // Sync up the MII bus
   mdioSync(pMplCtx);

   // Scan for a matching OUI.
   for (pMplCtx->phyDeviceAddr = 31; pMplCtx->phyDeviceAddr != 0; 
                      pMplCtx->phyDeviceAddr--)
   {
      // Check bits 3 - 18 of the OUI
      phyIdr1 = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_PHYIDR1);

      switch (phyIdr1)
      {
         case MP_PHY_ID1 :  // NSC PHY Ids
                           phyIdr2 = MplPhyMdioRead(pMplCtx, 
                                        pMplCtx->phyDeviceAddr, MII_PHYIDR2);

                           switch (phyIdr2 & 0xfff0)
                           {
                              case MP_PHY_REV : 
                              case ASPEN_PHY_REV : 
                                 pMplCtx->phySiliconRev = phyIdr2 & 0xfff0; 
                                 EXIT(MplPhyDetect);
                                 return NS_STATUS_SUCCESS;
                             default : break;
                           }
                           break;

#ifdef MPL_EXTERNAL_PHY
        // Add a case here to match your PHY Ids
        // EXTERNAL_PHY_HOOK  Modify this to suit your specific device needs
        case DEFINE_PHY_ID :
                            break;
#endif
      }
   }


   EXIT(MplPhyDetect);
   return NS_STATUS_HARDWARE_FAILURE;
}

//*****************************************************************************
//   MplPhyReset
//      Reset the PHY device
//      
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         A MacPhyter device was successfully probed
//      NS_STATUS_HARDWARE_FAILURE
//         Could not find a MacPhyter device
//
//*****************************************************************************
MPL_STATUS
   MplPhyReset(
      IN NS_VOID       *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT i;

   ENTER(MplPhyReset);

   // Hit the reset bit
   MplPhyMdioWrite(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR, PHY_RESET);

   // Wait (total 10 msec) for the PHY to come out of reset
   for (i = 0x0; i < 10; i ++)
   {
      OaiSleep(RESETINTERVAL_PHY);
      if ((MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR) & 
                              PHY_RESET) == 0x0)
      {
         // Reset done

#ifndef MPL_EXTERNAL_PHY
         // For NSC internal PHYs some more work remains to be done
         {
            NS_UINT32 regVal;

            // Apply Optimization patch
            phyAdjust(pMplCtx);

            // Enable Phy Interrupts, after clearing pending ones
            regVal = MPL_READ32(pMplCtx, MISR);
            MPL_WRITE32(pMplCtx, MISR, regVal & ~(MSK_LINK | MSK_ANC));
            MPL_WRITE32(pMplCtx, MICR, PHY_INT);
         }
#else
         // Set Interrupts and other init time operations
         // EXTERNAL_PHY_HOOK  Modify this to suit your specific device needs
         {
            DEFINE_PHY_INTERRUPTS;  
         }
#endif // MPL_EXTERNAL_PHY


         EXIT(MplPhyReset);
         return NS_STATUS_SUCCESS;
      }
   }

   EXIT(MplPhyReset);
   return NS_STATUS_HARDWARE_FAILURE;
}

//*****************************************************************************
//   MplPhyLinkSetup
//      Setup the link on the PHY with the requested settings
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The link was set on the PHY device
//      NS_STATUS_INVALID_PARAM
//         An invalid link configuration was detected
//      NS_STATUS_HARDWARE_FAILURE
//         Unexpected hardware error
//
//*****************************************************************************
MPL_STATUS
   MplPhyLinkSetup (
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 bmcrVal, anarVal, bmcrValTmp;

   ENTER(MplPhyLinkSetup);

   // Reset the PHY to apply patches before link being set
   if (MplPhyReset(pMplCtx) != NS_STATUS_SUCCESS)
   {
      return NS_STATUS_HARDWARE_FAILURE;
   }

   // Setup AutoNeg advertisements
   if (pMplCtx->linkCfg.mode == MPL_LINK_MODE_AUTO)
   {
#ifndef MPL_EXTERNAL_PHY
      // Prelink Patch Works - Only for Internal PHYs
      {
         // Apply fix for ATAN switch link-up issue
         atanPatchUp(pMplCtx);
      }
#endif // MPL_EXTERNAL_PHY

      // Defaults
      bmcrVal = AUTONEG_ENABLE | AUTONEG_RESTART;
      anarVal = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_ANAR);
      anarVal &= ~(ADV_100BASET_FD | ADV_100BASET_HD | ADV_10BASET_FD
                    | ADV_10BASET_HD | ADV_PAUSE | ADV_ASM_DIR);

      // Using a fall though switch case statement since
      // in AUTONEG mode the modes must be backwards compatible
      // When the user selects Half Duplex then he is backwards
      // compatible only in Half duplex modes and does not enter
      // any Full Duplex modes, but when in Full Duplex mode he
      // can get into any backward compatible mode.
      switch (pMplCtx->linkCfg.speed)
      {
         default:
         case MPL_LINK_SPEED_HUNDREDMBPS:
                   if (pMplCtx->linkCfg.duplex == MPL_LINK_DUPLEX_FULL)
                   {
                      anarVal |= ADV_100BASET_FD;
                   }

                   // Half duplex and backward compatible 
                   anarVal |= ADV_100BASET_HD;

         case MPL_LINK_SPEED_TENMBPS:
                   if (pMplCtx->linkCfg.duplex == MPL_LINK_DUPLEX_FULL)
                   {
                      anarVal |= ADV_10BASET_FD;
                   }

                   // Half duplex and backward compatible 
                   anarVal |= ADV_10BASET_HD;
                   break;
      }

      // Setup advertisement of Flow control (Pause)
      switch (pMplCtx->linkCfg.pauseType)
      {
         case MPL_LINK_PAUSE_TRANSMIT:

                   // Transmit of pause requests only 
                   anarVal |= ADV_ASM_DIR;
                   break;

         case MPL_LINK_PAUSE_RECEIVE:

                   // The Auto-Neg spec does not specify a way to adv receive
                   //  only of pause requests. So we will adv both asym and
                   //  sym pause support - Later after link is setup we can
                   //  disable the transmission of pause requests 
                   anarVal |= ADV_ASM_DIR | ADV_PAUSE;
                   break;

         case MPL_LINK_PAUSE_SYMMETRICAL: // Tx and Rx

                   // Both Transmit and Reception of pause requests
                   anarVal |= ADV_ASM_DIR | ADV_PAUSE;
                   break;

         default:
                   // No pause advertisement
                   break;
      }

      // If we need to do/restart AutoNegtiation then we
      // first check if we are in forced mode, if we are in forced
      // mode then we clear bit 12 of the BMCR (AUTO_NEG_ENABLE) and
      // then reset the bit to 1 and then write to register to restart
      // Auto Negotiation if required.

      bmcrValTmp = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR);
      if (!(bmcrValTmp & AUTONEG_ENABLE))
      {
         // We clear the AUTONEGENABLE bit and then reset the bit
         // before we start a AutoNegotiate process.

         bmcrValTmp &= ~AUTONEG_ENABLE;
         MplPhyMdioWrite(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR, bmcrValTmp);

         bmcrValTmp |= AUTONEG_ENABLE;
         MplPhyMdioWrite(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR, bmcrValTmp);
      }

   }
   else
   {
      // Setup for Forced mode operation

      // Defaults
      bmcrVal = 0x0;
      anarVal = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_ANAR);
      anarVal &= ~(ADV_100BASET_FD | ADV_100BASET_HD | ADV_10BASET_FD |
                     ADV_10BASET_HD | ADV_PAUSE | ADV_ASM_DIR);

      // Set the speed on the card individually
      // based on the user selection and force selection
      switch (pMplCtx->linkCfg.speed)
      {
         default:
         case MPL_LINK_SPEED_HUNDREDMBPS:
                   if (pMplCtx->linkCfg.duplex == MPL_LINK_DUPLEX_FULL)
                   {
                      bmcrVal |= FORCE_MODE_FD;
                   }

                   bmcrVal |= FORCE_SPEED_100;
                   break;

         case MPL_LINK_SPEED_TENMBPS:
                   if (pMplCtx->linkCfg.duplex == MPL_LINK_DUPLEX_FULL)
                   {
                      bmcrVal |= FORCE_MODE_FD;
                   }

                   break;
      }
   }

   // Enable advertized values
   MplPhyMdioWrite(pMplCtx, pMplCtx->phyDeviceAddr, MII_ANAR, anarVal);

   // Write the new configuration
   MplPhyMdioWrite(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMCR, bmcrVal);

   EXIT(MplPhyLinkSetup);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplPhyGetLinkStatus
//      Returns the status of the link
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_STATUS_NONE
//         An unexpected hardware error occurred while processing this request
//          and the link state was not retrieved.
//      MPL_LINK_STATUS_DOWN
//         The link is down.
//      MPL_LINK_STATUS_ACTIVE
//         The link is currently being configured or negotiated.
//      MPL_LINK_STATUS_UP
//         The link is up
//
//*****************************************************************************
MPL_LINK_STATUS
   MplPhyGetLinkStatus (
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regVal;

   ENTER(MplPhyGetLinkStatus);

   // Read the PhySts register to get the current link state
   // MacPhy requires the read of BMSR twice to correctly note
   //  the link change event
   regVal = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMSR);
   regVal = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMSR);

   // Check the link state
   if (regVal & LINK_STATUS)
   {
      // Link Up
      // If we are in Auto-Neg mode, make sure that we are completely 
      //  ready
      if (pMplCtx->linkCfg.mode == MPL_LINK_MODE_AUTO)
      {
         if (regVal & AUTONEG_COMP)
         {
            EXIT(MplPhyGetLinkStatus);
            return MPL_LINK_STATUS_UP;
         }

         // We are in the process of setting up link
         EXIT(MplPhyGetLinkStatus);
         return MPL_LINK_STATUS_ACTIVE;
      }

      EXIT(MplPhyGetLinkStatus);
      return MPL_LINK_STATUS_UP;
   }

   // All other cases no Link

   // This is a good time to apply linkup related patches
   //  (to link correctly with (potentially) a new partner)
#ifndef MPL_EXTERNAL_PHY
   if (pMplCtx->linkCfg.mode == MPL_LINK_MODE_AUTO)
   {
      // Apply fix for ATAN switch link-up issue
      atanPatchUp(pMplCtx);
   }
#endif //MPL_EXTERNAL_PHY

   // Cancel any PHY link timer
   OaiCancelTimer(pMplCtx->phyTimer);

   EXIT(MplPhyGetLinkStatus);
   return MPL_LINK_STATUS_DOWN;
}

//*****************************************************************************
//   MplPhyGetLinkSpeed
//      Returns current link speed. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_SPEED_TENMBPS 
//         The current link speed is 10Mbps
//      MPL_LINK_SPEED_HUNDREDMBPS
//         The current link speed is 100Mbps
//
//*****************************************************************************
MPL_LINK_SPEED 
   MplPhyGetLinkSpeed(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regVal;

   ENTER(MplPhyGetLinkSpeed);

#ifndef MPL_EXTERNAL_PHY
   {
      // Read the PhySts register to get the current link speed
      regVal = MPL_READ32(pMplCtx, PHYSTS);

      // Check the link state
      if (regVal & SPEED_10_STS)
      {
PRINT(("Phy Link SPEED 10Mbps\n"));
         EXIT(MplPhyGetLinkSpeed);
         return MPL_LINK_SPEED_TENMBPS;
      }
      else
      {
PRINT(("Phy Link SPEED 100Mbps\n"));
         EXIT(MplPhyGetLinkSpeed);
         return MPL_LINK_SPEED_HUNDREDMBPS;
      }
   }
#else
   // Use internal PHY registers to return current link SPEED
   // EXTERNAL_PHY_HOOK  Modify this to suit your specific device needs
   {
      pMplCtx = pMplCtx; regVal = regVal; //To supress warnings
      return DEFINE_CURRENT_SPEED; //One of MPL_LINK_SPEED_XXX
   }
#endif // MPL_EXTERNAL_PHY

}

//*****************************************************************************
//   MplPhyGetLinkDuplex
//      Returns current link duplex mode. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      MPL_LINK_DUPLEX_HALF
//         The current duplex mode is half
//      MPL_LINK_DUPLEX_FULL
//         The current duplex mode is full
//
//*****************************************************************************
MPL_LINK_DUPLEX
   MplPhyGetLinkDuplex(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regVal;

   ENTER(MplPhyGetLinkDuplex);

#ifndef MPL_EXTERNAL_PHY
   {
      // Read the PhySts register to get the current link mode
      regVal = MPL_READ32(pMplCtx, PHYSTS);

      // Check the link state
      if (regVal & DUPLEX_FULL_STS)
      {
PRINT(("Phy Link Duplex FULL \n"));
         EXIT(MplPhyGetLinkDuplex);
         return MPL_LINK_DUPLEX_FULL;
      }
      else
      {
PRINT(("Phy Link Duplex HALF \n"));
         EXIT(MplPhyGetLinkDuplex);
         return MPL_LINK_DUPLEX_HALF;
      }
   }
#else
   // Use internal PHY regs to return current DUPLEX mode
   // EXTERNAL_PHY_HOOK  Modify this to suit your specific device needs
   {
      pMplCtx = pMplCtx; regVal = regVal; //To supress warnings
      return DEFINE_CURRENT_DUPLEX; //One of MPL_LINK_DUPLEX_XXX
   }
#endif // MPL_EXTERNAL_PHY

}

//*****************************************************************************
//   MplPhyAutoNegDone
//      Returns if Auto Neg process is completed. 
//      NOTE: SHOULD be called only after determing that the link is UP - 
//            i.e MplPhyGetLinkStatus has returned MPL_LINK_STATUS_UP
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The link was setup following a Auto-Neg process
//      NS_FALSE
//         The link was setup in forced mode
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyAutoNegDone(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regVal;

   ENTER(MplPhyAutoNegDone);

   // Read the PhySts register to get the current mode
   regVal = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_BMSR);

   // Check the link state
   if (regVal & AUTONEG_COMP)
   {
PRINT(("Phy Link Auto DONE \n"));
      EXIT(MplPhyAutoNegDone);
      return NS_TRUE;
   }
   else
   {
PRINT(("Phy Link Auto NOT DONE \n"));
      EXIT(MplPhyAutoNegDone);
      return NS_FALSE;
   }
}

//*****************************************************************************
//   MplPhyLinkSetupComplete
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The link was sucessfully setup
//      NS_FALSE
//         The link is not up (Assert - should not happen)
//      
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyLinkSetupComplete(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 anar, anlPar;

   ENTER(MplPhyLinkSetupComplete);

   // Make sure we have a valid link before proceeding
   if (MplPhyGetLinkStatus(pMplHandle) != MPL_LINK_STATUS_UP)
   {
      // FM: Assert - We should never come here!!
      EXIT(MplPhyLinkSetupComplete);
      return NS_FALSE;
   }

   // Get the Duplex Status
   if (MplPhyGetLinkDuplex(pMplCtx) == MPL_LINK_DUPLEX_FULL)
   {
      // We are in Full duplex mode
      pMplCtx->linkCfg.duplex = MPL_LINK_DUPLEX_FULL;

      // Set Pause - Only in AutoNeg'ed Full Duplex mode
      if (pMplCtx->linkCfg.mode == MPL_LINK_MODE_AUTO)
      {
         // Read our adv values and that of our partner
         anar   = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_ANAR);
         anlPar = MplPhyMdioRead(pMplCtx, pMplCtx->phyDeviceAddr, MII_ANLPAR);

         /*
          * Look at the PAUSE and ASM_DIR bits between us and link partner
          *  to determine the pause setting. 
          * The following table, taken out of the IEEE 802.3ab/D6.0 spec
          * describes final PAUSE resolution
          * NOTE:  DC = Don't Care
          *
          *   LOCAL DEVICE  |   LINK PARTNER
          * PAUSE | ASM_DIR | PAUSE | ASM_DIR | PAUSE Resolution
          *-------|---------|-------|---------|--------------------
          *   0   |    0    |  DC   |   DC    | MPL_LINK_PAUSE_NONE
          *   0   |    1    |   0   |   DC    | MPL_LINK_PAUSE_NONE
          *   0   |    1    |   1   |    0    | MPL_LINK_PAUSE_NONE
          *   0   |    1    |   1   |    1    | MPL_LINK_PAUSE_TRANSMIT
          *   1   |    0    |   0   |   DC    | MPL_LINK_PAUSE_NONE
          *   1   |   DC    |   1   |   DC    | MPL_LINK_PAUSE_SYMMETRICAL
          *   1   |    1    |   0   |    0    | MPL_LINK_PAUSE_NONE
          *   1   |    1    |   0   |    1    | MPL_LINK_PAUSE_RECEIVE
          *
          */

         // Test for SYMMETRICAL pause mode
         if ((anar & ADV_PAUSE)  &&  (anlPar & ADV_PAUSE))
         {
           if (pMplCtx->linkCfg.pauseType == MPL_LINK_PAUSE_SYMMETRICAL)
           {
              pMplCtx->pauseType = MPL_LINK_PAUSE_SYMMETRICAL;
           }
           else
           {
              // Since we had earlier advertised Sym pause for rx-only mode too
              pMplCtx->pauseType = MPL_LINK_PAUSE_RECEIVE;
           }
         }
         else if (!(anar & ADV_PAUSE)  &&  (anar & ADV_ASM_DIR) &&
                    (anlPar & ADV_PAUSE) && (anlPar & ADV_ASM_DIR))
              {
                 // TRANSMIT only pause mode
                 pMplCtx->pauseType = MPL_LINK_PAUSE_TRANSMIT;
              }
              else if ((anar & ADV_PAUSE)  &&  (anar & ADV_ASM_DIR) &&
                         !(anlPar & ADV_PAUSE) && (anlPar & ADV_ASM_DIR))
                   {
                      // RECEIVE only pause mode
                      pMplCtx->pauseType = MPL_LINK_PAUSE_RECEIVE;
                   }
                   // At this point the IEEE spec says no pause generation
                   //  However unless the NSM has specifically asked for 
                   //  No-Pause or only Tx-Pause, we can enable Rx-Pause safely
                   //  since we are only receiveing pause requests and
                   //  not generating our own pause requests 
                   else if ((pMplCtx->linkCfg.pauseType == 
                                    MPL_LINK_PAUSE_TRANSMIT) ||
                            (pMplCtx->linkCfg.pauseType == MPL_LINK_PAUSE_NONE))
                        {
                           pMplCtx->pauseType = MPL_LINK_PAUSE_NONE;
                        }
                        else
                        {
                           pMplCtx->pauseType = MPL_LINK_PAUSE_RECEIVE;
                        }
      }
   }
   else
   {
      // We are in half duplex mode
      pMplCtx->linkCfg.duplex = MPL_LINK_DUPLEX_HALF;

      // Disable Pause
      pMplCtx->pauseType = MPL_LINK_PAUSE_NONE;
   }

   // Get the Speed Status
   if (MplPhyGetLinkSpeed(pMplCtx) == MPL_LINK_SPEED_HUNDREDMBPS)
   {
      // We are in 100Mbps
      pMplCtx->linkCfg.speed = MPL_LINK_SPEED_HUNDREDMBPS;
   }
   else
   {
      // We are in 10Mbps
      pMplCtx->linkCfg.speed = MPL_LINK_SPEED_TENMBPS;
   }

   // Postlink Patch Works - Only for Internal PHYs
#ifndef MPL_EXTERNAL_PHY
   {
      if (pMplCtx->linkCfg.mode == MPL_LINK_MODE_AUTO)
      {
        // Since we are linked up remove ATAN patch and go to
        //  full attenuation level
        atanPatchDown(pMplCtx);
      }

      // Start timer to watch coeff 100msec from now - Short Cable patch
      if ((pMplCtx->linkCfg.speed == MPL_LINK_SPEED_HUNDREDMBPS) &&
           (((pMplCtx->srr & 0xFF00) == 0x0300) ||
            ((pMplCtx->srr & 0xFF00) == 0x0400) ||
            (pMplCtx->srr == 0x0504)))
      {
         OaiStartTimer(pMplCtx->phyTimer, 100);
      }
   }
#endif // MPL_EXTERNAL_PHY

   EXIT(MplPhyLinkSetupComplete);
   return NS_TRUE;
}

//*****************************************************************************
//   MplPhyRequiresPatch
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//      NS_TRUE
//         The PHY requires patches to be applied
//      NS_FALSE
//         No Patch work required
//
//*****************************************************************************
NS_BOOLEAN
   MplPhyRequiresPatch(
      IN NS_VOID   *pMplHandle
      )
{
   NS_BOOLEAN retVal = NS_TRUE;

   ENTER(MplPhyRequiresPatch);

#ifndef MPL_EXTERNAL_PHY
   {
      MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

      switch (pMplCtx->phySiliconRev)
      {
         case MP_PHY_REV    :  retVal = NS_TRUE; break;
         case ASPEN_PHY_REV    :  retVal = NS_FALSE; break; //FM: TBD
      } 
   }
#endif // MPL_EXTERNAL_PHY
   
   EXIT(MplPhyRequiresPatch);
   return retVal;
}

//*****************************************************************************
//   MplPhyGetDeviceAddr
//      Return the Phy device address
//
//   Parameters
//      pMplHandle
//         MPL device handle
//
//   Return Value
//     Phy device addr
//
//*****************************************************************************
NS_UINT
   MplPhyGetDeviceAddr (
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplPhyGetDeviceAddr);
   EXIT(MplPhyGetDeviceAddr);
   return pMplCtx->phyDeviceAddr;
}

//*****************************************************************************
//   MplPhyMdioRead
//      Read Phy register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to read (in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//
//   Return Value
//     Reg data
//
//*****************************************************************************
NS_UINT32
   MplPhyMdioRead(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regData = 0x0;

   // If we are using the internal PHY then go thru the simple register
   //  API to access the PHY registers, if not do bit-banging on MDIO

#ifndef MPL_EXTERNAL_PHY
   {
      NS_UINT32 volatile *regAddr;

      // Using internal PHY
      regAddr = (NS_UINT32 volatile *) ((NS_UINT8 *)pMplCtx->pRegsBase + 
                                           PHY_MII_REG_BASE + (regIndex << 2));
      regData = MPL_LTOH32(OaiIoRead32(pMplCtx->pClientHandle, regAddr));
   }
#else
   {
      // Using external PHY - Go thru MDIO
      regData = mdioReadReg(pMplCtx, phyDevAddr, regIndex);
   }
#endif // MPL_EXTERNAL_PHY

   return regData;
}

//*****************************************************************************
//   MplPhyMdioWrite
//      Write Phy register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to write(in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//      regData
//         Data to write
//
//   Return Value
//     None
//
//*****************************************************************************
NS_VOID
   MplPhyMdioWrite(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex,
      IN NS_UINT32  regData
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   // If we are using the internal PHY then go thru the simple register
   //  API to access the PHY registers, if not do bit-banging on MDIO

#ifndef MPL_EXTERNAL_PHY
   {
      NS_UINT32 volatile *regAddr;

      // Using internal PHY
      regAddr = (NS_UINT32 volatile *) ((NS_UINT8 *)pMplCtx->pRegsBase + 
                                           PHY_MII_REG_BASE + (regIndex << 2));
      OaiIoWrite32(pMplCtx->pClientHandle, regAddr, MPL_HTOL32(regData));
   }
#else
   {
      // Use MDIO for external PHY
      mdioWriteReg(pMplCtx, phyDevAddr, regIndex, (NS_UINT16)regData);
   }
#endif // MPL_EXTERNAL_PHY

   return;
}

//*****************************************************************************
//   phyTimerHandler
//     Applies short cable patch at the end of 100msec
//
//   Parameters
//      pMplCtx
//         MPL device handle
//
//   Return Value
//      None
//
//*****************************************************************************
NS_VOID
   phyTimerHandler(
      IN NS_VOID   *pMplHandle
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   NS_UINT32 regVal;

   ENTER(MplPhyTimer);

   MPL_WRITE32(pMplCtx, PAGE, 0x0001);
   regVal = MPL_READ32(pMplCtx, DSPCFG);
   regVal &= 0x0FFF;
   MPL_WRITE32(pMplCtx, DSPCFG, regVal | 0x1000);

   OaiSleep(100); //100usec

   regVal = MPL_READ32(pMplCtx, TDATA);
   regVal &= 0xFF;

   if (regVal < 0x80 || regVal > 0xd7 ) 
   {
      MPL_WRITE32(pMplCtx, TDATA, 0x00E8);
      regVal = MPL_READ32(pMplCtx, DSPCFG);
      MPL_WRITE32(pMplCtx, DSPCFG, regVal | 0x0020);
   }

   MPL_WRITE32(pMplCtx, PAGE, 0x0);

   EXIT(MplPhyTimer);
   return;
}

#ifndef MPL_EXTERNAL_PHY
//*****************************************************************************
//   phyAdjust
//      Apply optimization patch
//
//   Parameters
//      pMplCtx
//         MPL device context
//
//   Return Value
//      None
//
//*****************************************************************************
static 
NS_VOID
   phyAdjust(
      IN MPL_CONTEXT *pMplCtx
      )
{
   NS_UINT32 regVal;

   if (pMplCtx->srr == 0x0200)
   {
      // MacPhy-1
      MPL_WRITE32(pMplCtx, PAGE, 0x0001);
      MPL_WRITE32(pMplCtx, PMDCSR, 0x0802);
      MPL_WRITE32(pMplCtx, FCOCTL, 0x0010);
      MPL_WRITE32(pMplCtx, SDCFG, 0x0333);
      MPL_WRITE32(pMplCtx, PGMCGM, 0x0860);
      MPL_WRITE32(pMplCtx, TMR, 0x2100);
      MPL_WRITE32(pMplCtx, CDCT12, 0x4f48);
      MPL_WRITE32(pMplCtx, PAGE, 0x0);
      regVal = MPL_READ32(pMplCtx, TENBT);
      MPL_WRITE32(pMplCtx, TENBT, regVal | 0x04);
   }
   else if ((pMplCtx->srr & 0xFF00) == 0x0300)
        {
           // MacPhy-1
           MPL_WRITE32(pMplCtx, PAGE, 0x0001);
           MPL_WRITE32(pMplCtx, PMDCSR, 0x189c);
           MPL_WRITE32(pMplCtx, TDATA, 0x0000);
           MPL_WRITE32(pMplCtx, DSPCFG, 0x5040);
           MPL_WRITE32(pMplCtx, SDCFG, 0x008c);
           MPL_WRITE32(pMplCtx, PAGE, 0x0);
        }
        else if (((pMplCtx->srr & 0xFF00) == 0x0400) ||
                   ((pMplCtx->srr & 0xFF00) == 0x0500))
             {
                // MacPhy-2
                MPL_WRITE32(pMplCtx, PAGE, 0x0001);
                MPL_WRITE32(pMplCtx, PMDCSR , 0x189C );
                MPL_WRITE32(pMplCtx, PAGE, 0x0);
             } 

   return;
}

//*****************************************************************************
//   atanPatchUp
//      Apply ATAN switch link-up patch
//
//   Parameters
//      pMplCtx
//         MPL device context
//
//   Return Value
//      None
//
//*****************************************************************************
static 
NS_VOID
   atanPatchUp(
      IN MPL_CONTEXT *pMplCtx
      )
{
   MPL_WRITE32(pMplCtx, PAGE, 0x0001);
   MPL_WRITE32(pMplCtx, DSPTST, 0x80b7);
   MPL_WRITE32(pMplCtx, PAGE, 0x0);

   return;
}

//*****************************************************************************
//   atanPatchDown
//      Remove ATAN switch link-up patch
//
//   Parameters
//      pMplCtx
//         MPL device context
//
//   Return Value
//      None
//
//*****************************************************************************
static 
NS_VOID
   atanPatchDown(
      IN MPL_CONTEXT *pMplCtx
      )
{
   MPL_WRITE32(pMplCtx, PAGE, 0x0001);
   MPL_WRITE32(pMplCtx, DSPTST, 0x00b7);
   MPL_WRITE32(pMplCtx, PAGE, 0x0);

   return;
}
#endif //MPL_EXTERNAL_PHY


//*****************************************************************************
//   mdioIdle
//      MDIO clk delay
//
//   Parameters
//      pMplCtx
//         MPL device context
//
//   Return Value
//      None
//
//*****************************************************************************
static 
NS_VOID
   mdioIdle(
      IN MPL_CONTEXT *pMplCtx
      )
{
   OaiSleep(1);
   MPL_WRITE32(pMplCtx, MEAR,  MDIO | MDDIR);
   OaiSleep(1);
   MPL_WRITE32(pMplCtx, MEAR,  MDIO | MDDIR | MDCLK);
   return;
}

//*****************************************************************************
//   mdioSync
//      MDIO bus sync
//
//   Parameters
//      pMplCtx
//         MPL device context
//
//   Return Value
//      None
//
//*****************************************************************************
static 
NS_VOID
   mdioSync(
      IN MPL_CONTEXT *pMplCtx
      )
{
   NS_UINT32 i;

   // Write 32 logic ones
   for (i = 0x0; i < 32; i++)
      mdioIdle(pMplCtx);

   OaiSleep(1);
   MPL_WRITE32(pMplCtx, MEAR,  MDIO | MDDIR);
   return;
}

#ifdef MPL_EXTERNAL_PHY
//*****************************************************************************
//   mdioReadReg
//      Read a Phy register via MDIO
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to read (in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//
//   Return Value
//     Reg data
//
//*****************************************************************************
NS_UINT16
   mdioReadReg(
      IN MPL_CONTEXT *pMplCtx,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex
      )
{
   NS_UINT32 mdioCmd;
   NS_UINT32 oData = 0x0, iData, i;

   // Create the read cmd seq
   mdioCmd = MII_READ | (phyDevAddr << MII_ADDR_OFF) |
                   (regIndex << MII_REG_OFF);

   // Sync up the bus
   mdioSync(pMplCtx);
 
   // Send Start + Read Cmd + Phy Addr + Reg Offset
   for (i = 0x0; i < 15 ; i++)
   {
      if (mdioCmd & 0x80000000)
      {
         // Write a 1-bit
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR,  (MDIO | MDDIR));// Clk Low,Data High
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR,  (MDIO | MDCLK | MDDIR));//Clk Hi,Data High
      }
      else
      {
         // Write a 0-bit
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR, MDDIR);// Clk Low, Data Low
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR, (MDCLK | MDDIR));// Clk Hi, Data Low
      }
      
      // Next bit
      mdioCmd <<= 1;
   }

   // Read-in data
   for (i = 0x0; i < 16; i++)
   {
      OaiSleep(1);
      MPL_WRITE32(pMplCtx, MEAR, MDIO); // Clock Low, Data High
      OaiSleep(1);
      MPL_WRITE32(pMplCtx, MEAR, (MDIO | MDCLK)); // Clock High, Data High
      iData = MPL_READ32(pMplCtx, MEAR);
      oData <<= 1;
      if (iData & MDIO)
         oData |= 1;
   }

   // Write an idle bit
   mdioIdle(pMplCtx);
   OaiSleep(1);
   MPL_WRITE32(pMplCtx, MEAR,  MDIO | MDDIR);

   return (NS_UINT16)oData;
}

//*****************************************************************************
//   mdioWriteReg
//      Write Phy register
//
//   Parameters
//      pMplHandle
//         MPL device handle
//      phyDevAddr
//         Device addr of the Phy
//      regIndex
//         Register to write(in terms of MII offset e.g. BMCR = 0x0, BMSR = 0x1)
//      regData
//         Data to write
//
//   Return Value
//     None
//
//*****************************************************************************
NS_VOID
   mdioWriteReg(
      IN MPL_CONTEXT *pMplCtx,
      IN NS_UINT    phyDevAddr,
      IN NS_UINT    regIndex,
      IN NS_UINT16  regData
      )
{
   NS_UINT32 mdioCmd, i;

   // Create the read cmd seq
   mdioCmd = MII_WRITE | (phyDevAddr << MII_ADDR_OFF) | 
                 (regIndex << MII_REG_OFF) | regData;

   // Sync up the bus
   mdioSync(pMplCtx);
 
   // Send Start + Write Cmd + Phy Addr + Reg Offset + Reg Data
   for (i = 0x0; i < 32; i++)
   {
      if (mdioCmd & 0x80000000)
      {
         // Write a 1-bit
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR,  (MDIO | MDDIR));// Clk Low,Data High
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR,  (MDIO | MDCLK | MDDIR));//Clk Hi,Data High
      }
      else
      {
         // Write a 0-bit
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR, MDDIR);// Clk Low, Data Low
         OaiSleep(1);
         MPL_WRITE32(pMplCtx, MEAR, (MDCLK | MDDIR));// Clk Hi, Data Low
      }
      
      // Next bit
      mdioCmd <<= 1;
   }

   // Write an idle bit
   mdioIdle(pMplCtx);
   OaiSleep(1);
   MPL_WRITE32(pMplCtx, MEAR,  MDIO | MDDIR);

   return;
}
#endif // MPL_EXTERNAL_PHY
