
//******************************************************************************
//
//  MPLPWR.C
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  MPL Power Management Module.
//
//  This file contains the API implementations for
//     o Configuring the power state on the device
//     o WoL related configuration and status reporting
//
//******************************************************************************
#include <mplinternal.h>

// Local helful debug macros
#undef ENTER
#undef EXIT
#undef ASSERT
#undef PRINT
#undef PRINTI
#undef PRINTS
#undef TBREAK

#define ENTER(fn) MPL_CENTER(DZONE_MPL_PWR, fn)
#define EXIT(fn) MPL_CEXIT(DZONE_MPL_PWR, fn)
#define ASSERT(le) MPL_CASSERT(DZONE_MPL_PWR, le)
#define PRINT(fmt) MPL_CTRACE(DZONE_MPL_PWR, fmt)
#define PRINTI(v) MPL_CTRACE(DZONE_MPL_PWR, (" "#v": %x \n",(NS_UINT)(v)))
#define PRINTS(v) MPL_CTRACE(DZONE_MPL_PWR, (" "#v": %s \n", v))
#define TBREAK(fmt) MPL_CTRACEBREAK(DZONE_MPL_PWR, fmt)

// Locals
static NS_VOID cfgWakeEvts (MPL_CONTEXT  *pMplCtx);
static NS_VOID cfgWakePatterns(MPL_CONTEXT  *pMplCtx);
#ifdef LINUX_POWER_DEBUG
static NS_VOID printWakeReason(MPL_CONTEXT  *pMplCtx);
#endif

//*****************************************************************************
//   MplPowerSetState
//      Set a new power state on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      newState
//         New power state that needs to be set on the device
//      pmeEnable
//         When set to NS_TRUE, MPL enables PME on the device
//         Some OSs do this automatically, some do not
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The new state was successfully set
//      NS_STATUS_FAILURE
//         The device could not be set to the new state
//
//*****************************************************************************
MPL_STATUS
   MplPowerSetState(
      IN NS_VOID         *pMplHandle,
      IN MPL_POWER_STATE  newState,
      IN NS_BOOLEAN       pmeEnable
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   MPL_STATUS status = NS_STATUS_SUCCESS;

   ENTER(MplPowerSetState);

   if ((pMplCtx->pwrState == MPL_POWER_STATE_HIGH) && 
          (newState == MPL_POWER_STATE_LOW))
   {
      // Entering Low Power state from a High Power state
      //  This function expects Int to be disabled and Rx Reset Complete
      //  before calling it
      
      // Take a register snap-shot
      setSoftRegs(pMplHandle, MPL_REGS_CURRENT);

      // Configure Wake events
      cfgWakeEvts(pMplCtx);
      
      // Configure Wake Patterns if any
      cfgWakePatterns(pMplCtx);
      
      // Put the receiver in "Silent Receive" mode
      MPL_WRITE32(pMplCtx, CR, RX_DIS);
      MPL_WRITE32(pMplCtx, RXDP, 0x0);
      MPL_WRITE32(pMplCtx, CR, RX_EN);
      
      // Enable PME if NSM wants us to - This depends on the OS, some do
      // it themselves, for the rest we do it
      if (pmeEnable == NS_TRUE)
      {
         // Using the PME backdoor
         if (MPL_READ32(pMplCtx, WCSR) & WAKEON_MASK)
         {
            // WOL event is enabled - So set PMEEN
            MPL_WRITE32(pMplCtx, CCSR, pMplCtx->ccsrReg | PMEEN | PMESTS);
         }
	 else
         {
            // Write whatever was there by default 
	    //  Note PMEEN bit is loaded from EEPROM
            MPL_WRITE32(pMplCtx, CCSR, pMplCtx->ccsrReg);
         }
      }

      // Set the new state - LOW
      pMplCtx->pwrState = MPL_POWER_STATE_LOW;
   }
   else if ((pMplCtx->pwrState == MPL_POWER_STATE_LOW) && 
               (newState == MPL_POWER_STATE_HIGH))
        {
           // Entering High Power state from a Low Power state
#ifdef LINUX_POWER_DEBUG
           printWakeReason(pMplCtx);
#endif 

           // If PME was enabled then clear status
           if (pmeEnable == NS_TRUE)
           {
              // Clear any wake events configured on the device
              MPL_WRITE32(pMplCtx, CCSR, PMESTS);
           }

           // Not doing MAC/PHY Resets

           // Set registers with last known snap-shot
           setHardRegs(pMplCtx);

           // Set the MAC address - to last know value
           MplSetMacAddress(pMplCtx, pMplCtx->permMacAddr);
      
#ifdef MPL_TASK_RECEIVE_FILTER
           // Reload Receive hash filter (Multicast and Unicast)
           MplTaskFilterReload(pMplCtx);
#endif // MPL_TASK_RECEIVE_FILTER

           // Notify the device about the head node on the Rxd ring
           // Not enabling RX yet, the replenish API will take care of it
           MPL_WRITE32(pMplCtx, RXDP,  pMplCtx->rxQueue.ringPA);

           // Notify the device about the head node on the Txd ring
           MPL_WRITE32(pMplCtx, TXDP,  pMplCtx->txQueue[0].ringPA);
           MPL_WRITE32(pMplCtx, TXDP1, pMplCtx->txQueue[1].ringPA);
           MPL_WRITE32(pMplCtx, TXDP2, pMplCtx->txQueue[2].ringPA);
           MPL_WRITE32(pMplCtx, TXDP3, pMplCtx->txQueue[3].ringPA);

           // Set the new state - HIGH
           pMplCtx->pwrState = MPL_POWER_STATE_HIGH;
        }

   EXIT(MplPowerSetState);
   return status;
}

//*****************************************************************************
//   MplWolCfg
//     Configure the WOL operation on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      wakeEnable
//         Flag to enable (set to NS_TRUE) or disable (set to NS_FALSE) Wol
//      wakeType
//         When WOL is enabled, represents a bit-map of wake-up events that
//         needs to be enabled. It is a logical OR of,
//          MPL_WOL_MAGIC - Enable wake on Magic Pattern.
//          MPL_WOL_PATTERN - Enable wake on pattern match (Patterns needs
//                            to be separately defined).
//          MPL_WOL_BROADCAST - Enable wake on any broadcast packet.
//          MPL_WOL_MULTICAST - Enable wake on multicast packet.
//          MPL_WOL_DIRECTED - Enable wake on directed (unicast) packet to 
//                             the device's address.
//          MPL_WOL_LINK - Enable wake on link status change.
//          MPL_WOL_ARP - Enable wake on any ARP packet.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The WOL events were successfully enabled or disabled (depending
//         on wakeEnable).
//      NS_STATUS_INVALID_PARM
//         The wake event type is not supported on the device.
//
//*****************************************************************************
MPL_STATUS
   MplWolCfg(
      IN NS_VOID     *pMplHandle,
      IN NS_BOOLEAN   wakeEnable,
      IN NS_UINT      wakeType
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;
   MPL_STATUS status = NS_STATUS_SUCCESS;

   ENTER(MplWolCfg);

   pMplCtx->pwrWakeType = 0x0;
   if (wakeEnable == NS_TRUE)
   {
      // Enable WOL - Make sure wake evt is one that we support
      if (wakeType & ~(MPL_WOL_MAGIC     |
                       MPL_WOL_PATTERNS  |
                       MPL_WOL_BROADCAST | 
                       MPL_WOL_MULTICAST | 
                       MPL_WOL_DIRECTED  | 
                       MPL_WOL_LINK      | 
                       MPL_WOL_ARP))
      {
         status = NS_STATUS_INVALID_PARM;
      }
      else
      {
         pMplCtx->pwrWakeType = wakeType;
      }
   }

   EXIT(MplWolCfg);
   return status;
}

//*****************************************************************************
//   MplWolGetCfg
//     Retrieve the current WOL configuration on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWakeType
//         Pointer to caller provider variable in which the bit-map of wake-up
//         events currently enabled is returned. It is a logical OR of,
//          MPL_WOL_MAGIC - Enable wake on Magic Pattern.
//          MPL_WOL_PATTERN - Enable wake on pattern match (Patterns needs to
//                            be separately defined).
//          MPL_WOL_BROADCAST - Enable wake on any broadcast packet.
//          MPL_WOL_MULTICAST - Enable wake on multicast packet.
//          MPL_WOL_DIRECTED - Enable wake on directed (unicast) packet to
//                             the device's address.
//          MPL_WOL_LINK - Enable wake on link status change.
//          MPL_WOL_ARP - Enable wake on any ARP packet.
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The WOL configuration was returned successfully
//
//*****************************************************************************
MPL_STATUS
   MplWolGetCfg(
      IN NS_VOID   *pMplHandle,
      IN NS_UINT   *pWakeType
      )
{
   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplWolGetCfg);

   *pWakeType = pMplCtx->pwrWakeType;

   EXIT(MplWolGetCfg);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplWolClearPattern
//     Clear all WOL wake patterns on the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake patterns were successfully cleared
//
//*****************************************************************************
MPL_STATUS
   MplWolClearPattern(
      IN NS_VOID   *pMplHandle
      )
{
//   MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplWolClearPattern);

   EXIT(MplWolClearPattern);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplWolAddPattern
//     Add a new WOL wake pattern to the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWolPattern
//         Pointer to a MPL_WOL_PATTERN structure in which the new pattern 
//         and its associated mask is noted
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake pattern was successfully added.
//      NS_STATUS_RESOURCES
//         The wake pattern not added since it exceeds the maximum supported
//         on this device.
//      NS_STATUS_INVALID_PARM
//         The wake pattern buffer is larger than the maximum supported by
//         the device.
//
//*****************************************************************************
MPL_STATUS
   MplWolAddPattern(
      IN NS_VOID         *pMplHandle,
      IN MPL_WOL_PATTERN *pWolPattern
      )
{
 //  MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplWolAddPattern);

   EXIT(MplWolAddPattern);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   MplWolRemovePattern
//     Remove a WOL wake pattern from the device
//
//   Parameters
//      pMplHandle
//         MPL device handle returned following a call to MplInitialize
//      pWolPattern
//         Pointer to a MPL_WOL_PATTERN structure in which the pattern 
//         and its associated mask is noted
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The wake pattern was deleted successfully.
//      NS_STATUS_INVALID_PARM
//         The wake pattern was not found in the list
//
//*****************************************************************************
MPL_STATUS
   MplWolRemovePattern(
      IN NS_VOID         *pMplHandle,
      IN MPL_WOL_PATTERN *pWolPattern
      )
{
 // MPL_CONTEXT *pMplCtx = (MPL_CONTEXT *)pMplHandle;

   ENTER(MplWolRemovePattern);

   EXIT(MplWolRemovePattern);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   cfgWakeEvts
//      Configures the wakes events on the chip
//
//   Parameters
//      pMplCtx
//         Pointer to MPL Context
//
//   Return Value
//      None 
//
//*****************************************************************************
static
NS_VOID
   cfgWakeEvts (
      IN MPL_CONTEXT  *pMplCtx
      )
{
   NS_UINT32 wcsrReg = 0x0, rfcrReg = 0x0;

   // Disable Rx Filter
   MPL_WRITE32(pMplCtx, RFCR, 0x0);

   if (pMplCtx->pwrWakeType & MPL_WOL_MAGIC)
   {
      // Wake on Magic Packet
      wcsrReg |= WAKEON_MAGICPKT;
      rfcrReg |= ACCEPT_ALLBCAST | ACCEPT_PERFECTMATCH;
   }

   if (pMplCtx->pwrWakeType & MPL_WOL_BROADCAST)
   {
      // Wake on Any Broadcast packet
      wcsrReg |= WAKEON_BROADCAST;
      rfcrReg |= ACCEPT_ALLBCAST;
   }

   if (pMplCtx->pwrWakeType & MPL_WOL_MULTICAST)
   {
      // Wake on Any Multicast packet
      wcsrReg |= WAKEON_MULTICAST;
      rfcrReg |= ACCEPT_ALLMCAST;
   }

   if (pMplCtx->pwrWakeType & MPL_WOL_DIRECTED)
   {
      // Wake on Unicast packet addressed to us
      wcsrReg |= WAKEON_UNICAST;
      rfcrReg |= ACCEPT_PERFECTMATCH;
   }

   if (pMplCtx->pwrWakeType & MPL_WOL_LINK)
   {
      // Wake on Link change
      wcsrReg |= WAKEON_PHYSTSCHNG;
   }

   if (pMplCtx->pwrWakeType & MPL_WOL_ARP)
   {
      // Wake on Any ARP packet
      wcsrReg |= WAKEON_ARPPKT;
      rfcrReg |= ACCEPT_ALLBCAST | ACCEPT_ALLARP;
   }

   // Tell the chip
   MPL_WRITE32(pMplCtx, RFCR, rfcrReg | RXFLTR_EN);
   MPL_WRITE32(pMplCtx, WCSR, wcsrReg); 

   return;
}

//*****************************************************************************
//   cfgWakePatterns
//      Configures the wakes patterns on the chip
//
//   Parameters
//      pMplCtx
//         Pointer to MPL Context
//
//   Return Value
//      None 
//
//*****************************************************************************
static
NS_VOID
   cfgWakePatterns(
      IN MPL_CONTEXT  *pMplCtx
      )
{
// Make sure to update WCSR (do a read and then add pattern codes)
// Restore rfcr from pMplCtx->rfcrReg since the original RFCR contents
//   now hold on their own i.e. no Evt specific RFCR
   return;
}

#ifdef LINUX_POWER_DEBUG
//*****************************************************************************
//   printWakeReason
//      Prints why we woke up
//
//   Parameters
//      pMplCtx
//         Pointer to MPL Context
//
//   Return Value
//      None 
//
//*****************************************************************************
static
NS_VOID
   printWakeReason(
      IN MPL_CONTEXT  *pMplCtx
      )
{
   NS_UINT32 wcsr;

   // Disable Rx Filter
   wcsr = MPL_READ32(pMplCtx, WCSR);

   if (wcsr & WAKEON_MAGICPKT)
   {
      // Wake on Magic Packet
      printk("MPL Woke on Magic Packet \n");
   }

   if (wcsr & WAKEON_BROADCAST)
   {
      // Wake on Any Broadcast packet
      printk("MPL Woke on Broadcast Packet \n");
   }

   if (wcsr & WAKEON_MULTICAST)
   {
      // Wake on Any Multicast packet
      printk("MPL Woke on Multicast Packet \n");
   }

   if (wcsr & WAKEON_UNICAST)
   {
      // Wake on Unicast packet addressed to us
      printk("MPL Woke on Unicast Packet \n");
   }

   if (wcsr & WAKEON_PHYSTSCHNG)
   {
      // Wake on Link change
      printk("MPL Woke on Link Change \n");
   }

   if (wcsr & WAKEON_ARPPKT)
   {
      // Wake on Any ARP packet
      printk("MPL Woke on ARP Packet \n");
   }

   if (wcsr & WAKEON_PATRN0)
   {
      // Wake on Pattern Match
      printk("MPL Woke on Pattern Match-0\n");
   }

   if (wcsr & WAKEON_PATRN1)
   {
      // Wake on Pattern Match
      printk("MPL Woke on Pattern Match-1\n");
   }

   if (wcsr & WAKEON_PATRN2)
   {
      // Wake on Pattern Match
      printk("MPL Woke on Pattern Match-2\n");
   }

   if (wcsr & WAKEON_PATRN3)
   {
      // Wake on Pattern Match
      printk("MPL Woke on Pattern Match-3\n");
   }

   return;
}
#endif
