
//******************************************************************************
//
//  NSMDIAG.C
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  NSM Adapter Diagostics Interface API
//
//   OpMode Combinations
//   ==========================================================
//  |          ||  NORMAL   |    LOAD_ONLY    |    MONITOR     |
//  | Op  Mode ||(non-Diag) |   (Diag Mode)   |    (Diag Mode) |
//   ==========================================================
//  | Operation||             O  W  N  E  R                    |
//  |----------------------------------------------------------
//  | H/w Init || NSM       | Diag App        |      NSM       |
//    ---------------------------------------------------------
//  | Tx/Rx    || NSM       | Diag App        | Diag App       |
//   ==========================================================
//
//  This file contains the API implementations for
//     o Adapter Txd and Rxd Setup - Diag Mode
//     o Packet Tx 
//     o Packet Rx 
//     o Cmd demultiplexer
//
//******************************************************************************

#include <nsmos.h>
#include <nsm.h>

#ifdef NSM_DIAG_MODE

// Local functions
static NS_BOOLEAN setupTxFrags(NSM_CONTEXT *adapter);
static NS_VOID freeTxFrags(NSM_CONTEXT *adapter);
static NS_BOOLEAN pktSetup(NSM_CONTEXT *adapter);
static NS_VOID pktFree(NSM_CONTEXT *adapter);
static NS_UINT
   txPkt( NSM_CONTEXT *adapter, NS_APP_XFER_CMD *txCmd, NS_UINT8 *txData);
static NS_BOOLEAN
   checkAvailTxFrag( NSM_CONTEXT *adapter, NS_UINT reqFrags);
static NS_UINT pktStatus(
       NSM_CONTEXT    *adapter, MPL_LIST *pList,
       MPL_PKT_FRAG  **ppTailFrag, spinlock_t *pLock,
       NS_APP_XFER_CMD *pCmd, NS_UINT8 *pData);
static NS_VOID pktValidate(NSM_CONTEXT *adapter, MPL_PKT_FRAG *pHeadFrag);
static NS_VOID flushTx(NSM_CONTEXT *adapter);
static NS_VOID flushRx(NSM_CONTEXT *adapter);
static NS_VOID getDiagStats(NSM_CONTEXT *adapter, NS_APP_STATS_CMD *pCmd);
static NS_UINT xferCfgSet( NSM_CONTEXT *adapter, NS_APP_XFER_CFG_CMD *pCmd);
static NS_UINT xferCfgGet( NSM_CONTEXT *adapter, NS_APP_XFER_CFG_CMD *pCmd);

//*****************************************************************************
//   NsmDiagInitialize
//     Diag module initialization
//
//   Parameters
//      adapter
//         NSM Context
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The configurations were successfully applied
//      NS_STATUS_RESOURCES
//         Failed to allocate required resources
//*****************************************************************************
NS_UINT
   NsmDiagInitialize(
      NSM_CONTEXT *adapter)
{
   NI(NsmDiagInitialize);   

   if (adapter->opMode != MPL_MODE_LOAD_ONLY)
   {
      // Setup the free MPL frag list (for Tx)
      if (setupTxFrags(adapter) != NS_TRUE)
      {
         NO(NsmDiagInitialize);
         return NS_STATUS_RESOURCES;
      }

      // Setup the Diag packet list (for both Tx and Rx use)
      if (pktSetup(adapter) != NS_TRUE)
      {
         freeTxFrags(adapter);
         pktFree(adapter);
         NO(NsmDiagInitialize);
         return NS_STATUS_RESOURCES;
      }
   }
   else
   {
      // Only initialize the lists - The population of the list
      //   happens when the app calls xferCfg
      MplListInit(&adapter->diagFreeList, NS_FALSE);
      MplListInit(&adapter->diagTxDoneList, NS_FALSE);
      MplListInit(&adapter->diagRxDoneList, NS_FALSE);
   }

   NO(NsmDiagInitialize);
   return NS_STATUS_SUCCESS;
}

//*****************************************************************************
//   NsmDiagUnInitialize
//     Diag module uninitialization
//
//   Parameters
//      adapter
//         NSM Context
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
   NsmDiagUnInitialize(
      NSM_CONTEXT *adapter)
{

   NI(NsmDiagUnInitialize);   

   // Free the Diag Pkt cache
   pktFree(adapter);

   // Free the MPL_FRAGs cache for Tx
   freeTxFrags(adapter);

   NO(NsmDiagUnInitialize);   
   return;
}

//*****************************************************************************
//   NsmDiagIoctl
//     Diag Cmd Multiplexer
//
//   Parameters
//      adapter
//         NSM Context
//      userAddr
//         Pointer to user buffer (Would need OS specific tweaking) 
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The configurations were successfully applied
//      NS_STATUS_INVALID_PARM
//         An invalid parameter value was detected
//      NS_STATUS_RESOURCES
//         Failed to allocate required resources
//      NS_STATUS_NOT_SUPPORTED
//         Operation not supported
//      NS_STATUS_FAILURE
//         Operation failed
//*****************************************************************************
NS_UINT
   NsmDiagIoctl(
      NSM_CONTEXT *adapter, 
      NS_VOID     *useraddr)
{
   NS_UINT ret = NS_STATUS_SUCCESS;
   NS_APP_REG_CMD regCmd;
   NS_APP_INTRO_CMD introCmd;
   NS_APP_XFER_CMD xferCmd;
   NS_APP_XFER_CFG_CMD cfgCmd;
   NS_APP_STATS_CMD statCmd;
   NS_APP_CMD baseCmd;
   NS_UINT cmd;

   NI(NsmDiagIoctl);

   // FM: Make all user access funcations as abstracted APIs

   if (get_user(cmd, (NS_UINT32 *)useraddr))
   {
      NO(NsmDiagIoctl);
      return NS_STATUS_FAILURE;;
   }

   // Handle basic Reg read/write kind of requests here
   //  This is available in all modes
   switch (cmd) 
   {
      case NSM_APP_INTRO:
                         if (copy_from_user(&introCmd, useraddr, 
                                                         sizeof(introCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            if (introCmd.app == NS_APP_SIGN)
                            {
                               introCmd.driver = NS_DRIVER_SIGN;
                               introCmd.numPorts = 0x01;
                               introCmd.driverVerMajor = DP_driverVersionMaj;
                               introCmd.driverVerMinor = DP_driverVersionMin;
                            }
                           
                            if (copy_to_user(useraddr, &introCmd, 
                                                      sizeof(introCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }
                         break;

      case NSM_REG_READ:
                         if (copy_from_user(&regCmd, useraddr, 
                                                        sizeof(regCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            regCmd.data = MplRegRead(adapter->mplContext, 
                                                           regCmd.offset);
                            if (copy_to_user(useraddr, &regCmd, 
                                                      sizeof(regCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }
                         break;

      case NSM_REG_WRITE:
                         if (copy_from_user(&regCmd, useraddr, 
                                                         sizeof(regCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            MplRegWrite(adapter->mplContext, regCmd.offset,
                                                      regCmd.data);
                         }
                         break;

      case NSM_PCI_READ:
                         if (copy_from_user(&regCmd, useraddr, 
                                                        sizeof(regCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            pci_read_config_dword(adapter->pdev, regCmd.offset,
                                                  (unsigned int *)&regCmd.data);
                            if (copy_to_user(useraddr, &regCmd, 
                                                      sizeof(regCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }
                         break;

      case NSM_PCI_WRITE:
                         if (copy_from_user(&regCmd, useraddr, 
                                                         sizeof(regCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            pci_write_config_dword(adapter->pdev, 
                                                    regCmd.offset, regCmd.data);
                         }
                         break;
      default :         
                          ret = NS_STATUS_NOT_SUPPORTED;
   }

   // Check if the request is for a Tx/Rx related operation
   // Note this is only available when we are in one of the two debug modes
   if (adapter->opMode != MPL_MODE_NORMAL && ret == NS_STATUS_NOT_SUPPORTED)
   {
      ret = NS_STATUS_SUCCESS;

      switch (cmd) 
      {
         case NSM_XFER_CFG_SET:
                         if (copy_from_user(&cfgCmd, useraddr, 
                                                         sizeof(cfgCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            // Call cfg handler
                            cfgCmd.status = xferCfgSet(adapter, &cfgCmd);

                            // Copy results back
                            if (copy_to_user(useraddr, &cfgCmd, 
                                                      sizeof(cfgCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }

                         break;

         case NSM_XFER_CFG_GET:
                         if (copy_from_user(&cfgCmd, useraddr, 
                                                         sizeof(cfgCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            // Call cfg handler
                            cfgCmd.status = xferCfgGet(adapter, &cfgCmd);

                            // Copy results back
                            if (copy_to_user(useraddr, &cfgCmd, 
                                                      sizeof(cfgCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }

                         break;

         case NSM_TRANSMIT:
                         if (copy_from_user(&xferCmd, useraddr, 
                                                         sizeof(xferCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            // Call Tx pkt handler
                            xferCmd.status = 
                                txPkt(adapter, &xferCmd, 
                                      (NS_UINT8 *)useraddr + sizeof(xferCmd));

                            // Copy results back
                            if (copy_to_user(useraddr, &xferCmd, 
                                                      sizeof(xferCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }

                         break;

         case NSM_TRANSMIT_STATUS:
                         if (copy_from_user(&xferCmd, useraddr, 
                                                         sizeof(xferCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            // Call pkt handler
                            xferCmd.status = 
                              pktStatus(adapter, &adapter->diagTxDoneList, 
                                        &adapter->txFragsTail, &adapter->txLock,
                                        &xferCmd, 
                                        (NS_UINT8 *)useraddr + sizeof(xferCmd));

                            // Copy results back
                            if (copy_to_user(useraddr, &xferCmd, 
                                                      sizeof(xferCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }

                         break;

         case NSM_TRANSMIT_FLUSH:
                         if (copy_from_user(&baseCmd, useraddr, 
                                                         sizeof(baseCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            flushTx(adapter);
                         }
                         break;

         case NSM_RECEIVE:
                         if (copy_from_user(&xferCmd, useraddr, 
                                                         sizeof(xferCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            // Call pkt handler
                            xferCmd.pktHandle = 0x0;
                            xferCmd.status = 
                             pktStatus(adapter, &adapter->diagRxDoneList,
                                       &adapter->rxFragsTail, &adapter->rxLock,
                                       &xferCmd,
                                       (NS_UINT8 *)useraddr + sizeof(xferCmd));

                            // Copy results back
                            if (copy_to_user(useraddr, &xferCmd, 
                                                      sizeof(xferCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }

                         break;

         case NSM_RECEIVE_FLUSH:
                         if (copy_from_user(&baseCmd, useraddr, 
                                                         sizeof(baseCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            flushRx(adapter);
                         }
                         break;

         case NSM_STATS:
                         if (copy_from_user(&statCmd, useraddr, 
                                                        sizeof(statCmd)))
                         {
                            ret = NS_STATUS_FAILURE;
                         }
                         else
                         {
                            getDiagStats(adapter, &statCmd);
                            if (copy_to_user(useraddr, &statCmd, 
                                                      sizeof(statCmd)))
                            {
                               ret = NS_STATUS_FAILURE;
                            }
                         }
                         break;

         default :         
                          ret = NS_STATUS_NOT_SUPPORTED;
      }
   }

   NO(NsmDiagIoctl);
   return ret;
}


//*****************************************************************************
//   NsmDiagTransmitDone
//     Transmit Done Notification  (Diag Mode)
//
//   Parameters
//      pClientDevHndl
//         NSM device handle registered with MPL (during MplInitialize)
//      packetCnt
//         Count of packets whose completion is being reported.
//      pTxDone
//         MPL allocated MPL_TRANSMIT_DONE structure array in which the 
//          NSM packet handles and statuses are returned.
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
NsmDiagTransmitDone(
    NS_VOID *pClientDevHndl,
    NS_UINT  packetCnt,
    MPL_TRANSMIT_DONE *pTxDone
    )
{
   NSM_CONTEXT *adapter = pClientDevHndl;
   NS_UINT i, pktSize, fragCnt;
   DIAG_PKT *dPkt;
   MPL_LIST_NODE *node;
   MPL_PKT_FRAG *pFrag, *pHeadFrag, *pTailFrag;

   NI(NsmDiagTransmitDone);   

   // Unmap and Free done skbs
   for(i = 0x0; i < packetCnt; i++)
   {
      // Get the head frag which was noted as NSM private
      pHeadFrag = pTxDone[i].pNsmPrivate;

      // Unmap Buffers
      pFrag = pHeadFrag; pTailFrag = NULL;
      pktSize = 0x0; fragCnt = 0x0;
      while (pFrag)
      {
         pci_unmap_single(adapter->pdev, pFrag->physAddr, pFrag->fragSize,
                            PCI_DMA_TODEVICE);
    
         pktSize += pFrag->fragSize;
         fragCnt++;

         // Get next frag
         pTailFrag = pFrag;
         pFrag = pFrag->pNextFrag;
      }

      // Update stats
      if (pTxDone[i].packetStatus == NS_STATUS_SUCCESS)
      {
         // Update counters
         adapter->diagStats.txPackets++;
         adapter->diagStats.txBytes += pktSize;
      }
      else
      {

         // Update counters
         adapter->diagStats.txErrors++;

         // Detailed Error analysis 
         if (pTxDone[i].cmdSts & CS_ERR_TXA)
         {
            adapter->diagStats.txAbortedErrors++;
         }

         if (pTxDone[i].cmdSts & CS_ERR_TX_EC)
         {
            adapter->diagStats.txCollErrors++;
         }

         if (pTxDone[i].cmdSts & CS_ERR_TFU)
         {
            adapter->diagStats.txFifoErrors++;
         }
         
         if (pTxDone[i].cmdSts & CS_ERR_TX_CRS)
         {
            adapter->diagStats.txCarrierErrors++;
         }

         if (pTxDone[i].cmdSts & CS_ERR_TX_OWC)
         {
            adapter->diagStats.txWindowErrors++;
         }
      }

      if (pTxDone[i].packetStatus != NS_STATUS_ABORTED)
      {
         // If we are are out of free pkt nodes to note this completion
         //  then force a stale one out of the completed list
         if (MPL_LIST_GET_SIZE(&adapter->diagFreeList) == 0x0)
         {
            // Ask ProcessDone function to do a basic validation before freeing
            NsmDiagProcessDoneList(adapter, 0x1, NS_TRUE,
                          &adapter->diagTxDoneList, &adapter->txFragsTail);
            adapter->diagStats.txDropped++;
         }

         // Get a free diag pkt node from the head of the free list
         node = MPL_LIST_GET_HEAD(&adapter->diagFreeList); 

         // Get the Diag Pkt linked to the curr node
         dPkt = MPL_LIST_GET_ENTRY(node, DIAG_PKT, link);

         // Note pkt details
         dPkt->pktStatus = pTxDone[i].packetStatus;
         dPkt->cmdSts = pTxDone[i].cmdSts;
         dPkt->pktSize = pktSize;
         dPkt->appPktHandle = (NS_UINT)pHeadFrag->pPacketHndl;
         dPkt->numDesc = fragCnt;

         // NSM Diag related
         dPkt->pFrag = pHeadFrag;

         // Move this packet from the free list to the done list
         MplListDel(&adapter->diagFreeList, &dPkt->link);
         MplListAddTail(&adapter->diagTxDoneList, &dPkt->link);
      }
      else
      {
         // Add the frags back to the free list
         adapter->txFragsTail->pNextFrag = pHeadFrag;
         adapter->txFragsTail = pTailFrag;
      }
   }

   NO(NsmDiagTransmitDone);
   return;
}


//*****************************************************************************
//   NsmDiagReceive
//     Diag mode Receive Done Notification
//
//   Parameters
//      pClientDevHndl
//         NSM device handle registered with MPL (during MplInitialize)
//      packetCnt
//         Count of packets whose reception is being reported.
//      pPkt
//         MPL allocated MPL_PKT structure array in which 
//         a list of the newly received packets are reported
//
//   Return Value
//      None
//*****************************************************************************
NS_VOID
NsmDiagReceive(
    NS_VOID *pClientDevHndl,
    NS_UINT  packetCnt,
    MPL_PKT *pPkt
    )
{
   NSM_CONTEXT *adapter = pClientDevHndl;
   MPL_PKT_FRAG *pTailFrag, *pHeadFrag;
   DIAG_PKT *dPkt;
   MPL_LIST_NODE *node;
   NS_UINT i, j;

   NI(NsmDiagReceive);

   // Process the receive list
   for(i = 0x0; i < packetCnt; i++)
   {
      // Get the head frag ptr
      pHeadFrag = pPkt[i].pFragHead;

      if (pPkt[i].packetStatus == NS_STATUS_SUCCESS)
      {
         // Update counters
         adapter->diagStats.rxPackets++;
         adapter->diagStats.rxBytes += pPkt[i].packetSize;
      }
      else
      {
         // Update counters
         adapter->diagStats.rxErrors++;

         // Detailed Error analysis
         if (pPkt[i].rxOOB.cmdSts & (CS_RXA | CS_ERR_RXO))
         {
            adapter->diagStats.rxOverErrors++;
         }

         if (pPkt[i].rxOOB.cmdSts & CS_ERR_RX_CRCE)
         {
            adapter->diagStats.rxCrcErrors++;
         }

         if (pPkt[i].rxOOB.cmdSts & (CS_ERR_RX_LONG | CS_ERR_RX_RUNT))
         {
            adapter->diagStats.rxLengthErrors++;
         }

         if (pPkt[i].rxOOB.cmdSts & (CS_ERR_RX_ISE | CS_ERR_RX_FAE))
         {
            adapter->diagStats.rxFrameErrors++;
         }
      }

      if (pPkt[i].packetStatus != NS_STATUS_ABORTED)
      {
         // If we are are out of free pkt nodes to note this completion
         //  then force a stale one out of the completed list
         if (MPL_LIST_GET_SIZE(&adapter->diagFreeList) == 0x0)
         {
            // Ask ProcessDone function to do a basic validation before freeing
            NsmDiagProcessDoneList(adapter, 0x1, NS_TRUE, 
                              &adapter->diagRxDoneList, &adapter->rxFragsTail);
            adapter->diagStats.rxDropped++;
         }

         // Get a free diag pkt node from the head of the free list
         node = MPL_LIST_GET_HEAD(&adapter->diagFreeList); 

         // Get the Diag Pkt linked to the curr node
         dPkt = MPL_LIST_GET_ENTRY(node, DIAG_PKT, link);

         // Note pkt details
         dPkt->pktStatus = pPkt[i].packetStatus;
         dPkt->cmdSts = pPkt[i].rxOOB.cmdSts;
         dPkt->pktSize = pPkt[i].packetSize;
         dPkt->appPktHandle = 0x0; // Reference handle for Rx pkts
         dPkt->numDesc = pPkt[i].fragCount;

         // NSM Diag related
         dPkt->pFrag = pHeadFrag;

         // Move this packet from the free list to the done list
         MplListDel(&adapter->diagFreeList, &dPkt->link);
         MplListAddTail(&adapter->diagRxDoneList, &dPkt->link);
      }
      else
      {
         pTailFrag = pHeadFrag;
         for (j = 0x0; j < (pPkt[i].fragCount - 1); j++)
         {
            pTailFrag = pTailFrag->pNextFrag;
         }

         // Add the frags back to the free list
         adapter->rxFragsTail->pNextFrag = pHeadFrag;
         pTailFrag->pNextFrag = NULL;
         adapter->rxFragsTail = pTailFrag;
      }
   }

   NO(NsmDiagReceive);
   return;
}

//*****************************************************************************
//   NsmDiagProcessDoneList
//     Process one or more packets in the Tx/Rx Done queue
//
//   Parameters
//      adapter
//         NSM Context
//      pktCnt
//         Number of packets to process.
//         Set this to 0x0 to process all packets in the completion queue
//      validate
//         NS_TRUE : Do a basic validation on the packet before releasing it
//         NS_FALSE : Just release it
//       pList
//         Done List (Tx/Rx) to process
//       ppTailFrag
//         Pointer to a frag pointer at which to append the freed frags
//   Return Value
//      None
//*****************************************************************************
NS_VOID
NsmDiagProcessDoneList(
      NSM_CONTEXT   *adapter, 
      NS_UINT        pktCnt, 
      NS_BOOLEAN     validate,
      MPL_LIST      *pList,
      MPL_PKT_FRAG **ppTailFrag)
{
   DIAG_PKT *dPkt;
   MPL_LIST_NODE *node;
   NS_UINT i;
   MPL_PKT_FRAG *pTail;

   NI(NsmDiagProcessDoneList);   

   // Check if we need to process all pkts
   if (pktCnt == 0x0)
   {
      pktCnt = MPL_LIST_GET_SIZE(pList);
   }

   // Process done list
   node = MPL_LIST_GET_HEAD(pList); 
   for (i = 0x0; i < pktCnt; i++)
   {
      if (MPL_LIST_GET_SIZE(pList) == 0x0)
      {
         break; //premature?
      }

      // Get the Diag Pkt linked to the curr node
      dPkt = MPL_LIST_GET_ENTRY(node, DIAG_PKT, link);

      // Get the next node
      node  = MPL_LIST_GET_NEXT(node);

      if (dPkt->pFrag)
      {
         // Check if we need to validate this packet before removing
         if (validate == NS_TRUE)
         {
            pktValidate(adapter, dPkt->pFrag);
         }

         // Put the frags back on the free frag list
         pTail = dPkt->pFrag;
         while (pTail->pNextFrag)
         {
            pTail = pTail->pNextFrag;
         }

         // Add the frags back to the free list
         (*ppTailFrag)->pNextFrag = dPkt->pFrag;
         *ppTailFrag = pTail;
      }

      // Move the diag pkt from done list to free list
      MplListDel(pList, &dPkt->link);
      MplListAddTail(&adapter->diagFreeList, &dPkt->link);
   }

   NO(NsmDiagProcessDoneList);   
   return;
}
// Local functions

//*****************************************************************************
//   setupTxFrags
//     Setup the MPL frags for Tx
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      NS_TRUE : Setup was successful
//      NS_FALSE : Setup failed
//*****************************************************************************
static NS_BOOLEAN
setupTxFrags(
      NSM_CONTEXT *adapter)
{
   NS_UINT allocSize, i, nodes;
   MPL_PKT_FRAG *pFrag = NULL, *prevFrag;
   NS_VOID *allocMem;
   NS_BOOLEAN ret = NS_FALSE;
   struct sk_buff *skb;

   NI(setupTxFrags);

   // Allocate memory for the frags = total ring size + sts queue size
   nodes = adapter->txdCnt + adapter->diagStsQueueSize;
            
   allocSize = sizeof(MPL_PKT_FRAG) * nodes;
   allocMem  = kmalloc(allocSize, GFP_KERNEL);

   if (allocMem == NULL)
   {
      NSM_DBG(DBG_ERR,("Failed to Alloc mem to tx frags \n"));
      NO(setupTxFrags);
      return ret;
   }
   else
   {
      // Zero up the memory
      memset((void *)(allocMem), 0x0, allocSize);

      pFrag = (MPL_PKT_FRAG *) allocMem;
      prevFrag = NULL;

      // Create linked list and attach a buffer
      for (i = 0x0; i < nodes; i++)
      {
         // Allocate buffer
         skb = dev_alloc_skb(adapter->txBufferLen);
         if (skb == NULL)
         {
            NSM_DBG(DBG_ERR,("SKB alloc in fragSetupTx Failed! \n"));
            break;
         }

         // Mark as being used by this device.
         skb->dev = adapter->netdev;

         // Set the Fragment
         pFrag->pAddr = skb->data;
         pFrag->pNsmPrivate = skb;

         // Set link
         if (prevFrag)
         {
            prevFrag->pNextFrag = pFrag;
         }
         prevFrag = pFrag;

         pFrag++;
      }
 
      // Note Head and Tail of this frag list
      // Note: If the skb allocation fails above then the Tail will point to
      //       the last fragment with successful allocation
      adapter->txFrags     = allocMem; 
      adapter->txFragsHead = (MPL_PKT_FRAG *)allocMem; 
      adapter->txFragsTail = prevFrag;

      if (prevFrag)
      {
         // Managed to allocate some buffers
         prevFrag->pNextFrag = NULL;
         ret = NS_TRUE;
      }
   }

   NO(setupTxFrags);
   return ret;
}


//*****************************************************************************
//   freeTxFrags
//     Free the MPL frags for Tx
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      None
//*****************************************************************************
static NS_VOID
freeTxFrags(
      NSM_CONTEXT *adapter)
{
   struct pci_dev *pdev = adapter->pdev;
   MPL_PKT_FRAG *pFrag = NULL;
   struct sk_buff* skb;

   NI(freeTxFrags);

   // Free attached buffers
   pFrag = adapter->txFragsHead;
   while (pFrag)
   {
      skb = (struct sk_buff *)pFrag->pNsmPrivate;
      if (skb != NULL)
      {
         // UnMap and Free the sk buffs
         pci_unmap_single(pdev, (dma_addr_t)pFrag->physAddr,
                          adapter->txBufferLen, 
                          PCI_DMA_TODEVICE);
         dev_kfree_skb(skb);
         pFrag->pNsmPrivate = NULL;
      }

      // Move to the next fragment
      pFrag = pFrag->pNextFrag;
   }

   // Free Frag memory
   if (adapter->txFrags)
   {
      kfree(adapter->txFrags);
      adapter->txFrags = NULL;
      adapter->txFragsHead = NULL;
   }

   NO(freeTxFrags);
   return;
}

//*****************************************************************************
//   pktSetup
//     Setup the Diag Pkts Status Queue for Tx and Rx completion handling
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      NS_TRUE : Setup was successful
//      NS_FALSE : Setup failed
//*****************************************************************************
static NS_BOOLEAN
pktSetup(
      NSM_CONTEXT *adapter)
{
   NS_UINT allocSize, i, nodes;
   NS_VOID *allocMem;
   NS_BOOLEAN ret = NS_FALSE;
   DIAG_PKT *pPkt;

   NI(pktSetup);

   // Allocate memory for the pkts = both for tx and rx
   nodes = adapter->diagStsQueueSize * 2;
            
   allocSize = sizeof(DIAG_PKT) * nodes;
   allocMem  = kmalloc(allocSize, GFP_KERNEL);
   if (allocMem == NULL)
   {
      NSM_DBG(DBG_ERR,("Failed to Alloc mem to diag pkts \n"));
      NO(pktSetup);
      return ret;
   }
   else
   {
      // Note mem
      adapter->diagPkts = allocMem; 

      // Zero up the memory
      memset((void *)(allocMem), 0x0, allocSize);
      pPkt = (DIAG_PKT *) allocMem;

      // Initialize Diag packet free list 
      MplListInit(&adapter->diagFreeList, NS_FALSE);

      // Add nodes to the list
      for (i = 0x0; i < nodes; i++)
      {
         MplListAddTail(&adapter->diagFreeList, &pPkt->link);
         pPkt++; //Get the next packet
      }
      
      // Initialize Diag packet done list (for Tx) - Starts with no nodes
      MplListInit(&adapter->diagTxDoneList, NS_FALSE);

      // Initialize Diag packet done list (for Rx) - Starts with no nodes
      MplListInit(&adapter->diagRxDoneList, NS_FALSE);

      ret = NS_TRUE;
   }

   NO(pktSetup);
   return ret;
}


//*****************************************************************************
//   pktFree
//     Free the diag packets in status queue
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      None
//*****************************************************************************
static NS_VOID
pktFree(
      NSM_CONTEXT *adapter)
{
   NI(pktFree);

   // Clear Tx done list
   NsmDiagProcessDoneList(adapter, 0x0, NS_FALSE,
                     &adapter->diagTxDoneList, &adapter->txFragsTail);

   // Clear Rx done list
   NsmDiagProcessDoneList(adapter, 0x0, NS_FALSE, 
                     &adapter->diagRxDoneList, &adapter->rxFragsTail);

   // UnInitialize Diag packet done list (for Tx)
   MplListDeInit(&adapter->diagTxDoneList);

   // UnInitialize Diag packet done list (for Rx)
   MplListDeInit(&adapter->diagRxDoneList);

   // UnInitialize Diag packet free list
   MplListDeInit(&adapter->diagFreeList);

   // Free Pkt memory
   if (adapter->diagPkts)
   {
      kfree(adapter->diagPkts);
      adapter->diagPkts = NULL;
   }

   NO(pktFree);
   return;
}


//*****************************************************************************
//   txPkt
//     Diag module tx 
//
//   Parameters
//      adapter
//         NSM Context
//     txCmd
//         Command block describing the Tx
//     txData
//         Data block for this Tx
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The initialization was successfully done
//      NS_STATUS_RESOURCES
//         Failed to allocate required resources
//      NS_STATUS_FAILURE
//         Failed to transmit
//*****************************************************************************
static NS_UINT
   txPkt(
      NSM_CONTEXT     *adapter,
      NS_APP_XFER_CMD *txCmd,
      NS_UINT8        *txData)
{
   MPL_PKT_FRAG *pFrag, *headFrag, *prevFrag, *qFrag;
   MPL_PKT mplPkt;
   NS_UINT status, fragsReq, bytesToGo, src, fragCnt, i;

   NI(txPkt);

   // Test if the link is up and the diag tx engine is initialized
   if ((adapter->linkStatus != MPL_LINK_STATUS_UP) ||
         (adapter->txFragsHead == NULL))
   {
      NO(txPkt);
      return NS_STATUS_FAILURE;
   }

   if (txCmd->packetSize == 0x0)
   {
      // What?
      NO(txPkt);
      return NS_STATUS_INVALID_PARM;
   }

   // Compute the number of frags needed
   fragsReq = (txCmd->packetSize + adapter->txBufferLen - 1) /
                                      adapter->txBufferLen;

   // Make sure we have enough frags to sent this pkt
   //  if not force some from the done list
   spin_lock_irq(&adapter->txLock);
   while(checkAvailTxFrag(adapter, fragsReq) != NS_TRUE)
   {
      // Force some frags from the Done list
      NsmDiagProcessDoneList(adapter, 0x1, NS_TRUE,
                          &adapter->diagTxDoneList, &adapter->txFragsTail);
   }
   spin_unlock_irq(&adapter->txLock);

   // Setup the frags
   headFrag = pFrag = adapter->txFragsHead;
   headFrag->pPacketHndl = (NS_VOID *)txCmd->pktHandle;

   // Initialize byte counters
   bytesToGo = txCmd->packetSize;
   src  = 0x0;
   prevFrag = NULL;
      
   while (bytesToGo)
   {
      if (adapter->txBufferLen >= bytesToGo)
      {
         // Final MPL Frag 
         pFrag->fragSize = bytesToGo;
      }
      else
      {
         // More data to go.. Map as much data as allowed and move on
         pFrag->fragSize  = adapter->txBufferLen;
      }
       
      // Set the frag, copy data to frag
      if (copy_from_user(pFrag->pAddr, &txData[src], pFrag->fragSize))
      {
         return NS_STATUS_FAILURE;
      }
      pFrag->physAddr  = pci_map_single(adapter->pdev, pFrag->pAddr, 
                                             pFrag->fragSize, PCI_DMA_TODEVICE);

      // Reduce bytes to go by the size copied
      bytesToGo -= pFrag->fragSize;

      // Note the current frag
      prevFrag = pFrag;

      // Update data offset
      src += pFrag->fragSize;

      // Get a new frag
      pFrag = pFrag->pNextFrag;
   }

   // Prep the MPL pkt
   mplPkt.pNextPacket = NULL;
   mplPkt.fragCount = fragsReq;

   // Set priority after validating
   if ((txCmd->pQueue < 0x1) || 
          (txCmd->pQueue > adapter->mplCaps.txCaps.priorityQCnt))
   {
      // Default to queue 1 - This is returned back to App
      txCmd->pQueue = 0x1;
   }

   mplPkt.txOOB.pQueue = txCmd->pQueue;
   mplPkt.pFragHead = headFrag;
   mplPkt.packetSize  = txCmd->packetSize;
   mplPkt.pNsmPrivate = headFrag; // Head frag as handle

   // Send Packet
   spin_lock_irq(&adapter->txLock);
   status = MplTransmit(adapter->mplContext, 0x01, &mplPkt);

   // Check status
   if (status == NS_STATUS_SUCCESS)
   {
      // Prep for returning status of this packet
      // Partial status is also returned back to the App to handle
      //  cases where the Tx never completes

      txCmd->numDesc = mplPkt.fragCount;

      // Copy physical address used for this transfer
      qFrag = headFrag;
      fragCnt  = (mplPkt.fragCount < NS_APP_MAX_FRAG_INFO) ?
                        mplPkt.fragCount : NS_APP_MAX_FRAG_INFO;
      for (i = 0x0; i < fragCnt; i++)
      {
         txCmd->descPhyAddr[i] = (NS_ADDR)qFrag->pMplPrivate;
         txCmd->bufPhyAddr[i]  = qFrag->physAddr;
         qFrag = qFrag->pNextFrag;
      }

      // Move the headFrag pointer - this update happens only on Tx success
      adapter->txFragsHead = pFrag;
      prevFrag->pNextFrag = NULL; // Detach frags for current pkt
   }

   // Release lock
   spin_unlock_irq(&adapter->txLock);

   NO(txPkt);
   return status;
}

//*****************************************************************************
//   checkAvailTxFrag
//     Check is the required Tx Frags are available for transmiting this pkt
//
//   Parameters
//      adapter
//         NSM Context
//      reqFrags
//         Number of frags required
//
//   Return Value
//      NS_TRUE
//         Available
//      NS_FALSE
//         Not Available
//*****************************************************************************
static NS_BOOLEAN
   checkAvailTxFrag(
      NSM_CONTEXT     *adapter,
      NS_UINT          reqFrags)
{
   NS_UINT cnt = 0x0;
   MPL_PKT_FRAG *pFrag;

   NI(checkAvailTxFrag);   

   pFrag = adapter->txFragsHead;
   while (cnt < reqFrags)
   { 
      if (pFrag == adapter->txFragsTail)
      {
         NO(checkAvailTxFrag);   
         return NS_FALSE;
      }
      pFrag = pFrag->pNextFrag;
      cnt++;
   }

   NO(checkAvailTxFrag);   
   return NS_TRUE;
}

//*****************************************************************************
//   pktStatus
//     Checks the status of previous Tx pkt
//
//   Parameters
//     adapter
//         NSM Context
//     pList
//         List to process from
//     ppTailFrag
//         Pointer to a frag pointer at which to append the freed frags
//     pLock
//         Pointer to the lock controlling access to parent list
//     pCmd
//         Command block describing the pkt
//     pData
//         Destination location to copy data
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The packet was successfully sent
//      NS_STATUS_INVALID_PARM
//         The specified packet handle was not found 
//         Note: This could happen when the status queue was oveflowed too
//      NS_STATUS_FAILURE
//         The packet completed with a failure
//      NS_STATUS_RESOURCES
//         The pkt could not be copied since the buffer is of smaller size
//*****************************************************************************
static NS_UINT
   pktStatus(
      NSM_CONTEXT     *adapter,
      MPL_LIST        *pList,
      MPL_PKT_FRAG   **ppTailFrag,
      spinlock_t      *pLock,
      NS_APP_XFER_CMD *pCmd,
      NS_UINT8        *pData)
{
   DIAG_PKT *dPkt;
   MPL_PKT_FRAG *pFrag, *pTail, *pHead = NULL;
   MPL_LIST_NODE *node;
   NS_UINT status = NS_STATUS_INVALID_PARM, dst, fragCnt;

   NI(pktStatus);   

   // Take lock and search for the pkt in the done list
   spin_lock_irq(pLock);
   for (node = MPL_LIST_GET_HEAD(pList); 
           MPL_LIST_CHK_END(pList, node) == NS_FALSE; 
               node = MPL_LIST_GET_NEXT(node))
   {
      dPkt = MPL_LIST_GET_ENTRY(node, DIAG_PKT, link);

      // Match the app pkt handle with what we have
      if (dPkt->appPktHandle == pCmd->pktHandle)
      {
         // Found the requested pkt

         // Prep response
         pCmd->numDesc = dPkt->numDesc;
         pCmd->cmdSts = dPkt->cmdSts;
         pCmd->packetSize = dPkt->pktSize;
         status = dPkt->pktStatus;

         // Note head frag pointer and detach from packet
         pHead = dPkt->pFrag;
         dPkt->pFrag = NULL;

         // Move the Diag pkt node from Rx done to free list
         MplListDel(pList, &dPkt->link);
         MplListAddTail(&adapter->diagFreeList, &dPkt->link);

         // Return count of completed pkts waiting for app
         pCmd->morePkts = MPL_LIST_GET_SIZE(pList);
         break;
      }
   }
   spin_unlock_irq(pLock);

   // If we managed to find the req node then free the frags
   //  since we are done with this packet
   if (pHead)
   {
      pTail = pFrag = pHead;
      dst = fragCnt = 0x0;

      // Do the copy of the data etc.. This is done outside of the locks
      while (pFrag)
      {
         if (pCmd->flags & NSM_APP_XFER_COPY)
         {
            // Check for overflow
            if (dst > pCmd->bufSize)
            {
               status = NS_STATUS_RESOURCES;
            }
            else
            {
               // Copy data back
               if (copy_to_user(&pData[dst], pFrag->pAddr, pFrag->fragSize))
               {
                  status = NS_STATUS_FAILURE;
               }
               dst += pFrag->fragSize;
            }
         }

         // Copy physical address used for this transfer
         if (fragCnt < NS_APP_MAX_FRAG_INFO) 
         {
            pCmd->descPhyAddr[fragCnt] = (NS_ADDR)pFrag->pMplPrivate;
            pCmd->bufPhyAddr[fragCnt++]  = pFrag->physAddr;
         }

         // Get next frag
         pTail = pFrag;
         pFrag = pFrag->pNextFrag;
      }

      // Add the frags back to the free list
      spin_lock_irq(pLock);
      (*ppTailFrag)->pNextFrag = pHead;
      *ppTailFrag = pTail;
      spin_unlock_irq(pLock);
   }

   NO(pktStatus);   
   return status;
}

//*****************************************************************************
//   pktValidate
//     Do some basic validation on the pkt, and update stats
//
//   Parameters
//      adapter
//         NSM Context
//      pHeadFrag
//         Pointer to the head fragment in the pkt
//   Return Value
//      None
//*****************************************************************************
static NS_VOID
pktValidate(
      NSM_CONTEXT *adapter,
      MPL_PKT_FRAG *pHeadFrag)
{
   NI(pktValidate);   
   NO(pktValidate);   
   return;
}

//*****************************************************************************
//   flushRx
//     Dump all pkts in done list and puts the frags and Diag Pkts back on
//     free lists
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      None
//*****************************************************************************
static NS_VOID
flushRx(
      NSM_CONTEXT *adapter)
{

   NI(flushRx);   

   spin_lock_irq(&adapter->rxLock);

   // Force all frags from the Done list
   NsmDiagProcessDoneList(adapter, 0x0, NS_FALSE,
                          &adapter->diagRxDoneList, &adapter->rxFragsTail);

   // Clear all stats
   adapter->diagStats.rxPackets = 0x0;
   adapter->diagStats.rxBytes = 0x0;
   adapter->diagStats.rxErrors = 0x0;
   adapter->diagStats.rxDropped = 0x0;
   adapter->diagStats.rxLengthErrors = 0x0;
   adapter->diagStats.rxOverErrors = 0x0;
   adapter->diagStats.rxCrcErrors = 0x0;
   adapter->diagStats.rxFrameErrors = 0x0;

   spin_unlock_irq(&adapter->rxLock);

   NO(flushRx);   
   return;
}

//*****************************************************************************
//   flushTx
//     Dump all pkts in done list and puts the frags and Diag Pkts back on
//     free lists
//
//   Parameters
//      adapter
//         NSM Context
//   Return Value
//      None
//*****************************************************************************
static NS_VOID
flushTx(
      NSM_CONTEXT *adapter)
{
   NI(flushTx);   

   spin_lock_irq(&adapter->txLock);

   // Force all frags from the Done list
   NsmDiagProcessDoneList(adapter, 0x0, NS_FALSE,
                          &adapter->diagTxDoneList, &adapter->txFragsTail);

   // Clear all stats
   adapter->diagStats.txPackets = 0x0;
   adapter->diagStats.txBytes = 0x0;
   adapter->diagStats.txErrors = 0x0;
   adapter->diagStats.txDropped = 0x0;
   adapter->diagStats.txCollErrors = 0x0;
   adapter->diagStats.txAbortedErrors = 0x0;
   adapter->diagStats.txCarrierErrors = 0x0;
   adapter->diagStats.txFifoErrors = 0x0;
   adapter->diagStats.txWindowErrors = 0x0;

   spin_unlock_irq(&adapter->txLock);

   NO(flushTx);   
   return;
}

//*****************************************************************************
//   getDiagStats
//     Diag module check status of previous Tx
//
//   Parameters
//     adapter
//         NSM Context
//     pCmd
//         Diag Stats command request
//
//   Return Value
//     None
//*****************************************************************************
static NS_VOID
   getDiagStats(
      NSM_CONTEXT       *adapter,
      NS_APP_STATS_CMD  *pCmd)
{
   NI(getDiagStats);   

   // Basic Tx, Rx
   pCmd->rxPackets = adapter->diagStats.rxPackets;
   pCmd->txPackets = adapter->diagStats.txPackets;
   pCmd->rxBytes = adapter->diagStats.rxBytes;
   pCmd->txBytes = adapter->diagStats.txBytes;
   pCmd->rxErrors = adapter->diagStats.rxErrors;
   pCmd->txErrors = adapter->diagStats.txErrors;
   pCmd->rxDropped = adapter->diagStats.rxDropped;
   pCmd->txDropped = adapter->diagStats.txDropped;

   // detailed rx_errors  - As seen in descriptor cmdsts
   pCmd->rxLengthErrors = adapter->diagStats.rxLengthErrors;
   pCmd->rxOverErrors = adapter->diagStats.rxOverErrors;
   pCmd->rxCrcErrors = adapter->diagStats.rxCrcErrors;
   pCmd->rxFrameErrors = adapter->diagStats.rxFrameErrors;

   // detailed tx_errors  - As seen in descriptor cmdsts
   pCmd->txCollErrors = adapter->diagStats.txCollErrors;
   pCmd->txAbortedErrors = adapter->diagStats.txAbortedErrors;
   pCmd->txCarrierErrors = adapter->diagStats.txCarrierErrors;
   pCmd->txFifoErrors = adapter->diagStats.txFifoErrors;
   pCmd->txWindowErrors = adapter->diagStats.txWindowErrors;

   NO(getDiagStats);   
   return;
}

//*****************************************************************************
//   xferCfgSet
//     Configure the Tx/Rx engine
//
//   Parameters
//     adapter
//         NSM Context
//     pCmd
//         Diag xfer cfg command request
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The requested configurations were successfully applied
//      NS_STATUS_INVALID_PARM
//         The configuration values are not valid
//      NS_STATUS_FAILURE
//         The requested configuration could not be applied on the device
//      NS_STATUS_RESOURCES
//         Resources required for this configuration could not be allocated
//*****************************************************************************
static NS_UINT
   xferCfgSet(
      NSM_CONTEXT         *adapter,
      NS_APP_XFER_CFG_CMD *pCmd)
{
   MPL_TRANSMIT_CFG txCfg;
   MPL_RECEIVE_CFG rxCfg;
   NS_UINT i;
   NS_UINT status;

   NI(xferCfgSet);

   // Step0: Validate the parameters before doing any work
   if (pCmd->txpQueueCnt > adapter->mplCaps.txCaps.priorityQCnt)
   {
      return NS_STATUS_INVALID_PARM;
   }

   // Step1: Reset current configuration

   // Reset Tx and Rx engines - This gives all buffers frags back to NSM
   spin_lock_irq(&adapter->txLock);
   MplTransmitReset(adapter->mplContext);
   spin_unlock_irq(&adapter->txLock);

   spin_lock_irq(&adapter->rxLock);
   MplReceiveReset(adapter->mplContext);
   spin_unlock_irq(&adapter->rxLock);

   // Free the Diag pkts - This flushes the pkts in the done list too
   pktFree(adapter);

   // Free the fragments, tx and rx
   freeTxFrags(adapter);
   freeRxFrags(adapter);

   // Step2: Set the new values
   adapter->rxdCnt = pCmd->rxDescCnt;
   adapter->txdCnt = pCmd->txDescCnt;
   adapter->diagStsQueueSize = pCmd->stsQueueSize;
   adapter->rxBufferLen = pCmd->rxBufferLen; 
   adapter->txBufferLen = pCmd->txBufferLen; 
   adapter->txQCnt = pCmd->txpQueueCnt;
   adapter->rxQCnt = NSM_DEFAULT_RX_Q_CNT; //Force it to one

   // Step3: Apply the new values
   txCfg.cfgFlags = MPL_TRANSMIT_CFG_QUEUES;
   txCfg.maxIndications  = NSM_DEFAULT_TX_MAX_INDY;
   txCfg.priorityQueueCnt = adapter->txQCnt;
   for (i = 0x0; i < adapter->txQCnt; i++)
   {
      txCfg.descCnt[i] = adapter->txdCnt;
   }

   // Tell MPL about the new Tx configurations
   status = MplTransmitCfg(adapter->mplContext, &txCfg);
   if (status == NS_STATUS_SUCCESS)
   {
      rxCfg.cfgFlags = MPL_RECEIVE_CFG_QUEUES;
      rxCfg.maxIndications  = NSM_DEFAULT_RX_MAX_INDY;

      rxCfg.priorityQueueCnt = adapter->rxQCnt;
      for (i = 0x0; i < adapter->rxQCnt; i++)
      {
         rxCfg.descCnt[i] = adapter->rxdCnt;
      }

      // Tell MPL about the new Rx configurations
      status = MplReceiveCfg(adapter->mplContext, &rxCfg);
   }

   // Diag related setup
   if (status == NS_STATUS_SUCCESS)
   {
      // Setup new frags based on the new settings
      if ((setupTxFrags(adapter) == NS_TRUE) &&
            (setupRxFrags(adapter) == NS_TRUE))
      {
         // Set up status queue Diag packets based on new status Queue size
         if (pktSetup(adapter) == NS_TRUE)
         {
            // Replenish the device's receive engine with new buffers
            replenishTask((unsigned long)adapter);
         }
         else
         {
            freeRxFrags(adapter);
            freeTxFrags(adapter);
            status = NS_STATUS_RESOURCES;
         }
      }
      else
      {
         freeRxFrags(adapter);
         freeTxFrags(adapter);
         status = NS_STATUS_RESOURCES;
      }
   }

   NO(xferCfgSet);
   return status;
}

//*****************************************************************************
//   xferCfgGet
//     Get the Tx/Rx engine configurations
//
//   Parameters
//     adapter
//         NSM Context
//     pCmd
//         Diag xfer cfg command request
//
//   Return Value
//      NS_STATUS_SUCCESS
//         The requested configurations were successfully applied
//*****************************************************************************
static NS_UINT
   xferCfgGet(
      NSM_CONTEXT         *adapter,
      NS_APP_XFER_CFG_CMD *pCmd)
{
   NI(xferCfgGet);

   pCmd->rxDescCnt = adapter->rxdCnt;
   pCmd->txDescCnt = adapter->txdCnt;
   pCmd->stsQueueSize = adapter->diagStsQueueSize;
   pCmd->rxBufferLen = adapter->rxBufferLen; 
   pCmd->txBufferLen = adapter->txBufferLen; 
   pCmd->txpQueueCnt = adapter->txQCnt;

   NO(xferCfgGet);
   return NS_STATUS_SUCCESS;
}

#endif //NSM_DIAG_MODE
