
//******************************************************************************
//
//  NSMOS.C
//
//  Copyright (c) 2005 National Semiconductor Corporation.
//  All Rights Reserved
//
//  NSM Driver for MPL based MacPhyter Devices
//
//******************************************************************************

#include <nsm.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("NSC");
MODULE_DESCRIPTION("NSC DP8381X 10/100Mbps MacPhyter Ethernet");

// Max NICs supported by this driver
#define DP_MAX_NIC 8

// Globals                                                           
char DP_driverName[] = "DP8381X";
char DP_driverString[]="NSC Macphyter Driver";
char DP_driverVersionMaj = 1;  // Major Ver
char DP_driverVersionMin = 6;  // Minor Ver
char DP_driverVersion[] = "n.1.6";
char DP_copyright[] = "Copyright (c) 2005-2007 National Semiconductor";
char DP83815_ID[] = "DP83815 (MacPhyter-I)";
char DP83816_ID[] = "DP83816 (MacPhyter-II)";
char DP83818_ID[] = "DP83818 (MacPhyter-III)";
char UNKNOWN_ID[] = "UNKNOWN";

// Module Params

// Initialize all to -1 i.e not configured
#define DP_PARAM_INIT { [0 ... (DP_MAX_NIC - 1)] = -1}

#define DP_PARAM(PARM, DESC) \
static const int __devinitdata PARM[DP_MAX_NIC] = DP_PARAM_INIT; \
MODULE_PARM(PARM, "1-" __MODULE_STRING(DP_MAX_NIC) "i"); \
MODULE_PARM_DESC(PARM, DESC);

// Number of Transmit Descriptors (32bytes each)
// Default : 64
DP_PARAM(TxDCnt, "Number Of Transmit Descriptors");

// Number of Transmit Queues to be enabled
// Default : 1, Max : 4 (only on DP83818)
DP_PARAM(TxQCnt, "Number Of Transmit Queues");

// Number of Receive Descriptors (32bytes each)
// Default : 64
DP_PARAM(RxDCnt, "Number Of Receive Descriptors");

// Number of Receive Queues to be enabled
// Default : 1, Max : 1
DP_PARAM(RxQCnt, "Number Of Receive Queues");

// Desired Link Setup Mode
// 0 : Auto Negotiate (Default)
// 1 : Force Speed and Duplex Mode
DP_PARAM(LinkMode, "Link Setup Mode");

// Desired Link Speed 
// Note : If set in AutoNeg mode then this is the Max speed advertized
// 0 : 100Mbps (Default)
// 1 : 10Mbps 
DP_PARAM(LinkSpeed, "Link Speed");

// Desired Link Duplex Mode
// Note : If set in AutoNeg mode then this is the mode advertized
// 0 : Full Duplex (Default)
// 1 : Half Duplex
DP_PARAM(LinkDuplex, "Link Duplex Mode");

// Desired Link Pause Mode
// Note : If set in AutoNeg mode then this is the mode advertized
// 0 : Receive Pause Only (Default)
// 1 : Transmit Pause Only 
// 2 : Transmit and Receive Pause (Symmetrical)
// 3 : No Pause generation
DP_PARAM(LinkPause, "Link Pause Mode");

// Interrupt Holdoff in usec (Micro)
// Default : 100
// Max : 25500 (i.e. 25.5 msec)
DP_PARAM(IntTimeHold, "Interrupt Holdoff");

// Interrupt Holdoff in Tx pkt count
// Default : 2
// Max : 63
DP_PARAM(IntTxHold, "Interrupt Holdoff in Tx Packets");

// Interrupt Holdoff in Rx pkt count
// Default : 2
// Max : 63
DP_PARAM(IntRxHold, "Interrupt Holdoff in Rx Packets");

// Driver Operation Mode
// 0 : Normal (Default)
// 1 : Monitor (National Semiconductor Internal)
// 2 : Load Only (National Semiconductor Internal)
DP_PARAM(OpMode, "Driver Operational Mode");

// Desired Wol Events (Logical OR)
// 0x01  :  MPL_WOL_MAGIC - Enable wake on Magic Pattern.
// 0x04  :  MPL_WOL_BROADCAST - Enable wake on any broadcast packet.
// 0x08  :  MPL_WOL_MULTICAST - Enable wake on multicast packet.
// 0x10  :  MPL_WOL_DIRECTED - Enable wake on directed (unicast) packet to 
//                             the device's address.
// 0x20  :  MPL_WOL_LINK - Enable wake on link status change.
// 0x40  :  MPL_WOL_ARP - Enable wake on any ARP packet.
DP_PARAM(Wol, "Wake-on-LAN Events");

#ifdef NSM_DIAG_MODE
// Number of pkts possible in the status queue
// Default : 256
DP_PARAM(DiagStsQueue, "Size of the Diag Status Queue");
#endif //NSM_DIAG_MODE

#ifdef NSM_DEBUG
// NSM (OS Specific Module) Debug Zones to be enabled
DP_PARAM(NsmDebugZones, "Debug Zones For NSM Module");

// MPL (Hardware Abstraction Layer) Debug Zones to be enabled
DP_PARAM(MplDebugZones, "Debug Zones For MPL Module");

// Debug Zones for this file
   NS_UINT NsmDbgSettings =
                     DBG_INFO | 
                     DBG_FUNC |
                     DBG_TMP  |
                     DBG_ERR; 

   NS_UINT MplPathZones = 
                     DZONE_MPL_INIT_DOWN |
                     DZONE_MPL_TRANS     |
                     DZONE_MPL_RECV      |
                     DZONE_MPL_INTR      |
                     DZONE_MPL_LINK      |
                     DZONE_MPL_PWR       |
                     DZONE_MPL_DIAG      |
                     DZONE_MPL_MISC;

   NS_UINT MplMsgZones = 
                     DZONE_MPL_INIT_DOWN |
                     DZONE_MPL_TRANS     |
                     DZONE_MPL_RECV      |
                     DZONE_MPL_INTR      |
                     DZONE_MPL_LINK      |
                     DZONE_MPL_PWR       |
                     DZONE_MPL_DIAG      |
                     DZONE_MPL_MISC;
#else
   NS_UINT NsmDbgSettings = 0x0; 
   NS_UINT MplPathZones = 0x0; 
   NS_UINT MplMsgZones = 0x0; 
#endif

// Adapter count
unsigned NsmAdapterCnt = 0x0;

/*********************************************************************
 *  PCI device list for NSC MacPhyter adapters
 *  { Vendor ID, Device ID, SubVendor ID, SubDevice ID,
 *    Class, Class Mask, String Index }
 *  Last device should be 0
 *********************************************************************/

#define PCI_DEVICE_ID_DP8381X 0x0020
static struct pci_device_id DP_pciTable[] = {
   { PCI_VENDOR_ID_NS, PCI_DEVICE_ID_DP8381X, 
     PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
   { 0, }
};
MODULE_DEVICE_TABLE(pci, DP_pciTable);


// PCI driver struct for this driver
static struct pci_driver DP_driver = {
   name:     DP_driverName,
   id_table: DP_pciTable,
   probe:    NsmProbe,
   remove:   NsmRemove,
#ifdef CONFIG_PM
   suspend:  NsmSuspend,
   resume:   NsmResume
#endif
};

// Locals
static NS_VOID readPciInfo(NSM_CONTEXT *adapter);
static NS_VOID setDataFields(NSM_CONTEXT *adapter);
static NS_VOID readUserOptions(NSM_CONTEXT *adapter);
static NS_BOOLEAN configMac(NSM_CONTEXT *adapter);
static NS_BOOLEAN configLink(NSM_CONTEXT *adapter);
static NS_VOID linkTask(unsigned long data);

static 
NS_SINT __init NsmInitModule(void)
{
   NI(NsmInitModule);

   // Print driver ver
   printk(KERN_INFO"%s - version %s\n%s\n", DP_driverString,
                                   DP_driverVersion,
                                   DP_copyright);

   /* Register to PCI subsystem */
   NO(NsmInitModule);
   return pci_module_init(&DP_driver);
}

static 
NS_VOID __exit NsmExitModule(void)
{
   NI(NsmExitModule);
   // UnRegister from PCI subsystem
   pci_unregister_driver(&DP_driver);

   NO(NsmExitModule);
   return;
}

// Module entry/exit points
module_init(NsmInitModule);
module_exit(NsmExitModule);

NS_SINT
NsmProbe(struct pci_dev *pdev, const struct pci_device_id *id)
{
   struct net_device *netdev = NULL;
   NSM_CONTEXT *adapter;
   NS_CHAR *nicId = NULL;

   NI(NsmProbe);

   // Make sure we are not exceeding our max card count
   if (NsmAdapterCnt == DP_MAX_NIC)
   {
      NSM_DBG(DBG_ERR,("Card Found exceeds limit \n"));

      NO(NsmProbe);
      return -ENODEV;
   }
    
   // Enable this PCI device
   if(pci_enable_device(pdev) != 0) \
   {
      NSM_DBG(DBG_ERR,("pci_enable_device failed\n"));

      NO(NsmProbe);
      return -ENODEV;
   }

   // Set the device as Master capable
   pci_set_master(pdev);

   // Check to see if our PCI addressing needs are supported
   if(!pci_dma_supported(pdev, NSM_DMA_MASK))
   {
      NSM_DBG(DBG_ERR,("PCI DMA not supported by the system\n"));

      NO(NsmProbe);
      return -ENODEV;
   }
   else
      pdev->dma_mask = NSM_DMA_MASK;

   // Init the net device, allocate adapter context
   netdev = alloc_etherdev(sizeof (NSM_CONTEXT));
   if(netdev == NULL) 
   {
      NSM_DBG(DBG_ERR,("Unable to allocate net_device struct\n"));

      NO(NsmProbe);
      return -ENOMEM;
   }

   // Init adapter context
   adapter = (NSM_CONTEXT *) netdev->priv;
   memset(adapter, 0, sizeof(NSM_CONTEXT));
   adapter->netdev = netdev;
   adapter->pdev = pdev;
   adapter->regAddr = NULL;
   pci_set_drvdata(pdev, netdev);

   // Default MTU
   netdev->mtu = ENET_MTU;

   if (pci_request_regions(pdev, DP_driverName))
   {
      NSM_DBG(DBG_ERR,("pci_request_region failed\n"));
      NsmRemove(pdev);

      NO(NsmProbe);
      return -ENOMEM;
   }

#ifdef NSM_IO_MAP
   // We are operating on the IO space
   netdev->base_addr = pci_resource_start(pdev, BAR_0);
   adapter->regAddr =  (NS_VOID *)netdev->base_addr;
   netdev->mem_start = pci_resource_start(pdev, BAR_0);
   netdev->mem_end = netdev->mem_start + pci_resource_len(pdev, BAR_0);
#else
   // We are operating on the Mem space
   // Remap the IO region
   adapter->regAddr = ioremap(pci_resource_start(pdev, BAR_1), 
                               pci_resource_len(pdev, BAR_1));

   if(adapter->regAddr == NULL) 
   {
      NSM_DBG(DBG_ERR,("ioremap failed\n"));
      NsmRemove(pdev);

      NO(NsmProbe);
      return -ENOMEM;
   }
   else
   {
     // Let the Net stack know about the IO region
     netdev->base_addr =  pci_resource_start(pdev, BAR_1);
     netdev->mem_start = pci_resource_start(pdev, BAR_1);
     netdev->mem_end = netdev->mem_start + pci_resource_len(pdev, BAR_1);
   }
#endif

   // Read the Pci info
   readPciInfo(adapter);

   // Set default values for this adapter struct
   setDataFields(adapter);

   // Read module params and note in adapter struct
   readUserOptions(adapter);

   // Enable MPL debug levels
   MplDebEnablePathTracing(MplPathZones);
   MplDebEnableMsgsAndChecks(MplMsgZones);

   // MPL Intializations
   adapter->mplContext = NULL;
   if (MplInitialize(adapter, adapter->regAddr, adapter->opMode, 
                              &adapter->mplContext) 
        != NS_STATUS_SUCCESS)
   {
      NsmRemove(pdev);
      NO(NsmProbe);
      printk(KERN_INFO"%s - Failed to Initialize Device! \n", 
                                   DP_driverString);
      return -ENODEV;
   }

   // Set the Netdevice entry points for this adapter
   netdev->open               = &NsmOpen;
   netdev->stop               = &NsmClose;
   netdev->hard_start_xmit    = &NsmXmit;
   netdev->get_stats          = &NsmStats;
   netdev->set_multicast_list = &NsmSetMulti;
   netdev->set_mac_address    = &NsmSetMac;
   netdev->do_ioctl           = &NsmIoctl;
   netdev->change_mtu         = &NsmChangeMtu;

   // Register this device to Net Stack
   SET_MODULE_OWNER(netdev);
#ifdef L26
   SET_NETDEV_DEV(netdev, &pdev->dev);
#endif
   if (register_netdev(netdev))
   {
      NsmRemove(pdev);

      NO(NsmProbe);
      return -ENODEV;
   }
   else
   {
      adapter->netdevReg = NS_TRUE;
   }

   // Hold-off the netdev till we have a link
   netif_carrier_off(netdev);

   // Note our irq - The line itself requested after we are open
   netdev->irq = pdev->irq;

   // Set Netdevice features - FM: Checksum?
   netdev->features = NETIF_F_SG;

   // Note the instance count for this adapter and increment cards found
   adapter->cardIndex = ++NsmAdapterCnt;

   nicId = UNKNOWN_ID;
   if (adapter->opMode != MPL_MODE_LOAD_ONLY)
   {
      // Get the current mac addr
      MplGetMacAddress(adapter->mplContext, NS_FALSE, netdev->dev_addr);

      // Display device
      adapter->deviceId = MplGetDeviceId(adapter->mplContext);
      switch (adapter->deviceId)
      {
         case MPL_DEVICE_ID_DP83815 : nicId = DP83815_ID;
                                      break;
         case MPL_DEVICE_ID_DP83816 : nicId = DP83816_ID;
                                      break;
         case MPL_DEVICE_ID_DP83818 : nicId = DP83818_ID;
                                      break;
         default                    : nicId = UNKNOWN_ID;
      }
   }

   adapter->verMajor = DP_driverVersionMaj;
   adapter->verMinor = DP_driverVersionMin;

   printk("%s: v-%d.%d NSC-%s:%x-%x Mem:0x%p  IRQ:%d  RegAddr:0x%p Mode:%d \n",
             netdev->name,adapter->verMajor, adapter->verMinor,
              nicId, adapter->pciSubVendorId, 
               adapter->pciSubDeviceId, (void *)netdev->mem_start, netdev->irq,
                adapter->regAddr, adapter->opMode);

   // AOK!
   NO(NsmProbe);
   return 0;
}

#ifdef CONFIG_PM
NS_SINT
NsmSuspend(struct pci_dev *pdev, NS_UINT state)
{
   struct net_device *netdev = pci_get_drvdata(pdev);
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;
   MPL_STATUS status;

   rtnl_lock();
   if (netif_running(netdev)) 
   {
      // We are currently running so, prep for suspend
      
      // Disable further Interrupts
      MplInterruptDisable(adapter->mplContext);

      // Stop Netstack from sending more pkts
      netif_stop_queue(netdev);

      // Reset Receive Engine - To get back pending fragments
      spin_lock_irq(&adapter->rxLock);
      MplReceiveReset(adapter->mplContext);
      spin_unlock_irq(&adapter->rxLock);

      // Reset Transmit Engine - To get back pending pkt
      spin_lock_irq(&adapter->txLock);
      MplTransmitReset(adapter->mplContext);
      spin_unlock_irq(&adapter->txLock);

#ifdef LINUX_POWER_DEBUG
      MplWolCfg(adapter->mplContext, NS_TRUE, adapter->wolModes);
#endif //NSM_DEBUG

      // Put device in LOW_POWER mode - Ask MPL to set PME for us
      status = MplPowerSetState(adapter->mplContext, 
                                    MPL_POWER_STATE_LOW, NS_TRUE);
   }

   // Detach our device from active list
   netif_device_detach(netdev);
   rtnl_unlock();
   return 0;
}

NS_SINT
NsmResume(struct pci_dev *pdev)
{
   struct net_device *netdev = pci_get_drvdata(pdev);
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;
   MPL_STATUS status;
   NS_SINT ret;

   rtnl_lock();
   if (netif_device_present(netdev))
   {
      rtnl_unlock();
      return 0x0;
   }

   if (netif_running(netdev)) 
   {
      // Enable PCI device
      ret = pci_enable_device(pdev);

      // Put device in HIGH_POWER mode
      status = MplPowerSetState(adapter->mplContext, 
                                    MPL_POWER_STATE_HIGH, NS_TRUE);

      // Replenish MPL with Receive Buffers
      replenishTask((unsigned long)adapter);

      // Enable Interrupts
      MplInterruptEnable(adapter->mplContext);

      // Configure the Link - Done after Ints are enabled
      configLink(adapter);
   }

   // Attach our device to active list
   netif_device_attach(netdev);
   rtnl_unlock();
   return 0x0;
}
#endif

NS_VOID
NsmRemove(struct pci_dev *pdev)
{
   struct net_device *netdev;
   NSM_CONTEXT *adapter;

   NI(NsmRemove);

   netdev = pci_get_drvdata(pdev);

   // Adapter not found!
   if(netdev == NULL)
   {
      NSM_DBG(DBG_ERR,("Adapter Not Found! \n"));
      NO(NsmRemove);
      return;
   }

   // Get our private device struct
   adapter = (NSM_CONTEXT *) netdev->priv;

   // Unregister this Net device from the OS
   if (adapter->netdevReg == NS_TRUE)
   {
      unregister_netdev(netdev);
   }

   // MPL:UnInitialize the adapter
   if (adapter->mplContext != NULL) 
   {
      MplUnload(adapter->mplContext); 
   }

   // Free system resources
#ifdef NSM_IO_MAP
   if(adapter->regAddr != NULL) 
   {
      adapter->regAddr = NULL;
   }
#else
   if(adapter->regAddr != NULL) 
   {
      // First unmap the memory
      iounmap(adapter->regAddr);
      adapter->regAddr = NULL;
   }
#endif
   // Release the PCI address space
   pci_release_regions(pdev);

   // Free the Netdev struct and my NSM context struct
   kfree(netdev);

   // Decrement cards found
   NsmAdapterCnt--; 

   NO(NsmRemove);
   return;
}

NS_SINT
NsmOpen(struct net_device *netdev)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;

   NI(NsmOpen);

   // Register Interrupt Service Routine
   if (request_irq(netdev->irq, &NsmIsr, SA_SHIRQ, DP_driverName, adapter)) 
   {
      NSM_DBG(DBG_ERR,("Failed to Get IRQ \n"));

      NO(NsmOpen);
      return -EAGAIN;
   }

   // Initialize SpinLocks
   spin_lock_init(&adapter->txLock);
   spin_lock_init(&adapter->rxLock);
   spin_lock_init(&adapter->linkLock);

   // Get Mpl Capabilities
   MplGetCaps(adapter->mplContext, &adapter->mplCaps);

   // Normal and Monitor mode : Cfg Mac, Link, RxFrags
   // Load only mode : Open only
   if (adapter->opMode != MPL_MODE_LOAD_ONLY)
   {
      // Configure the MAC
      if (configMac(adapter) == NS_FALSE)
      {
         NSM_DBG(DBG_ERR,("Config Mac Failed \n"));
         free_irq(netdev->irq, adapter);

         NO(NsmOpen);
         return -EAGAIN;
      }

      // Initialize link monitor tasklet
      tasklet_init(&adapter->linkMonTask, linkTask, (unsigned long) adapter);

      //Task Offloads Enables
      // Rx Filter (Multicast and Unicast Filtering)
      MplTaskFilterCfg(adapter->mplContext, NS_TRUE);

      // Allocate and setup MPL_FRAGs to hold Receive Buffers 
      //  These are wrapper structures used to post new buffers to MPL rx
      if (setupRxFrags(adapter) == NS_FALSE)
      {
         NSM_DBG(DBG_ERR,("Frag Setup Failed \n"));
         free_irq(netdev->irq, adapter);

         NO(NsmOpen);
         return -ENOMEM;
      }
   }

#ifdef NSM_DIAG_MODE
   // Initialize Diag Module
   if (adapter->opMode != MPL_MODE_NORMAL)
   {
      if (NsmDiagInitialize(adapter) != NS_STATUS_SUCCESS)
      {
            NSM_DBG(DBG_ERR,("Diag Setup Failed \n"));

            // Free the MPL Pkt Fragments 
            freeRxFrags(adapter);

            free_irq(netdev->irq, adapter);

            NO(NsmOpen);
            return -ENOMEM;
      }
   }
#endif //NSM_DIAG_MODE

   // Open the adapter for operations
   if (adapter->opMode != MPL_MODE_LOAD_ONLY)
   {
      // Replenish MPL with Receive Buffers
      replenishTask((unsigned long)adapter);

      // Open the MPL Adapter
      MplOpen(adapter->mplContext);

      // Configure the Link - Done after Ints are enabled
      configLink(adapter);
   }
   else
   {
      // Open the MPL Adapter - No other setups required here
      // We just assume the link is up - the diag app would need
      //  to ensure this is the case before doing any tx/rx
      MplOpen(adapter->mplContext);
      adapter->linkStatus = MPL_LINK_STATUS_UP;
   }


   // If the Link is not UP, notify Net Stack
   if ((adapter->linkStatus != MPL_LINK_STATUS_UP) ||
                (adapter->opMode != MPL_MODE_NORMAL))
   {
      netif_stop_queue(netdev);
   }

   NO(NsmOpen);
   return 0x0;
}


NS_SINT
NsmXmit(struct sk_buff *skb, struct net_device *netdev)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;
   MPL_PKT mplPkt;
   MPL_PKT_FRAG mplFrag;
   MPL_STATUS status;

   NI(NsmXmit);
   
   // Test if the link is down
   if (adapter->linkStatus != MPL_LINK_STATUS_UP)
   {
      NSM_DBG(DBG_ERR,("Link Down \n"));
      netif_stop_queue(netdev);

      NO(NsmXmit);
      return -EAGAIN;
   }

   mplPkt.pNextPacket = NULL;
   mplPkt.fragCount = 0x01;

   // Set priority - FM: Map to higher level protocol?
   mplPkt.txOOB.pQueue = 1;

   // Prep MPL frag
   mplFrag.pNextFrag = NULL;
   mplFrag.fragSize  = skb->len;
   mplFrag.physAddr  = pci_map_single(adapter->pdev, 
                                       skb->data, skb->len, PCI_DMA_TODEVICE);
   mplFrag.pAddr     = skb->data;

   // Store in the physAddr of the skb in the private field
   *(NS_UINT *)skb->cb = mplFrag.physAddr;

   // Send Packet
   mplPkt.pFragHead = &mplFrag;
   mplPkt.packetSize  = skb->len;
   mplPkt.pNsmPrivate = skb; //Set Skb ptr as NSM private
   spin_lock_irq(&adapter->txLock);
   status = MplTransmit(adapter->mplContext, 0x01, &mplPkt);
   spin_unlock_irq(&adapter->txLock);

   // Check status
   if (status != NS_STATUS_SUCCESS)
   {
      if (mplPkt.packetStatus == NS_STATUS_RESOURCES)
      {
         // Stop the Net Stack
         netif_stop_queue(netdev);
         return -ENOMEM;
      }
      else
      {
         // Drop the pkt - Free the skb
         pci_unmap_single(adapter->pdev, mplFrag.physAddr, 
                              skb->len, PCI_DMA_TODEVICE);
         dev_kfree_skb(skb);
         adapter->netStats.tx_dropped++;
      }
   }

   NO(NsmXmit);
   return 0x0;
}

void
NsmSetMulti(struct net_device *netdev)
{   
   NSM_CONTEXT *adapter = netdev->priv;
   NS_UINT filterFlag, i;
   struct dev_mc_list *mcAddr;

   NI(NsmSetMulti);

   if (adapter->opMode != MPL_MODE_NORMAL)
   {
      return;
   }

   // Check if we need to go to Promiscuous Mode
   if (netdev->flags & IFF_PROMISC) 
   {
      filterFlag = MPL_RECEIVE_FILTER_PROMISCUOUS_MODE;
   }
   else 
   {
      // We always receive directed unicast and broadcast
      filterFlag = MPL_RECEIVE_FILTER_DIRECTED_UNICAST |
                     MPL_RECEIVE_FILTER_ACCEPTALL_BROADCAST;

      if (netdev->flags & IFF_ALLMULTI)
      {
         // All multicasts
         filterFlag |= MPL_RECEIVE_FILTER_ACCEPTALL_MCAST;
      }
      else
      {
         // Accept multicasts directed to our interest list
         filterFlag |= MPL_RECEIVE_FILTER_DIRECTED_MCAST;

         // Add multicast list to the device
         if(netdev->mc_count > 0x0)
         {
            // Clear the current MC list
            MplTaskFilterMcastClearList(adapter->mplContext);
 
            // Report the list to MPL
            mcAddr = netdev->mc_list;
            for (i = 0x0; i < netdev->mc_count; i++)
            {
               if (mcAddr != NULL)
               {
                  // Add the MC addr
                  MplTaskFilterMcastAddAddr(adapter->mplContext, 
                                              mcAddr->dmi_addr);

                  // Get the next MC addr
                  mcAddr = mcAddr->next;
               }
            }
         }
      }
   }
   
   MplReceiveSetFilter(adapter->mplContext, filterFlag);
   NO(NsmSetMulti);
   return;
}

struct net_device_stats *
NsmStats(struct net_device *netdev)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;

   // NI(NsmStats);

   //FM: Update Stats from MPL
   // NO(NsmStats);
   return &adapter->netStats;
}

NS_SINT
NsmSetMac(struct net_device *netdev, void *p)
{
   struct sockaddr *mac = p;
   NSM_CONTEXT *adapter = netdev->priv;

   NI(NsmSetMac);
   memcpy(netdev->dev_addr, mac->sa_data, netdev->addr_len);

   MplSetMacAddress(adapter->mplContext, netdev->dev_addr);

   NO(NsmSetMac);
   return 0x0;
}

NS_SINT
NsmIoctl(struct net_device *netdev, struct ifreq *rq, NS_SINT cmd)
{
   NS_SINT ret = 0x0;
   NSM_CONTEXT *adapter = netdev->priv;
   struct mii_ioctl_data *data = (struct mii_ioctl_data *)&rq->ifr_data;

   // NI(NsmIoctl);

   switch(cmd) 
   {
#ifdef NSM_DIAG_MODE
      // National's Integrity Tool
      case NS_APP_IOCTL_CODE: 
                          {
                             NS_UINT status;

                             // Handle the ioctl
                             status = NsmDiagIoctl(adapter, 
                                              (NS_VOID *) rq->ifr_data);
                             if (status != NS_STATUS_SUCCESS)
                             {
                                switch (status)
                                {
                                   case NS_STATUS_NOT_SUPPORTED: 
                                                          ret = -EOPNOTSUPP;
                                                          break;
                                   case NS_STATUS_INVALID_PARM: 
                                                          ret = -EINVAL;
                                                          break;
                                   case NS_STATUS_RESOURCES: 
                                                          ret = -ENOMEM;
                                                          break;
                                   default : 
                                                          ret = -EFAULT;
                                }
                             }
                             break;
                          }
#endif //NSM_DIAG_MODE

      // Other Linux Community Tools
      case SIOCETHTOOL    :
                             ret = -EOPNOTSUPP; //Soon to be supported
                             break; 

      case SIOCGMIIPHY    :
                             // Get the Phy Addr Index
                             data->phy_id = 
                                  MplPhyGetDeviceAddr(adapter->mplContext);
                             // Fall thru

      case SIOCGMIIREG    :
                             // Read MII register
                             data->val_out = 
                                   MplPhyMdioRead(adapter->mplContext,
                                                   data->phy_id & 0x1F,
                                                   data->reg_num & 0x1F);
                             break;

      case SIOCSMIIREG    :
                             if (!capable(CAP_NET_ADMIN))
                             {
                                ret = -EPERM;
                             }
                             else
                             {
                                // Write MII register
                                 MplPhyMdioWrite(adapter->mplContext,
                                                   data->phy_id & 0x1F,
                                                   data->reg_num & 0x1F,
                                                   data->val_in);
                             }
                             break;

      default             :    
                             adapter = NULL;
                             ret = -EPERM;
    }

   // NO(NsmIoctl);
   return ret;
}


NS_SINT
NsmChangeMtu(struct net_device *netdev, int new_mtu)
{
//FM: Need to validate with MPL CAPS
   NO(NsmChangeMtu);

   NO(NsmChangeMtu);
   return 0x0;
}

#ifdef L26
NS_SINT
#else
NS_VOID
#endif
NsmIsr(NS_SINT irq, void *data, struct pt_regs *regs)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *)data;
   NS_UINT fragsConsumed = 0x0;

   // Check if there are pending interrupts
   if (MplInterruptCheck(adapter->mplContext) == NS_STATUS_SUCCESS)
   {
      // Check if we have receive events
      if (MplInterruptCheckReceive(adapter->mplContext) == NS_STATUS_SUCCESS)
      {
         // Let MPL process this receive
         spin_lock(&adapter->rxLock);
         MplReceive(adapter->mplContext, 0x0, &fragsConsumed);
         spin_unlock(&adapter->rxLock);

         adapter->rxdEmpty += fragsConsumed;

         // We need to continously replenish the Rx engine with 
         //  new buffers - Check it is time to do so
         if (adapter->rxdEmpty >= (adapter->rxdCnt / NSM_DEFAULT_RX_REPL))
         {
            spin_lock(&adapter->rxLock);
            replenishTask((unsigned long)adapter);
            spin_unlock(&adapter->rxLock);

            // Rxds that are empty should be zero now!
            adapter->rxdEmpty = 0x0;
         }
      }

      // Check if we have transmit events
      if (MplInterruptCheckTransmit(adapter->mplContext) == NS_STATUS_SUCCESS)
      {
         spin_lock(&adapter->txLock);
         MplTransmitDone(adapter->mplContext, 0x0);
         spin_unlock(&adapter->txLock);
      }

      // Check if we have internal events
      if (MplInterruptCheckInternal(adapter->mplContext) == NS_STATUS_SUCCESS)
      {
         MplInterruptDoneInternal(adapter->mplContext, 0x0);
      }

      // Re-Enable Interrupts
      MplInterruptEnable(adapter->mplContext);

#ifdef L26
      return IRQ_HANDLED;
#else
      return; 
#endif
   }
   else
   {
#ifdef L26
      return IRQ_NONE;
#else
      return; 
#endif
   }
}

NS_SINT
NsmClose(struct net_device *netdev)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *) netdev->priv;

   NI(NsmClose);

   // Stop the Net Stack - Just in case
   netif_carrier_off(netdev);
   netif_stop_queue(netdev);

   // Disable Interrupts - Doing this for load-only mode too?
   MplInterruptDisable(adapter->mplContext);

   // Stop link monitoring
   if (adapter->opMode != MPL_MODE_LOAD_ONLY)
   {
      // Kill Link Monitoring Tasklet
      tasklet_kill(&adapter->linkMonTask);
   }

   // Reset Transmit Engine - To get back pending fragments
   spin_lock_irq(&adapter->txLock);
   MplTransmitReset(adapter->mplContext);
   spin_unlock_irq(&adapter->txLock);

   // Reset Receive Engine - To get back pending fragments
   spin_lock_irq(&adapter->rxLock);
   MplReceiveReset(adapter->mplContext);
   spin_unlock_irq(&adapter->rxLock);

#ifdef NSM_DIAG_MODE
   // UnInitialize Diag Module
   if (adapter->opMode != MPL_MODE_NORMAL)
   {
      NsmDiagUnInitialize(adapter);
   }
#endif //NSM_DIAG_MODE

   // Free the MPL Pkt Fragments 
   freeRxFrags(adapter);

   // Close MPL
   MplClose(adapter->mplContext);

   //Task Offloads Diables
   // Rx Filter (Multicast and Unicast Filtering)
   MplTaskFilterCfg(adapter->mplContext, NS_FALSE);

   // Release Interrupt
   free_irq(netdev->irq, adapter);

   NO(NsmClose);
   return 0x0;
}


// Local Functions

static NS_VOID 
readPciInfo(NSM_CONTEXT *adapter)
{
   struct pci_dev *pdev = adapter->pdev;
   
   NI(readPciInfo);

   pci_read_config_byte(pdev, PCI_REVISION_ID, &adapter->pciRevId);
   pci_read_config_word(pdev, PCI_SUBSYSTEM_VENDOR_ID,
                             &adapter->pciSubVendorId);
   pci_read_config_word(pdev, PCI_SUBSYSTEM_ID, &adapter->pciSubDeviceId);

   NO(readPciInfo);
   return;
}

static NS_VOID 
setDataFields(NSM_CONTEXT *adapter)
{
   struct net_device *netdev = adapter->netdev;

   NI(setDataFields);

   // The rx buffer size
   if (netdev->mtu <= ENET_MTU)
   {
      adapter->rxBufferLen = ENET_BUFFER_SIZE; 
   }
   else
   {
      adapter->rxBufferLen = netdev->mtu + 32; 
   }

#ifdef NSM_DIAG_MODE
   // Make Tx buffer length same as Rx
   adapter->txBufferLen = adapter->rxBufferLen; 
#endif
  
   // Note the MTU ..include the Eth Hdr size and CRC len also
   adapter->mtu = netdev->mtu + ENET_HEADER_SIZE + CRC_LENGTH;

   NO(setDataFields);
   return;
}

static NS_VOID 
readUserOptions(NSM_CONTEXT *adapter)
{
   NI(readUserOptions);

   // Transmit Descriptor Count
   if (TxDCnt[NsmAdapterCnt] != -1) 
   {
      adapter->txdCnt = TxDCnt[NsmAdapterCnt];
   }
   else
   {
      adapter->txdCnt = NSM_DEFAULT_TXD_CNT;
   }

   // Transmit Priority Queue
   if (TxQCnt[NsmAdapterCnt] != -1) 
   {
      adapter->txQCnt = TxQCnt[NsmAdapterCnt];
   }
   else
   {
      adapter->txQCnt = NSM_DEFAULT_TX_Q_CNT;
   }

   // Receive Descriptor Count
   if (RxDCnt[NsmAdapterCnt] != -1) 
   {
      adapter->rxdCnt = RxDCnt[NsmAdapterCnt];
   }
   else
   {
      adapter->rxdCnt = NSM_DEFAULT_RXD_CNT;
   }

   // Receive Priority Queue
   if (RxQCnt[NsmAdapterCnt] != -1) 
   {
      // This currently has to be = 0x01
      adapter->rxQCnt = NSM_DEFAULT_RX_Q_CNT;
   }
   else
   {
      adapter->rxQCnt = NSM_DEFAULT_RX_Q_CNT;
   }

   // AutoNeg
   if (LinkMode[NsmAdapterCnt] != -1) 
   {
      switch (LinkMode[NsmAdapterCnt]) 
      {
         case 0x0   :   adapter->linkMode = MPL_LINK_MODE_AUTO;
                        break;
         case 0x1   :   adapter->linkMode = MPL_LINK_MODE_FORCED;
                        break;
         default    :   adapter->linkMode = NSM_DEFAULT_LINK_MODE;
      }
   }
   else
   {
      adapter->linkMode = NSM_DEFAULT_LINK_MODE;
   }

   // Speed 
   if (LinkSpeed[NsmAdapterCnt] != -1) 
   {
      switch (LinkSpeed[NsmAdapterCnt]) 
      {
         case 0x0   :   adapter->linkSpeed = MPL_LINK_SPEED_HUNDREDMBPS;
                        break;
         case 0x1   :   adapter->linkSpeed = MPL_LINK_SPEED_TENMBPS;
                        break;
         default    :   adapter->linkSpeed = NSM_DEFAULT_LINK_SPEED;
      }
   }
   else
   {
      adapter->linkSpeed = NSM_DEFAULT_LINK_SPEED;
   }

   // Duplex
   if (LinkDuplex[NsmAdapterCnt] != -1) 
   {
      switch (LinkDuplex[NsmAdapterCnt]) 
      {
         case 0x0   :   adapter->linkDuplex = MPL_LINK_DUPLEX_FULL;
                        break;
         case 0x1   :   adapter->linkDuplex = MPL_LINK_DUPLEX_HALF;
                        break;
         default    :   adapter->linkDuplex = NSM_DEFAULT_LINK_DUPLEX;
      }
   }
   else
   {
      adapter->linkDuplex = NSM_DEFAULT_LINK_DUPLEX;
   }

   // Pause
   if (LinkPause[NsmAdapterCnt] != -1) 
   {
      switch (LinkPause[NsmAdapterCnt]) 
      {
         case 0x0   :   adapter->linkPause = MPL_LINK_PAUSE_RECEIVE;
                        break;
         case 0x1   :   adapter->linkPause = MPL_LINK_PAUSE_TRANSMIT;
                        break;
         case 0x2   :   adapter->linkPause = MPL_LINK_PAUSE_SYMMETRICAL;
                        break;
         case 0x3   :   adapter->linkPause = MPL_LINK_PAUSE_NONE;
                        break;
         default    :   adapter->linkPause = NSM_DEFAULT_LINK_PAUSE;
      }
   }
   else
   {
      adapter->linkPause = NSM_DEFAULT_LINK_PAUSE;
   }

   // Intr Holdoff
   if (IntTimeHold[NsmAdapterCnt] != -1) 
   {
      adapter->timerHold = IntTimeHold[NsmAdapterCnt];
   }
   else
   {
      adapter->timerHold = NSM_DEFAULT_INTR_TIMER_HOLD;
   }

   // Intr Holdoff
   if (IntTxHold[NsmAdapterCnt] != -1) 
   {
      adapter->txHold = IntTxHold[NsmAdapterCnt];
   }
   else
      adapter->txHold = NSM_DEFAULT_INTR_TX_HOLD;

   // Intr Holdoff
   if (IntRxHold[NsmAdapterCnt] != -1) 
   {
      adapter->rxHold = IntRxHold[NsmAdapterCnt];
   }
   else
      adapter->rxHold = NSM_DEFAULT_INTR_RX_HOLD;

   // Operation Mode
   if (OpMode[NsmAdapterCnt] != -1) 
   {
      switch (OpMode[NsmAdapterCnt]) 
      {
         case 0x0   :   adapter->opMode = MPL_MODE_NORMAL;
                        break;
         case 0x1   :   adapter->opMode = MPL_MODE_MONITOR;
                        break;
         case 0x2   :   adapter->opMode = MPL_MODE_LOAD_ONLY;
                        break;
         default    :   adapter->opMode = NSM_DEFAULT_MODE;
      }
   }
   else
   {
      adapter->opMode = NSM_DEFAULT_MODE;
   }

   // WoL Modes
   if (Wol[NsmAdapterCnt] != -1) 
   {
      adapter->wolModes = Wol[NsmAdapterCnt];
   }

#ifdef NSM_DIAG_MODE
   // Diag Sts Queue length (in terms of packets)
   if (DiagStsQueue[NsmAdapterCnt] != -1) 
   {
      adapter->diagStsQueueSize = DiagStsQueue[NsmAdapterCnt];
   }
   else
   {
      adapter->diagStsQueueSize = NSM_DIAG_DEFAULT_STS_QUEUE_SIZE;
   }
#else
   // When NOT in diag mode, force the mode to be Normal
   adapter->opMode = MPL_MODE_NORMAL;
#endif //NSM_DIAG_MODE

#ifdef NSM_DEBUG
   // NSM Debug Zones
   if (NsmDebugZones[NsmAdapterCnt] != -1) 
   {
      NsmDbgSettings = NsmDebugZones[NsmAdapterCnt];
   }

   // Mpl Debug Zones
   if (MplDebugZones[NsmAdapterCnt] != -1) 
   {
      MplPathZones = MplDebugZones[NsmAdapterCnt];
   }
#endif //NSM_DEBUG

   NO(readUserOptions);
   return;
}

static NS_BOOLEAN
configMac(NSM_CONTEXT *adapter)
{
   MPL_STATUS status = NS_STATUS_SUCCESS;
   MPL_TRANSMIT_CFG txCfg;
   MPL_RECEIVE_CFG  rxCfg;
   NS_UINT8 i;

   NI(configMac);

   // Transmit Engine Config

   // Make sure the priority queues requested is within what is supported
   if (adapter->txQCnt > adapter->mplCaps.txCaps.priorityQCnt)
   {
      adapter->txQCnt = NSM_DEFAULT_TX_Q_CNT;
   }

   txCfg.priorityQueueCnt = adapter->txQCnt;
   for (i = 0x0; i < adapter->txQCnt; i++)
   {
      txCfg.descCnt[i]   = adapter->txdCnt; //FM: Multiple Qs, diff txds?
   }

   // Maximum indications on Tx completions
   txCfg.maxIndications  = NSM_DEFAULT_TX_MAX_INDY;

   if (adapter->opMode == MPL_MODE_NORMAL)
   {
      // Transmit Engine - Not End User configurable Value (atleast initially)
      txCfg.fillThreshold   = NSM_DEFAULT_TX_FILL_THOLD;
      txCfg.drainThreshold  = NSM_DEFAULT_TX_DRAIN_THOLD;
      txCfg.maxDmaBurst     = NSM_DEFAULT_TX_MAX_DMA;

      txCfg.cfgFlags = MPL_TRANSMIT_CFG_FIFO |
                         MPL_TRANSMIT_CFG_QUEUES |
                           MPL_TRANSMIT_CFG_AUTO_PAD;
   }
   else
   {
      // In diag mode only set the Txd ring - No FIFO setup
      txCfg.cfgFlags = MPL_TRANSMIT_CFG_QUEUES;
   }

   // Let MPL do the setup
   status = MplTransmitCfg(adapter->mplContext, &txCfg);

   if (status != NS_STATUS_SUCCESS)
   {
      NSM_DBG(DBG_ERR,("Tx Config Failed \n"));
      NO(configMac);
      return NS_FALSE;
   }

   // Receive Engine - End User (driver load or UI) configurable
   rxCfg.priorityQueueCnt = adapter->rxQCnt;
   for (i = 0x0; i < adapter->rxQCnt; i++)
   {
      rxCfg.descCnt[i]   = adapter->rxdCnt; //FM: Multiple Qs, diff txds?
   }

   // Maximum indications on Rx completions
   rxCfg.maxIndications  = NSM_DEFAULT_RX_MAX_INDY;

   if (adapter->opMode == MPL_MODE_NORMAL)
   {
      // Receive Engine - Not End User configurable Value (atleast initially)
      rxCfg.drainThreshold  = NSM_DEFAULT_RX_DRAIN_THOLD;
      rxCfg.maxDmaBurst     = NSM_DEFAULT_RX_MAX_DMA;

      rxCfg.cfgFlags        = MPL_RECEIVE_CFG_FIFO |
                               MPL_RECEIVE_CFG_QUEUES;
   }
   else
   {
      rxCfg.cfgFlags        = MPL_RECEIVE_CFG_QUEUES;
   }

   // Let MPL do the setup
   status = MplReceiveCfg(adapter->mplContext, &rxCfg);
   if (status != NS_STATUS_SUCCESS)
   {
      NSM_DBG(DBG_ERR,("Rx Config Failed \n"));
      NO(configMac);
      return NS_FALSE;
   }

   // Configure Max MTU
   status = MplCfgMTU (adapter->mplContext, adapter->mtu);
   if (status != NS_STATUS_SUCCESS)
   {
      NSM_DBG(DBG_ERR,("MTU Config Failed \n"));
      NO(configMac);
      return NS_FALSE;
   }

   // Interrupt
   MplInterruptCfg(adapter->mplContext, 
                    adapter->timerHold, adapter->txHold, adapter->rxHold);

   NO(configMac);
   return NS_TRUE;
}

static NS_BOOLEAN
configLink(NSM_CONTEXT *adapter)
{
   MPL_STATUS status;
   MPL_LINK_CFG linkCfg;
   NS_BOOLEAN ret = NS_TRUE;

   NI(configLink);

   memset((void *)&linkCfg, 0x0, sizeof(linkCfg));

   // MPL Link APIs are non-reentrant
   spin_lock_irq(&adapter->linkLock);

   // Report Net stack that the link is down - since it is being set
   netif_carrier_off(adapter->netdev);

   // Link
   linkCfg.mode   = adapter->linkMode;
   linkCfg.speed  = adapter->linkSpeed;
   linkCfg.duplex = adapter->linkDuplex;

   // Pause configuration
   linkCfg.pauseType = adapter->linkPause;
   if (linkCfg.pauseType & MPL_LINK_PAUSE_TRANSMIT)
   {
      // For pause transmits specify the threshold
      linkCfg.pauseCtr    = NSM_DEFAULT_LINK_PAUSE_COUNTER;
      linkCfg.pauseRxDALO = NSM_DEFAULT_LINK_PAUSE_DALO; // Data FIFO Hi
      linkCfg.pauseRxDAHI = NSM_DEFAULT_LINK_PAUSE_DAHI; // Data FIFO Hi
      linkCfg.pauseRxSTLO = NSM_DEFAULT_LINK_PAUSE_STHI; // Status FIFO Hi
      linkCfg.pauseRxSTHI = NSM_DEFAULT_LINK_PAUSE_STHI; // Status FIFO Hi
   }

   // Tell MPL to configure the link
   status = MplLinkCfg(adapter->mplContext, &linkCfg);
   if (status == NS_STATUS_SUCCESS)
   {
       // The existing link matches our needs - Complete the Setup
       if (MplLinkUpComplete(adapter->mplContext) == NS_STATUS_SUCCESS)
       {
          NSM_DBG(DBG_TMP,("Link is UP \n"));

          adapter->linkStatus = MPL_LINK_STATUS_UP;

          // Indicate Upper layers to start Tx and Rx for this adapter
          // If previously reported a Link down - Report AOK now
          //  This is only in normal mode
          if (adapter->opMode == MPL_MODE_NORMAL)
          {
             if (!netif_carrier_ok(adapter->netdev))
             {
                netif_carrier_on(adapter->netdev);
                netif_start_queue(adapter->netdev);
             }    
          }
       }
       else
       {
           NSM_DBG(DBG_ERR,("Link Config Failed \n"));

           adapter->linkStatus = MPL_LINK_STATUS_DOWN;
           ret = NS_FALSE;
       }
   }
   else if (status == NS_STATUS_ASYNCH_COMPLETION)
        {
           NSM_DBG(DBG_TMP,("Link is being Pursued \n"));
           adapter->linkStatus = MPL_LINK_STATUS_ACTIVE;
        }
        else 
        {
           adapter->linkStatus = MPL_LINK_STATUS_DOWN;
           NSM_DBG(DBG_ERR,("Link Config Failed \n"));
           ret = NS_FALSE;
        }

   spin_unlock_irq(&adapter->linkLock);
   NO(configLink);
   return ret;
}

NS_BOOLEAN
setupRxFrags(NSM_CONTEXT *adapter)
{
   NS_UINT allocSize, i, nodes;
   MPL_PKT_FRAG *pFrag = NULL;
   NS_VOID *allocMem;

   NI(setupRxFrags);

   // Allocate memory for the frags
   // The one additional fragment will ensure no race condition
   //  between the consumer (Replenish Task) and Producer (RxDone)
   nodes = adapter->rxdCnt + 1;

#ifdef NSM_DIAG_MODE
   // Additional nodes to handle rx completion queuing
   nodes += adapter->diagStsQueueSize;
#endif //NSM_DIAG_MODE

   allocSize = sizeof(MPL_PKT_FRAG) * nodes;
   allocMem  = kmalloc(allocSize, GFP_KERNEL);

   if (allocMem == NULL)
   {
      NSM_DBG(DBG_ERR,("Failed to Alloc mem to rx frags \n"));
      NO(setupRxFrags);
      return NS_FALSE;
   }
   else
   {
      // Zero up the memory
      memset((void *)(allocMem), 0x0, allocSize);

      pFrag = (MPL_PKT_FRAG *) allocMem;

      // Create linked list
      for (i = 0x0; i < (nodes - 1); i++)
      {
         // Set link
         pFrag->pNextFrag = (pFrag + 1);
         pFrag++;
      }
 
      // Last node has next NULL
      pFrag->pNextFrag = NULL;
 
      // Note Head and Tail of this frag list
      adapter->rxFrags     = allocMem; 
      adapter->rxFragsHead = (MPL_PKT_FRAG *)allocMem; 
      adapter->rxFragsTail = pFrag;
   }

   NO(setupRxFrags);
   return NS_TRUE;
}

NS_VOID
freeRxFrags(NSM_CONTEXT *adapter)
{
   struct pci_dev *pdev = adapter->pdev;
   MPL_PKT_FRAG *pFrag = NULL;
   struct sk_buff* skb;

   NI(freeRxFrags);

   // Free attached buffers
   pFrag = adapter->rxFragsHead;
   while (pFrag)
   {
      skb = (struct sk_buff *)pFrag->pNsmPrivate;
      if (skb != NULL)
      {
         // UnMap and Free the sk buffs
         pci_unmap_single(pdev, (dma_addr_t)pFrag->physAddr,
                          adapter->rxBufferLen, 
                          PCI_DMA_FROMDEVICE);
         dev_kfree_skb(skb);
         pFrag->pNsmPrivate = NULL;
      }

      // Move to the next fragment
      pFrag = pFrag->pNextFrag;
   }

   // Free Frag memory
   if (adapter->rxFrags)
   {
      kfree(adapter->rxFrags);
      adapter->rxFrags = NULL;
      adapter->rxFragsHead = NULL;
   }

   NO(freeRxFrags);
   return;
}

NS_VOID
replenishTask(unsigned long data)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *)data;
   MPL_PKT_FRAG *pFrag = NULL, *pFragsToMpl = NULL;
   struct sk_buff* skb;
   NS_UINT i, rxdsEmpty, bufCnt = 0x0;

   // Note: This function needs to be called from Intr context
   //       This function is non-reenterant
   NI(replenishTask);

   MplReceiveGetEmptyDesc(adapter->mplContext, adapter->rxQCnt , &rxdsEmpty);

   if (rxdsEmpty == 0x0)
   {
      NO(replenishTask);
      return;
   }

#ifdef NSM_DIAG_MODE
   // While in diag mode, there is a chance that we might run out of
   //  fragments in the free list, while waiting for the app to come
   //  get the packets from us - If we enter this situation then force
   //  a packet from the Rx done list to free some frags
   if (adapter->rxFragsHead == adapter->rxFragsTail)
   {
      NsmDiagProcessDoneList(adapter, 0x1, NS_TRUE, 
                              &adapter->diagRxDoneList, &adapter->rxFragsTail);
   }
#endif //NSM_DIAG_MODE

   // Allocate the required number of buffers
   pFrag = adapter->rxFragsHead;
   for (i = 0x0; i < rxdsEmpty; i++)
   {
#ifdef NSM_DIAG_MODE
      if ( pFrag == adapter->rxFragsTail)
      {
         // Better luck next time
         break;
      }
#endif //NSM_DIAG_MODE

      // Check if this frag need a new SKB
      if (pFrag->pNsmPrivate == NULL)
      {
         // Allocate Skb
         skb = dev_alloc_skb(adapter->rxBufferLen);
         if (skb == NULL)
         {
            NSM_DBG(DBG_ERR,("SKB alloc in Repl Failed! \n"));

            // Better luck next time
            break;
         }
         else
         { 
            // Mark as being used by this device.
            skb->dev = adapter->netdev;
         }

         // Map the Skb buffer
         pFrag->physAddr  = pci_map_single(adapter->pdev, 
                                skb->tail, adapter->rxBufferLen, 
                                PCI_DMA_FROMDEVICE);

         // Set the Fragment
         pFrag->pAddr = skb->data;
         pFrag->pNsmPrivate = skb;
      }

      // Set the size of the buffer
      pFrag->fragSize = adapter->rxBufferLen;

      // Get the next frag
      pFrag = pFrag->pNextFrag;

      bufCnt++;
   }

   // Check if we managed to allocate new buffers
   if (bufCnt != 0x0)
   {
      pFragsToMpl = adapter->rxFragsHead;

      // Report new buffers to MPL
      MplReceiveReplenish(adapter->mplContext, bufCnt, &pFragsToMpl);

      // Check if MPL returned back some buffers
      if (pFragsToMpl != adapter->rxFragsHead)
      {
         // Move Head to the returned list
         adapter->rxFragsHead = pFragsToMpl;
      }
      else
      {
         // Move Head to the next frag on the cache
         adapter->rxFragsHead = pFrag;
      }
   }

   NO(replenishTask);
   return;
}

static NS_VOID
linkTask(unsigned long data)
{
   NSM_CONTEXT *adapter = (NSM_CONTEXT *)data;
   MPL_LINK_STATUS linkStatus;

   NI(linkTask);

   // MPL Link APIs are non-reentrant
   spin_lock_irq(&adapter->linkLock);

   // Get the current Link State
   linkStatus = MplLinkProcessChange(adapter->mplContext);

   switch (linkStatus) 
   {
      case MPL_LINK_STATUS_UP:

          // Let MPL finish up the Linking process
          if (MplLinkUpComplete(adapter->mplContext) == NS_STATUS_SUCCESS)
          {
             NSM_DBG(DBG_TMP,("Link is UP \n"));
             adapter->linkStatus = MPL_LINK_STATUS_UP;

             // If previously reported a Link down - Report AOK now
             //  This is only in normal mode
             if (adapter->opMode == MPL_MODE_NORMAL)
             {
                if (!netif_carrier_ok(adapter->netdev))
                {
                   netif_carrier_on(adapter->netdev);
                   netif_start_queue(adapter->netdev);
                }    
             }
          }

          break;

      case MPL_LINK_STATUS_ACTIVE:

          NSM_DBG(DBG_TMP,("Link is Actively Pursued \n"));
          adapter->linkStatus = MPL_LINK_STATUS_ACTIVE;

          // Reschedule this task after a brief delay
          mdelay(5); //FM: Eval
          tasklet_schedule(&adapter->linkMonTask);
          break;

      case MPL_LINK_STATUS_DOWN:

          NSM_DBG(DBG_TMP,("Link is Down \n"));
          adapter->linkStatus = MPL_LINK_STATUS_DOWN;

          // If previously reported a Link Up- Report down now
          if (netif_carrier_ok(adapter->netdev))
          {
             netif_carrier_off(adapter->netdev);
             netif_stop_queue(adapter->netdev);
          } 
          break;

    default:
          break;
    }

   spin_unlock_irq(&adapter->linkLock);

   NO(linkTask);
   return;
}
