/***********************************************************************
*									
* EEPROM.C                                                     
*									
* This is a collection of routines to interface to the EEPROM
*  for the MacPhyter Chip
*
***********************************************************************/

// Include files

#include <dos.h>
#include <time.h>
#include <sys\timeb.h>
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <time.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "PCIBIOS.H"

// Function prototypes

#include "FUNCTION.H"

UINT16 eepromDefaultValue[11] = {
		0xD008,		// 0000h, Subsystem Vendor ID
		0x0400,		// 0001h, Subsystem Device ID
		0x2CD0,		// 0002h, Minimum Grant, Maximum Latency
		0xCF82,		// 0003h, Configuration/Power Management Stuff
		0x0000,		// 0004h, SecureOn Password
		0x0000,		// 0005h, SecureOn Password
		0x0000,		// 0006h, SecureOn Password
		0x0000,		// 0007h, Ethernet ID
		0x0000,		// 0008h, Ethernet ID
		0x0000,		// 0009h, Ethernet ID
		0xA098		// 000Ah, WOL, Receive Filter/Match
};

UINT32 OUID; // Organizationally Unique Identifier
UINT32 startMACAddr;
UINT32 endMACAddr;
UINT32 currentMACAddr;

// Globals
extern unsigned short RegBase;
extern unsigned short batchMode;

extern UINT16 flip_16 (UINT16 us);

/************************************************
*	Function:   DisplayEEPROM
*	Purpose:	display EEPROM contents
*	Returns:    void
*************************************************/
void DisplayEEPROM()
{
	int i=0;
	WORD value;

	for( i=0; i<0xC; i++ )
	{
		ReadEEWord( RegBase + MEAR, i, &value );

		printf( "\n%01X= %04X  ", i, value );
	}
            
	printf( "\n\n" );
}

/************************************************
*	Function:   ReadMACAddress
*	Purpose:    load readable form of the MAC address
*	Returns:    void
*************************************************/
void ReadMACAddress(  UINT16 *nicAddress )
{
	UINT32 i;
	UINT16 mask;
	UINT16 word1 = 0;
	UINT16 word2 = 0;
	UINT16 word3 = 0;
	UINT16 word4 = 0;

    //
    // Read 16 bit words 6 - 9 from the EEProm.  They contain the hardwares MAC
    // address in a rather cryptic format.
    //
    ReadEEWord( RegBase + MEAR, 0x06, &word1 );
    ReadEEWord( RegBase + MEAR, 0x07, &word2 );
    ReadEEWord( RegBase + MEAR, 0x08, &word3 );
    ReadEEWord( RegBase + MEAR, 0x09, &word4 );

    //
    // Decode the cryptic format into what we can use a word at a time.
    //
    nicAddress[ 0 ] = word1 & 1;
    nicAddress[ 1 ] = word2 & 1;
    nicAddress[ 2 ] = word3 & 1;

    i = 15;
    mask = 0x2;
    while ( i-- )
	{
		if ( word2 & 0x8000 )
		{
			nicAddress[ 0 ] |= mask;
		}
		word2 = word2 << 1;
		mask = mask << 1;
    }

    i = 15;
    mask = 0x2;
	while ( i-- )
	{
		if ( word3 & 0x8000 )
		{
			nicAddress[ 1 ] |= mask;
		}
		word3 = word3 << 1;
		mask = mask << 1;
    }

    i = 15;
    mask = 0x2;
    while ( i-- )
	{
		if ( word4 & 0x8000 )
		{
			nicAddress[ 2 ] |= mask;
		}
		word4 = word4 << 1;
		mask = mask << 1;
    }
 }
 
/************************************************
*	Function:	WriteMACAddress
*	Purpose:	Write readable form of the MAC address to EEPROM
*	Returns:	1 if FAIL, 0 if Succees
*************************************************/
int writeMacAddress( unsigned *nicAddress )
{
	UINT32      mask;
	UINT32      l;
	int         nbits = 9;
	unsigned    value, sixOff, nineOff, checkSum = 0, word1, word2, word3;
	unsigned    sword[4];

    ReadEEWord( RegBase + MEAR, 0x06, &sixOff );
	sixOff &= ~1;

    ReadEEWord( RegBase + MEAR, 0x09, &nineOff );
	nineOff &= 1;

    word1 = nicAddress[ 0 ];
    word2 = nicAddress[ 1 ];
    word3 = nicAddress[ 2 ];
    
    writeEeEnable();
	
	value = sixOff;
	if( word1 & 1 ) value |= 1;
    sword[0] = value;
    WriteEEWord( RegBase + MEAR, 0x06, value );
    
	value = 0;
    l = 15;
    mask = 0x2;
	while( l-- )
	{
		if( word1 & 0x8000 ) value |= mask;
		mask <<= 1;
		word1 <<= 1;
	}
    
    if( word2 & 1 ) value |= 1;
    sword[1] = value;
    WriteEEWord( RegBase + MEAR, 0x07, value );

	value = 0;
    l = 15;
    mask = 0x2;
	while( l-- )
	{
		if( word2 & 0x8000 ) value |= mask;
		mask <<= 1;
		word2 <<= 1;
	}
    
    if( word3 & 1 ) value |= 1;
    sword[2] = value;
    WriteEEWord( RegBase + MEAR, 0x08, value );

    value = nineOff;
    l = 15;
    mask = 0x2;
	while( l-- )
	{
		if( word3 & 0x8000 ) value |= mask;
		mask <<= 1;
		word3 <<= 1;
	}
    
    sword[3] = value;
    WriteEEWord( RegBase + MEAR, 0x09, value );
    
    writeEeDisable();
	
    for( l=0; l<4; l++ )
    {
    	ReadEEWord( RegBase + MEAR, 6+l, &value );
		if( value != sword[l] )
		{
			printf( "\nEEPROM programming failed!  Enter Alternate MAC address.\n" );
			break;
		}
    }
    
    if (doEeCheckSum())
	{
		return 1;
	}

	if (batchMode)
	{
		return writeMacFile();
	}

	return 0;
}

int writeID( int ssId, BYTE venDev )
{
	int idOffset = 0;

	// Calculate the offset:
	if (venDev == 'D') idOffset = 1;

    writeEeEnable();
    WriteEEWord( RegBase + MEAR, idOffset, ssId );
    writeEeDisable();

	return doEeCheckSum();
}

/************************************************
*	Function:   WriteEEWord
*	Purpose:    write a 16-bit value into the specified eeprom location
*	Returns:    void
*************************************************/
void WriteEEWord( unsigned short pMear, UINT32 wordOffset, UINT16 value )
{
	UINT32 mask;
	UINT32 l;
	int	nbits = 9;
	UINT32 invalue = (UINT32) value;

    outpd( RegBase + MEAR, 0 );           /* clock out CS low */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );

    invalue = ((UINT32)( 0x0140|wordOffset) << 16 ) | invalue;
    
    nbits = 25;

    mask = ((UINT32)1) << (nbits-1);

    while (nbits--)
	{
        /* assert chip select, and setup data in */
        l = ( invalue & mask ) ? EECS | EEDI : EECS;

        outpd( RegBase + MEAR, l );
	    mdelay( 5 );
        
        outpd( RegBase + MEAR, l | EECLK );
        mdelay( 5 );
    
        mask >>= 1;
    }

    // IOW32( mear, 0 );           /* terminate write  */
    outpd( RegBase + MEAR, 0 );               // terminate operation
    mdelay( 5 );
 
    /* wait for operation to complete */
    outpd( RegBase + MEAR, EECS );        /* assert CS */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECS | EECLK );
    mdelay( 5 );
 
    for (l = 0; l < 10; l++)
	{
        mdelay( 5 );
        if( inpd( RegBase + MEAR ) & EEDO )
            break;
    }
    
    outpd( RegBase + MEAR, 0 );           /* clock out CS low */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );
}

/************************************************
*	Function:   checkSumCheck
*	Purpose:    Use the EEBIST bit on MacPhyter to
*				validate the EEPROM checksum
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int checkSumCheck()
{
	unsigned long pciTestReg = EEBIST_EN;
	
	outpd( RegBase + PCITEST_CNTRL, EEBIST_EN );
	
	while (pciTestReg & EEBIST_EN)
	{
		pciTestReg = inpd( RegBase + PCITEST_CNTRL );
	}
	
	if (inpd( RegBase + PCITEST_CNTRL ) & EEBIST_FAIL)
	{
		printf( "\n!!WARNING:  Checksum FAILED!\n" );
		return( 1 );
	}
	else
	{
		printf( "\nEEBIST Test Passes!\n" );
	}
		
	return( 0 );
}

/************************************************
*	Function:   ReadEEWord
*	Purpose:    Read 16-bit value from the specified location
*	Returns:    void
*************************************************/
void ReadEEWord( unsigned short pMear, UINT32 offset, UINT16* pEeData )
{
	UINT16 value;

    outpd( RegBase + MEAR, 0 );
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );       // clock out no chipselect
    mdelay( 5 );
 
    eeput( (UINT32)(EEread|offset), 9 );
    
    value = eeget( 16 );
    
    outpd( RegBase + MEAR, 0 );           // terminate read 
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );       // clock out no chipselect
    mdelay( 5 );
 
    *pEeData = ( UINT16 )value;

    outpd( RegBase + MEAR, 0 );           // terminate read 
    mdelay( 5 );
}

/************************************************
*	Function:   EEPUT
*	Purpose:    Clock OUT data to EEPROM
*	Returns:    void
*************************************************/
void eeput( UINT32 value, int nbits )
{
	UINT32 mask = ((UINT32)1) << (nbits-1);
	UINT32 l;

    while( nbits-- )
	{
        // assert chip select, and setup data in
        l = ( value & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        mdelay( 5 );
 
        outpd( RegBase + MEAR, l | EECLK );
        mdelay( 5 );
 
        mask >>= 1;
    }
}

/************************************************
*	Function:   EEGET
*	Purpose:    Clock IN data from EEPROM
*	Returns:    void
*************************************************/
UINT16 eeget( int nbits )
{
	UINT16 mask = 1 << (nbits-1);
	UINT16 value = 0;

    while( nbits-- )
	{
        // assert chip select, and clock in data
        outpd( RegBase + MEAR, EECS );
        mdelay( 5 );
 
        outpd( RegBase + MEAR, EECS | EECLK );
	    mdelay( 5 );
 
        if( inpd( RegBase + MEAR ) & EEDO )
                value |= mask;
	    mdelay( 5 );
    
        mask >>= 1;
    }
    
    return( value );
}

/************************************************
*	Function:   WriteEeEnable
*	Purpose:    ENABLE EEPROM Write Mode
*	Returns:    void
*************************************************/
void writeEeEnable()
{
	UINT32 mask;
	UINT32 l;
	int	nbits = 9;
    //
    // Start the sequence with Chip Select 0 for a 4 microsecond cycle.
    //
    outpd( RegBase + MEAR, 0 );
    mdelay( 5 );

    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );

    mask = ((UINT32)1) << (nbits-1);

    while( nbits-- )
	{
        /* assert chip select, and setup data in */
        l = ( 0x0130 & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        outpd( RegBase + MEAR, l | EECLK );
        mask >>= 1;
    }

    outpd( RegBase + MEAR, 0 );               // terminate operation
    mdelay( 5 );

    //IOW32( mear, 0 );           /* clock out CS low */
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );
}

/************************************************
*	Function:   WriteEEDisable
*	Purpose:    Disable EEPROM write Mode
*	Returns:    void
*************************************************/
void writeEeDisable()
{
	UINT32 mask;
	UINT32 l;
	int	nbits = 9;

    //ed->eeput( (uint32)EEwriteDisable, 9 );
    nbits = 9;
    mask = ((UINT32)1) << (nbits-1);

    while( nbits-- )
	{
        /* assert chip select, and setup data in */
        l = ( 0x0100 & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        outpd( RegBase + MEAR, l | EECLK );
        mask >>= 1;
    }

    outpd( RegBase + MEAR, 0 );               // terminate operation
}

/************************************************
*	Function:   doEeCheckSum()
*	Purpose:    Read EEPROM contents, calculate checksum,
*				write it, verify the write and validate it.
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int doEeCheckSum()
{
	int l;
	WORD value, checkSum;
	unsigned char csLow, csHigh, csSum, csVals[ 11 ];

    // Calculate Checksums:
    for( l=0; l<11; l++ )
    {
    	ReadEEWord( RegBase + MEAR, l, &value );
    	    
    	csLow = value & 0xff;
		value >>= 8;
    	csHigh = value & 0xff;
    	
    	csVals[ l ] = csLow + csHigh;
    }
    
    // Calculate Checksums:
    csSum =  0;
    for( l=0; l<11; l++ )
	{
		csSum += csVals[ l ];
	}
	
    csSum += 0x55;
    checkSum = (~csSum) + 1;
    checkSum = ( checkSum << 8 ) + 0x55;

    writeEeEnable();
    WriteEEWord( RegBase + MEAR, 0x0B, checkSum );
    writeEeDisable();
	
    ReadEEWord( RegBase + MEAR, 0xB, &value );
    if (value != checkSum) 
	{
		printf( "\n!!WARNING Checksum not programmed correctly!\n" );
		printf( "\n  checkSUM: %04X  actual: %04X", checkSum, value );
	}
        
    if (checkSumCheck()) // Invalid checksum
	{
		return 1;
	}

	outpd( RegBase + PCITEST_CNTRL, EELOAD_EN ); // Reload configuration information from EEPROM
	delay( 1 );

	return 0;
}

/*****************************************************
*	Function:   WriteEeDefaults:
*	Purpose:    Initialize the EEPROM without touching the 
*				MAC address.
*
*	Returns:    1 if FAIL, 0 if Succees
*       
*       The Following was taken from EPMON Programming script:
*       
*       eeprom  0       d008            # subsystem vendor
*       eeprom  1       0400            # subsys id
*       eeprom  2       2cd0            # min gnt, max lat
*       eeprom  3       cf82            # config stuff
*       eeprom  4       0000            # secure on pw stuff
*       eeprom  5       0000            # secure on pw stuff
*       eeprom  6       0000            # LSb is part of Ethernet ID 
*       eeprom  7       2001            # Ethernet ID
*       eeprom  8       d1a1            # Ethernet ID
*       eeprom  9       $Loc9           # Ethernet ID
*       eeprom  a       a098
*       eeprom  b       $value          # checksum
********************************************************************/
int writeEeDefaults()
{
	FILE *stream;
	char buf[120];
	UINT16 eepromValue[11];
	UINT32 i;
	UINT16 value;
	unsigned char inputString[25];
	int nicAddress[3];

	UINT16 Addressbit;

	// Read EEPROM values from file if the file exists, otherwise set as
	// default values
    if ((stream = fopen("EEPROM.TXT", "rt")) != NULL)
	{
		for (i = 0; i <= 0xA; i++)
		{
			fgets(buf, sizeof(buf), stream);
			sscanf(buf, "%04X", &eepromValue[i]);
		}
		fclose(stream);
	}
	else
	{
		for (i = 0; i <= 0xA; i++)
		{
			eepromValue[i] = eepromDefaultValue[i];
		}
	}

	// Read subsystem ID's from file if the file exists, and convert the ID's
	// in readable form to the format that DP83815/816 uses
    if ((stream = fopen("SUBSYSID.TXT", "rt")) != NULL)
	{
		for (i = 0; i <= 1; i++)
		{
			fgets(buf, sizeof(buf), stream);
			sscanf(buf, "%04X", &value);
			eepromValue[i] = flip_16(value);
		}
		fclose(stream);
	}

    writeEeEnable();

	// Write into EEPROM
	for (i = 0; i <= 0xA; i++)
	{
		if (i <=5 || i == 0xA)
		{
			WriteEEWord( RegBase + MEAR, i, eepromValue[i] );
		}
	}

    // Save the least significant bit for the MAC address:
    ReadEEWord( RegBase + MEAR, 0x6, &Addressbit );
    Addressbit &= 1;
    WriteEEWord( RegBase + MEAR, 0x6, Addressbit );

    // Clear the least significant bit for the MAC address:
    ReadEEWord( RegBase + MEAR, 0x9, &Addressbit );
    Addressbit &= 0xfffe;
    WriteEEWord( RegBase + MEAR, 0x9, Addressbit );

    writeEeDisable();
   
	// Check if the EEPROM values are written correctly
	for (i = 0; i <= 0xA; i++)
	{
		if (i <= 5 || i == 0xA)
		{
			ReadEEWord( RegBase + MEAR, i, &value );
			if (value != eepromValue[i])
			{
				printf("EEPROM values are not written correctly!\n");
				return 1;
			}
		}
	}
    
	// Write MAC address also if it is in batch mode and the MAC address
	// file exists
	if (batchMode)
	{
		if (readMacFile(inputString))
		{
			return 1;
		}
		nicAddress[0] = (UINT16)*( inputString+2 );
		nicAddress[0] = (nicAddress[0] << 8) + (UINT16)*( inputString );
		nicAddress[1] = (UINT16)*( inputString+6 );
		nicAddress[1] = (nicAddress[1] << 8) + (UINT16)*( inputString+4 );
		nicAddress[2] = (UINT16)*( inputString+10 );
		nicAddress[2] = (nicAddress[2] << 8) + (UINT16)*( inputString+8 );
		if (writeMacAddress(nicAddress))
		{
			return 1;
		}

		// Call findNIC routine again to display the new address
		if (findNIC(TRUE))
		{
			return 1;
		}

		if (currentMACAddr == endMACAddr)
		{
			return 1;
		}

		return 0;
	}
	else
	{
		return doEeCheckSum();
	}
}

/************************************************
*	Function:   readMacFile
*	Purpose:	Opens up a file, retrieves MacAddress,
*				increments it and returns the new number.
*	Returns:    1 = FAIL; 0 = SUCCESS; loads the input;
*************************************************/
int readMacFile( unsigned char *inputString )
{	
	FILE *stream;
	char buf[120];

	// Open up MACADDR.TXT file in the cwd
    if ((stream = fopen("MACADDR.TXT", "rt")) != NULL)
	{
		fgets(buf, sizeof(buf), stream);
		sscanf(buf, "%06lX%06lX", &OUID, &startMACAddr);
		fgets(buf, sizeof(buf), stream);
		sscanf(buf, "%06lX%06lX", &OUID, &endMACAddr);
		fgets(buf, sizeof(buf), stream);
		sscanf(buf, "%06lX%06lX", &OUID, &currentMACAddr);
		fclose(stream);
	}
	else
	{
	    printf("\nUnable to open up MACADDR.TXT in current working directory.");
	    return 1;
	}

	// Check if the MAC address is valid
	if (((OUID == 0x000000) && (currentMACAddr == 0x000000))
		|| ((OUID == 0xFFFFFF) && (currentMACAddr == 0xFFFFFF)))
	{
	    printf("\nInvalid MAC address: %06lX%06lX", OUID, currentMACAddr);
		return 1;
	}
	else if ((currentMACAddr < startMACAddr) || (currentMACAddr > endMACAddr))
	{
	    printf("\nMAC address is out of the range!");
		return 1;
	}

   	sscanf(buf, "%02X%02X%02X%02X%02X%02X",
                inputString,
                (inputString+2),
                (inputString+4),
                (inputString+6),
                (inputString+8),
                (inputString+10)
                );

	return 0;
}


/************************************************
*	Function:   writeMacFile
*	Purpose:	Opens up a file, writes the new number.
*	Returns:    1 = FAIL; 0 = SUCCESS; loads the input;
*************************************************/
int writeMacFile(void)
{
	FILE *stream;

	// Stop program if current MAC address reachs the end MAC address
	if (currentMACAddr < endMACAddr)
	{
		currentMACAddr++;

		// Open up MACADDR.TXT file in the cwd
		if ((stream = fopen("MACADDR.TXT", "wt")) != NULL)
		{
			fprintf(stream, "%06lX%06lX\t\t// Start MAC Address\n", OUID, startMACAddr);
			fprintf(stream, "%06lX%06lX\t\t// End MAC Address\n", OUID, endMACAddr);
			fprintf(stream, "%06lX%06lX\t\t// Current MAC Address\n", OUID, currentMACAddr);
			fclose(stream);
		}
		else
		{
			printf("\nUnable to open up MACADDR.TXT in current working directory.");
			return 1;
		}
	}
	else
	{
	    printf("\nEnd MAC address is reached.\n");
	}
	 
	return 0;
}